<?php

namespace WPEP\Admin;

use WPEP\Controller as WPEP_Controller;

class Controller {

  /**
   * @var Coordinator\AddOnsSection
   */
  public $addOnsSection;
  /**
   * @var Coordinator\Assets
   */
  public $assets;
  /**
   * @var Coordinator\DataSystem
   */
  public $dataSystem;
  /**
   * @var Coordinator\Embed
   */
  public $embed;
  /**
   * @var Coordinator\PlatformHealth
   */
  public $platformHealth;
  /**
   * @var Coordinator\Discussion
   */
  public $discussion;
  /**
   * @var Coordinator\Post
   */
  public $post;
  /**
   * @var Coordinator\License
   */
  public $license;
  /**
   * @var Coordinator\Menu
   */
  public $menu;
  /**
   * @var Coordinator\FieldLibrary
   */
  public $fieldLibrary;
  /**
   * @var Coordinator\Migration
   */
  public $migration;
  /**
   * @var Coordinator\ShortCodeBuilder
   */
  public $shortCodeBuilder;
  /**
   * @var Coordinator\Statistics
   */
  public $statistics;
  /**
   * @var Coordinator\Settings
   */
  public $settings;
  /**
   * @var Coordinator\SetupProcess
   */
  public $setupProcess;
  /**
   * @var Coordinator\UserEdit
   */
  public $userEdit;
  /**
   * @var Coordinator\LearningCenter
   */
  public $learningCenter;

  /**
   * @return Controller
   */
  public function __construct() {
    $this->migration     = new Coordinator\Migration();

    return $this;
  }

  public function dispatch() {
    if( !is_admin() )
      return false;

    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\Admin\Controller->dispatch() - start' );

    $this->_define();
    $this->_setup();

    add_action( "init", [ $this, '_setup_action_init' ], 10 );
    add_action( "admin_init", [ $this, '_setup_admin_action_init' ], 10 );
    add_action( "admin_menu", [ $this, '_setup_action_admin_menu' ], 10 );

    add_filter( 'plugin_action_links_' . plugin_basename( WPEP_BASE_FILE_PATH ), [ $this, 'action_links' ] );
    add_filter( 'user_row_actions', [ $this, 'user_row_actions' ], 10, 2 );

    add_action( 'create_course_category', [ $this, '_term_created' ] );

    if( wpep_get_setting_flag( 'configuration-setup-rules-flushed' ) == false )
      add_action( 'init', [ $this, '_setup_rules_flush_action_init' ], 7 );

    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\Admin\Controller->dispatch() - end' );

    return true;
  }

  private function _define() {
    $this->addOnsSection = new Coordinator\AddOnsSection();
    $this->assets        = new Coordinator\Assets();
    $this->dataSystem    = new Coordinator\DataSystem();
    $this->discussion    = new Coordinator\Discussion();
    $this->embed         = new Coordinator\Embed();
    $this->platformHealth= new Coordinator\PlatformHealth();
    $this->post          = new Coordinator\Post();
    $this->license       = new Coordinator\License();
    $this->menu          = new Coordinator\Menu();
    $this->fieldLibrary  = new Coordinator\FieldLibrary();
    $this->settings      = new Coordinator\Settings();
    $this->shortCodeBuilder  = new Coordinator\ShortCodeBuilder();
    $this->statistics        = new Coordinator\Statistics();
    $this->setupProcess      = new Coordinator\SetupProcess();
    $this->userEdit          = new Coordinator\UserEdit();
    $this->learningCenter    = new Coordinator\LearningCenter();
  }

  private function _setup() {
    $this->addOnsSection->setup();
    $this->assets->setup();
    $this->dataSystem->setup();
    $this->embed->setup();
    $this->platformHealth->setup();
    $this->post->setup();
    $this->license->setup();
    $this->fieldLibrary->setup();
    $this->settings->setup();
    $this->shortCodeBuilder->setup();
    $this->userEdit->setup();
  }

  public function _setup_action_init() {
    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\Admin\Controller->_setup_action_init() - start' );

    $this->migration->_init();
    $this->post->_init();
    $this->learningCenter->_init();
    $this->platformHealth->_init();
    $this->statistics->_init();

    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\Admin\Controller->_setup_action_init() - end' );
  }

  public function _setup_admin_action_init() {
    if( function_exists( 'wp_add_privacy_policy_content' ) ) {
      wp_add_privacy_policy_content( 'WPEP', $this->get_privacy_message() );
    }

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) && isset( $_GET[ 'wpep-dynamic-login-links' ] ) ) {
      $operation = $_GET[ 'wpep-dynamic-login-links' ];

      do_action(
        'wpep_deliver_csv_export_file',
        'wpep-auto-login-export-' . date_i18n( get_option( 'date_format' ) ),
        [
          'name'            => __( "Name", "wpep" ),
          'email_address'   => __( "Email Address", "wpep" ),
          'user_token'      => __( "Login Token", "wpep" ),
          "home_auto_login" => __( "Home Auto Login", "wpep" )
        ],
        wpep_controller()->userEngagement->autoLogin->get_data_export( $operation === 'reset-tokens' )
      );

      exit;

    }
  }

  public function _setup_action_admin_menu() {
    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\Admin\Controller->_setup_action_admin_menu() - start' );

    $this->menu->_register_menu();
    $this->discussion->_register_menu();
    $this->settings->_register_menu();
    $this->statistics->_register_menu();
    $this->platformHealth->_register_menu();
    $this->addOnsSection->_register_menu();
    $this->learningCenter->_register_menu();

    foreach (wpep_content_type()->standardContentTypes as $contentType ) {
      if ( !in_array('custom-fields', $contentType->get_post_supports() ) )
        continue;

      remove_meta_box( 'postcustom', $contentType->get_post_type(), 'normal' );
    }

    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\Admin\Controller->_setup_action_admin_menu() - end' );
  }

  public function _setup_rules_flush_action_init() {
    if( wpep_controller()->request->is_request_type( 'ajax' ) )
      return;

    try {
      flush_rewrite_rules();

      WPEP_Controller::instance()->capabilities->register_caps_and_roles();

      \WPEPCronController::reset_cron_events();

      WPEP_Controller::instance()->platformHealth->delete_service_status_notification_count_transient();
    } catch ( \Exception $exception ) {

    }

    wpep_set_setting( 'configuration-setup-rules-flushed', 1 );

    add_action( "init", function() {
      wpep_redirect( wpep_get_current_url() );
      exit;
    }, 500 );

  }

  public function _term_created( $term_id ) {
    update_term_meta( $term_id, 'wpep_order', intval( get_term_meta( $term_id, 'wpep_order', true  ) ) );
  }

  public function action_links( $links ) {
    $links[] = '<a href="'. esc_url( wpep_admin_url_settings() ) .'">' . __( "Settings", 'wpep' )  . '</a>';
    return $links;
  }

  public function user_row_actions( $actions, $user_object ) {
    if( wpep_content_type()->is_enabled( WPEP_POST_TYPE_COURSE ) )
      $actions[ 'wpep_course_statistics' ] = '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'content-types', 'content-type' => WPEP_POST_TYPE_COURSE, 'user_id' => $user_object->ID ] ) . '">' . __( "Course Statistics", "wpep" ) . '</a>';

    if( wpep_content_type()->is_enabled( WPEP_POST_TYPE_ASSESSMENT ) )
      $actions[ 'wpep_assessment_statistics' ] = '<a href="' . wpep_admin_url_statistics( [ 'tab' => 'content-types', 'content-type' => WPEP_POST_TYPE_ASSESSMENT, 'user_id' => $user_object->ID ] ) . '">' . __( "Assessment Statistics", "wpep" ) . '</a>';

    if( wpep_public_profiles_enabled() )
      $actions[ 'wpep_public_profile' ] = '<a href="' . wpep_user_profile_public_link( $user_object ). '">' . __( "WPEP Public Profile", "wpep" ) . '</a>';

    return $actions;
  }

  public function get_privacy_message() {
    $content = '
			<div contenteditable="false">' .
      '<p class="wp-policy-help">' .
      __( 'This sample language includes the basics around what personal data your WPEP powered eLearning Platform may be collecting, storing and sharing, as well as who may have access to that data. 

Depending on what settings are enabled and which additional plugins are used, the specific information shared by your store will vary. 

We recommend consulting with a lawyer when deciding what information to disclose on your privacy policy.
.', 'wpep' ) .
      '</p>' .
      '</div>' .
      '<p>' . __( 'We collect information about you while doing Courses & Assessments.', 'wpep' ) . '</p>' .
      '<h2>' . __( 'What we collect and store', 'wpep' ) . '</h2>' .
      '<p>' . __( 'While you visit our elearning platform, we’ll track:', 'wpep' ) . '</p>' .
      '<ul>' .
      '<li>' . __( 'Courses you have started, including the time you have started them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Courses you have completed, including the time you have completed them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Lessons you have completed, including the time you have completed them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Learning Paths you have chosen within courses.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Going through Assessments will store your answers, assessment status & the timestamp of your last interaction with the assessment.', 'wpep' ) . '</li>' .
      '</ul>' .
      '<h2>' . __( 'Who on our team has access', 'wpep' ) . '</h2>' .
      '<p>' . __( 'Members of our team such as administrators, platform managers and individual courses managers and instructors have access to the information you provide us. They can access:', 'wpep' ) . '</p>' .
      '<ul>' .
      '<li>' . __( 'Courses you have started, including the time you have started them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Courses you have completed, including the time you have completed them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Lessons you have completed, including the time you have completed them.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Learning Paths you have chosen within courses.', 'wpep' ) . '</li>' .
      '<li>' . __( 'Going through Assessments will store your answers, assessment status & the timestamp of your last interaction with the assessment..', 'wpep' ) . '</li>' .
      '</ul>' .
      '<p>' . __( 'Our team members have access to this information to improve the training & to support you on your learning journey.', 'wpep' ) . '</p>';

    return apply_filters( 'wpep_privacy_policy_content', $content );
  }

}