<?php

namespace WPEP\Admin\Coordinator\DataSystem;

use WPEP\Container\FormFields as WPEP_FormFieds;
use WPEP\Container\FileSystem as WPEP_FileSystem;
use WPEP\Container\DataWalker as WPEP_DataWalker;

class CSVUsers {

  use WPEP_FormFieds,
    WPEP_FileSystem,
    WPEP_DataWalker;
  
  private $_option_name = '_wpep_data_system_csv_users_import_progress';

  public function setup() {
    add_action( 'wp_ajax_wpep_data_system_import_csv_users_process', [ $this, 'import_csv_process' ] );
  }

  public function import_csv_process() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) || !current_user_can( WPEP_CAP_MANAGE_MEMBERS )  )
      exit( '<div class="wpep-notification">' . __( "Your account is not allowed to complete this operation", "wpep" ) . '</div>');

    // Lets allow multiple users to go trough this at the same time.
    $this->_option_name = $this->_option_name . '_' . get_current_user_id();

    @ini_set('memory_limit', '512M');
    @ini_set('max_execution_time', '180');

    $sync_stage = isset( $_POST['sync_stage'] ) ? $_POST['sync_stage'] : 'dynamic';

    if( $sync_stage === 'import-file' && isset( $_POST[ 'import_file' ] ) ) {
      $this->_sync_stage_import_file();
      exit;
    }

    $import_progress_data = get_option( $this->_option_name, false );

    if( $import_progress_data == false ) {
      $this->_select_file();

      exit;
    }

    $csv_file = file( $import_progress_data['file'] );

    if( $csv_file === false ) {
      delete_option( $this->_option_name );

      echo '<div class="wpep-notification wpep-notification-danger">' . __( "The CSV File couldn't be read, please retry. ", "wpep" ) . '</div>';

      $this->_select_file();
      exit;
    }

    if( $sync_stage === 'import-start' ) {
      $import_progress_data['review_status'] = 1;
      update_option( $this->_option_name, $import_progress_data );

      $this->_sync_stage_email_notifications( $import_progress_data );

      exit;
    } elseif( $sync_stage === 'import-finish' ) {
      delete_option( $this->_option_name );

      $this->_select_file();

      exit;
    } elseif( $sync_stage === 'import-cancel' ) {
      delete_option( $this->_option_name );

      $this->_select_file();

      exit;
    } elseif( $sync_stage === 'import-review' || $import_progress_data['review_status'] == 0 ) {
      $this->_sync_stage_review_file( $csv_file );

      exit;
    } elseif( $sync_stage === 'import-update-email-notifications' ) {
      if( isset( $_POST['email_notification_status'] ) )
        $import_progress_data['email_notification_status'] = intval( $_POST['email_notification_status'] );

      if( isset( $_POST[ 'email_notification_title' ] ) )
        $import_progress_data['email_notification_title']  = stripslashes( $_POST[ 'email_notification_title' ] );

      if( isset( $_POST[ 'email_notification_content' ] ) )
        $import_progress_data['email_notification_content']  = stripslashes( $_POST[ 'email_notification_content' ] );

      update_option( $this->_option_name, $import_progress_data );

      if( !$import_progress_data['email_notification_status'] ) {
        $this->_sync_stage_import_users( $import_progress_data, $csv_file );
      } else {
        $this->_sync_stage_preview_email_notification( $import_progress_data );
      }
      exit;
    } elseif( $sync_stage === 'import-email-notifications' || $import_progress_data['email_notification_status'] == -1 ) {
      $this->_sync_stage_email_notifications( $import_progress_data );

      exit;
    } elseif( $sync_stage === 'import-process' ) {
      $this->_sync_stage_import_users( $import_progress_data, $csv_file );

      exit;
    } elseif( $sync_stage === 'import-completed' ) {
      $this->_sync_stage_process_completed( $import_progress_data );
      exit;
    }

    do_action(
      'wpep_administration_csv_users_import_sync_stage',
      $sync_stage,
      $import_progress_data,
      $this->_option_name
    );

    if( $import_progress_data['import_line'] >= $import_progress_data['total_lines'] ) {
      $this->_sync_stage_process_completed( $import_progress_data );
      exit;
    } else {
      $this->_sync_stage_paused( $import_progress_data );
      exit;
    }
  }

  private function _sync_stage_paused( $import_progress_data ) {
    $import_navigation = '<input class="wpep-button wpep-button-secondary" 
                                 type="submit" 
                                 data-switch-sync-step="import-cancel"
                                 value="' . __( "Cancel Import", "wpep" ) . '"/>
                          <input class="wpep-button wpep-button-primary" 
                                 type="submit" 
                                 data-switch-sync-step="import-process"
                                 value="' . __( "Continue Import", "wpep" ) . '"/>';

    echo '<div class="wpep-title-container">';
    echo    '<h2 class="wpep-title">' . __( "Paused Import Detected", "wpep" ) . '</h2>';
    echo    '<div class="wpep-import-operations-container">';
    echo      apply_filters( 'wpep_administration_csv_users_import_navigation', $import_navigation, 'import-paused', $import_progress_data );
    echo    '</div>';
    echo '</div>';

    echo $this->_import_progress_report( $import_progress_data );

    echo '<div class="clearfix"></div>';

    echo '<div class="wpep-import-operations-container">';

    echo '</div>';
  }

  private function _sync_stage_import_file() {
    $csv_file = $_POST[ 'import_file' ];

    if( !wpep_string_ends_with( $csv_file, '.csv' ) ) {
      echo '<div class="wpep-notification wpep-notification-danger">' . __( "You must upload a CSV file, must end with .csv", "wpep" ) . '</div>';

      $this->_select_file();

      return;
    }

    $upload_dir   = wp_upload_dir();

    $csv_file = str_replace( $upload_dir['baseurl'], $upload_dir['basedir'], $csv_file );

    $file = file( $csv_file );

    if ( $file !== FALSE ) {
      update_option( $this->_option_name, [
        'started_at'    => time(),
        'completed_at'  => 0,
        'file'          => $csv_file,
        'review_status' => 0,
        'import_line'   => 0,
        'total_lines'   => count( $file ),
        'users_created' => 0,
        'users_total'   => 0,
        'user_ids'      => [],
        'email_notification_status'   => -1,
        'email_notification_title'    => '',
        'email_notification_content'  => ''
      ] );

      $this->_sync_stage_review_file( $file );

      return;
    }

    echo '<div class="wpep-notification wpep-notification-danger">' . __( "The CSV File couldn't be read, please retry. ", "wpep" ) . '</div>';

    $this->_select_file();
  }

  public function _sync_stage_review_file( $csv_file ) {
    $csv_file_lines = count( $csv_file );

    if( $csv_file_lines <= 10 ) {
      $data = $this->_csv_to_array( $csv_file );
    } else {
      $data = array_merge(
        $this->_csv_to_array( array_slice( $csv_file, 0, 5 ), 1 ),
        [
          [
            'position'      => '...',
            'first_name'    => '...',
            'last_name'     => '...',
            'email_address' => '...',
            'password'      => '...'
          ]
        ],
        $this->_csv_to_array( array_slice( $csv_file, $csv_file_lines - 5, 5 ), $csv_file_lines - 4 )
      );
    }

    echo '<div class="wpep-title-container">';
    echo    '<h2 class="wpep-title">' . __( "Step 2: Review CSV File", "wpep" ) . '</h2>';
    echo    '<div class="wpep-import-operations-container">';
    echo    '<input class="wpep-button wpep-button-secondary" 
                    type="submit" 
                    data-switch-sync-step="import-cancel"
                    value="' . __( "Back", "wpep" ) . '"/>';
    echo    '<input class="wpep-button wpep-button-primary" 
                    type="submit" 
                    data-switch-sync-step="import-start"
                    value="' . __( "Continue", "wpep" ) . '"/>';
    echo    '</div>';
    echo '</div>';

    echo '<div class="wpep-import-review-container">';

    wpep_admin_display_table(
      false,
      [
        'position'      => '#',
        'first_name'    => __( "First Name", "wpep" ),
        'last_name'     => __( "Last Name", "wpep" ),
        'email_address' => __( "Email Address", "wpep" ),
        'password'      => __( "Password", "wpep" )
      ],
      $data
    );

    echo '</div>';
    exit;
  }

  /**
   * @param array $import_progress_data
   */
  public function _sync_stage_email_notifications( $import_progress_data ) {
    if( !isset( $import_progress_data['email_notification_status'] ) )
      $import_progress_data['email_notification_status'] = 1;
    if( !isset( $import_progress_data['email_notification_title'] ) || $import_progress_data['email_notification_title'] == '' ) {
      $site_name = ( is_multisite() ? get_network()->site_name : wp_specialchars_decode(get_option('blogname'), ENT_QUOTES) );

      $import_progress_data['email_notification_title'] = sprintf( __( "Your Account Details %s", "wpep" ), $site_name );
    }

    if( !isset( $import_progress_data['email_notification_content'] ) || $import_progress_data['email_notification_content'] == '' ) {
      $import_progress_data['email_notification_content'] = __( "Hello [user_name]", "wpep" ) . "\n" .
                                                            sprintf( __( "Your account for %s has been created", "wpep" ), ( isset( $site_name ) ? $site_name : '???' ) ) . "\n" .
                                                            __( "Email Address / Username : [email_address]", "wpep" ) . "\n" .
                                                            __( "Password : [password]", "wpep" ) . "\n" .
                                                            sprintf( __( "To Login visit : %s", "wpep" ), '<a href="[site_url]">[site_url]</a>' ) . "\n";
    }

    echo '<div class="wpep-title-container">';
    echo    '<h3 class="wpep-title">' . __( "Step 3: Configure Email Notifications", "wpep" ) . '</h3>';

    echo    '<div class="wpep-import-operations-container">';
    echo      '<input class="wpep-button wpep-button-secondary" 
                      type="submit" 
                      data-switch-sync-step="import-review"
                      value="' . __( "Back", "wpep" ) . '"/>';
    echo      '<input class="wpep-button wpep-button-primary" 
                      type="submit" 
                      data-switch-sync-step="import-update-email-notifications"
                      value="' . __( "Continue", "wpep" ) . '"/>';
    echo    '</div>';
    echo '</div>';

    echo  '<div class="wpep-utility-data-sync-row">';
    echo    '<label for="#wpep-email-notification-status">' . __( "Status", "wpep" ) . '</label>';
    echo    '<select id="wpep-email-notification-status" name="email_notification_status">';
    echo      '<option value="1" ' . wpep_utility_selected( $import_progress_data['email_notification_status'], '1', false ) . '>' . __( "Enabled", "wpep") . '</option>';
    echo      '<option value="0" ' . wpep_utility_selected( $import_progress_data['email_notification_status'], '0', false ) . '>' . __( "Disabled", "wpep") . '</option>';
    echo    '</select>';
    echo  '</div>';

    echo  '<div class="wpep-utility-data-sync-row">';
    echo    '<label for="#wpep-email-notification-status">' . __( "Title", "wpep" ) . '</label>';
    echo    '<input type="text" name="email_notification_title" class="regular-text" value="' . $import_progress_data['email_notification_title'] . '"/>';
    echo  '</div>';


    echo  '<div class="wpep-utility-data-sync-row">';
    echo    '<label for="#wpep-email-notification-content">' . __( "Content", "wpep" ) . '</label>';
    wpep_editor( $import_progress_data['email_notification_content'], 'wpep-email-notification-content', 'email_notification_content' );
    echo    '<div class="wpep-description">';
    echo      wpep_controller()->userEngagement->get_available_shortcode_list( false, true );
    echo    '</div>';
    echo  '</div>';

    do_action( 'admin_print_footer_scripts' );
  }

  public function _sync_stage_preview_email_notification( $import_progress_data ) {
    echo '<div class="wpep-title-container">';
    echo    '<h3 class="wpep-title">' . __( "Step 4: Preview Email", "wpep" ) . '</h3>';
    echo    '<div class="wpep-import-operations-container">';
    echo      '<input class="wpep-button wpep-button-secondary" 
                      type="submit" 
                      data-switch-sync-step="import-email-notifications"
                      value="' . __( "Back", "wpep" ) . '"/>';
    echo      '<input class="wpep-button wpep-button-primary" 
                      type="submit" 
                      data-switch-sync-step="import-process"
                      value="' . __( "Continue", "wpep" ) . '"/>';
    echo    '</div>';
    echo '</div>';

    echo '<ul class="wpep-process-information-list">';
    echo  '<li>' . __( "Already registered users will not receive Email Notifications.", "wpep" ) . '</li>';
    echo  '<li>' . __( "The notification will be sent right away, as the CSV file is processed.", "wpep" ) . '</li>';
    echo '</ul>';

    echo '<div class="wpep-simple-iframe-preview-container">';
    echo    '<iframe class="wpep-simple-iframe-preview" src="' . add_query_arg( 'wpep-preview-email', urlencode( $import_progress_data['email_notification_content'] ), get_site_url() ) . '"></iframe>';
    echo '</div>';
  }

  public function _sync_stage_import_users( $import_progress_data, $csv_file ) {
    if( $import_progress_data['import_line'] < $import_progress_data['total_lines'] ) {
      $current_entries = $this->_csv_to_array( array_slice( $csv_file, $import_progress_data['import_line'], WPEP_USER_IMPORT_ENTRIES_PER_JOB ) );

      foreach( $current_entries as $current_entry_key => $current_entry ) {
        if( !isset( $current_entry[ 'email_address' ] ) ) {
          unset( $current_entries[ $current_entry_key ] );
          continue;
        }

        $user = get_user_by( 'email', $current_entry[ 'email_address' ] );

        if( $user !== false ) {
          $current_entries[ $current_entry_key ][ 'status' ] = __( "Already Registered", "wpep" );

          if( in_array( $user->ID, $import_progress_data['user_ids'] ) )
            continue;

          $import_progress_data['user_ids'][] = $user->ID;
          $import_progress_data['users_total']++;

          continue;
        }

        $password = ( isset( $current_entry['password'] ) && $current_entry['password'] !== '' ? $current_entry['password'] : wp_generate_password() );

        $user_id = wp_create_user( $current_entry['email_address'], $password, $current_entry['email_address'] );

        if( is_wp_error( $user_id ) ) {
          $current_entries[ $current_entry_key ][ 'status' ] = __( "Failed", "wpep" );
          continue;
        }

        $import_progress_data['user_ids'][] = $user_id;
        $import_progress_data['users_total']++;
        $import_progress_data['users_created']++;

        $current_entries[ $current_entry_key ][ 'status' ] = __( "Registered", "wpep" );
        $current_entries[ $current_entry_key ][ 'password' ] = $password;

        wp_update_user( [
          'ID'          => $user_id,
          'first_name'  => $current_entry['first_name'],
          'last_name'   => $current_entry['last_name'],
        ] );

        if( !$import_progress_data['email_notification_status'] )
          continue;

        $content = $import_progress_data['email_notification_content'];
        $content = str_replace( '[password]', $password, $content );

        wpep_controller()->userEngagement->send_email(
          $user_id,
          $import_progress_data['email_notification_title'],
          $content,
          true
        );

        $current_entries[ $current_entry_key ][ 'status' ] = __( "Email Sent", "wpep" );
      }

      if( $import_progress_data['import_line'] >= $import_progress_data['total_lines'] )
        $import_progress_data['completed_at'] = time();

      $import_progress_data['import_line'] += WPEP_USER_IMPORT_ENTRIES_PER_JOB;
      update_option( $this->_option_name, $import_progress_data );
    }


    if( $import_progress_data['import_line'] >= $import_progress_data['total_lines'] ) {
      if( $import_progress_data['completed_at'] === 0 ) {
        $import_progress_data['completed_at'] = time();
        update_option( $this->_option_name, $import_progress_data );
      }

      $this->_sync_stage_process_completed( $import_progress_data );
      exit;
    } else {
      echo '<div class="wpep-title-container">';
      echo   '<h3 class="wpep-title">' . sprintf(
          __( "Processed %s / %s lines.", "wpep" ),
          ( $import_progress_data['import_line'] > $import_progress_data['total_lines'] ? $import_progress_data['total_lines'] : $import_progress_data['import_line'] ),
          $import_progress_data['total_lines']
        ) . '</h3>';
      echo   '<input class="wpep-button wpep-button-secondary" 
                     type="submit" 
                     data-switch-sync-step="import-pause"
                     data-switch-sync-step-type="optional"
                     value="' . __( "Pause", "wpep" ) . '"/>';
      echo '</div>';

      if( isset( $current_entries ) && !empty( $current_entries ) ) {
        wpep_admin_display_table(
          false,
          [
            'status'        => __( 'Status', "wpep" ),
            'first_name'    => __( "First Name", "wpep" ),
            'last_name'     => __( "Last Name", "wpep" ),
            'email_address' => __( "Email Address", "wpep" ),
            'password'      => __( "Password", "wpep" )
          ],
          $current_entries
        );
      }

      exit;
    }
  }

  private function _sync_stage_process_completed( $import_progress_data ) {
    if( !isset( $import_progress_data['completed_at'] )
      || ( $import_progress_data['completed_at'] != 0 && $import_progress_data['completed_at'] + DAY_IN_SECONDS < time() ) ) {
      delete_option( $this->_option_name );

      $this->_select_file();

      return;
    }

    $import_navigation = '<input class="wpep-button wpep-button-secondary" 
                                 type="submit" 
                                 data-switch-sync-step="import-finish"
                                 value="' . __( "New Import", "wpep" ) . '"/>';

    echo '<div class="wpep-title-container">';
    echo   '<h3 class="wpep-title">' . __( "Import Completed", "wpep" ) . '</h3>';
    echo    '<div class="wpep-import-operations-container">';
    echo      apply_filters( 'wpep_administration_csv_users_import_navigation', $import_navigation, 'import-completed', $import_progress_data );
    echo    '</div>';
    echo '</div>';

    echo $this->_import_progress_report( $import_progress_data );
  }

  private function _import_progress_report( $import_progress_data ) {
    $response = '';

    $response .= '<ul class="wpep-process-information-list">';
    if( isset( $import_progress_data['completed_at' ] ) && $import_progress_data['completed_at' ] != 0 ) {
      $response .= '<li>' . sprintf( __( "Import has been completed %s ago, at %s", "wpep" ), human_time_diff( time(), $import_progress_data['completed_at' ] ), wpep_format_date_with_time( $import_progress_data['completed_at' ] ) ) . '</li>';
    }
    $response .=   '<li>' . sprintf( __( "Import has started %s ago, at %s", "wpep" ), human_time_diff( time(), $import_progress_data['started_at' ] ), wpep_format_date_with_time( $import_progress_data['started_at' ] ) ) . '</li>';
    $response .=   '<li>' . sprintf( __( "Processed %s / %s CSV Lines." ), ( $import_progress_data['import_line'] > $import_progress_data['total_lines'] ? $import_progress_data['total_lines'] : $import_progress_data['import_line'] ) , $import_progress_data['total_lines'] ). '</li>';
    $response .=   '<li>' . sprintf( __( "%s Users Total." ), $import_progress_data['users_total'] ) . ' ' . ( $import_progress_data[ 'email_notification_status' ] == 1 ? __( "( Email Notifications enabled )", "wpep" ) : '' ) . '</li>';
    $response .=   '<li>' . sprintf( __( "%s New Users." ), $import_progress_data['users_created'] ). '</li>';
    $response .=   '<li>' . sprintf( __( "%s Existent Users." ), $import_progress_data['users_total'] - $import_progress_data['users_created'] ). '</li>';
    $response .= '</ul>';

    return $response;
  }

  private function _select_file() {
    echo '<div class="wpep-title-container">';
    echo   '<h3 class="wpep-title">' . __( "Step 1: Upload CSV File", "wpep" ) . '</h3>';
    echo   '<input class="wpep-button wpep-button-primary" 
                   type="submit" 
                   data-switch-sync-step="import-file"
                   value="' . __( "Continue", "wpep" ) . '"/>';
    echo '</div>';

    echo '<div class="wpep-import-file-container">';
    $this->display_file_field("import_file", "import_file" );

    echo    '<p class="wpep-description">';
    echo      '<strong><a href="' . WPEP_BASE_URL_PATH . 'assets/wpep-csv-users-import-sample.csv">' . __( "CSV Format", "wpep" ) . '</a></strong>';
    echo      __( " : First Name;Last Name;Email Address;Password.", "wpep" );
    echo    '</p>';
    echo '</div>';
  }

  private function _csv_to_array( $csv_lines, $line_position = 1 ) {
    $response = [];

    foreach ( $csv_lines as $line) {
      $line = str_getcsv( $line, ';' );

      if( $line[0] === 'First Name' && $line[1] === 'Last Name' )
        continue;

      $response[] = [
        'position'      => $line_position,
        'first_name'    => $line[0],
        'last_name'     => $line[1],
        'email_address' => $line[2],
        'password'      => isset( $line[3] ) ? $line[3] : 0
      ];

      $line_position++;
    }

    return $response;
  }

}
