<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings   as WPEP_Settings;

class Settings {

  /**
   * @var array
   */
  protected $_settingsPages;

  public function setup() {
    // We want our functions with inits to also receive fresh information.
    add_action( 'init', [ $this, '_init' ], 0 );
  }

  public function _register_menu() {
    wpep_admin_register_menu(
      __("WPEP Settings", 'wpep'),
      __( "Settings", 'wpep'),
      WPEP_CAP_MANAGE_SETTINGS,
      "wpep-settings",
      [ $this, "display" ]
    );
  }

  public $displayed_form_submit = false;

  public function _init() {
    if( !is_admin() )
      return;

    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return;

    if( !isset( $_POST['wpep_settings_save'] ) )
      return;

    if( isset( $_POST['wpep_settings'] ) ) {
      WPEP_Settings::instance()->update_settings( $_POST['wpep_settings'] );

      WPEP_Controller::instance()->platformHealth->delete_service_status_notification_count_transient();
    }

    if( isset( $_POST['wpep_settings_post_order'] ) )
      WPEP_Settings::instance()->update_post_order( array_keys( $_POST['wpep_settings_post_order'] ) );

    if( isset( $_POST['wpep_settings_taxonomy_order'] ) )
      WPEP_Settings::instance()->update_taxonomy_order( array_keys( $_POST['wpep_settings_taxonomy_order'] ) );

    if( isset( $_POST['wpep_add_on_settings'] ) ) {
      WPEP_Settings::instance()->update_addon_settings( $_POST['wpep_add_on_settings'] );

      WPEP_Controller::instance()->platformHealth->delete_service_status_notification_count_transient();
    }

    do_action( "wpep_administration_settings_saved", $_POST );

    $url = add_query_arg( 'wpep-settings-saved', 'true', wpep_get_current_url() );
    $url = remove_query_arg( 'wpep-walk-trough', $url );

    wpep_redirect( $url );
    exit;
  }

  public function display() {
    $settings_pages = $this->get_settings_pages();

    echo '<div class="wrap wpep-settings-container" data-wpep-page="settings" data-wpep-tab-count="' . count( $settings_pages ) . '">';
    echo '<h2 class="wpep-title-page">' . __( "WPEP Settings", 'wpep') . '</h2>';

    if( isset( $_GET['wpep-settings-saved']) ) {
      echo '<div class="updated notice is-dismissible">';
      echo  '<p>' . __('Settings have been updated.', 'wpep') . '</p>';
      echo  '<button class="notice-dismiss" type="button"><span class="screen-reader-text">' . __( "Dismiss this notice.", 'wpep') . '</span></button>';
      echo '</div>';
    }

    $current_tab = ( isset( $_GET['tab'] ) ? $_GET['tab'] : key( $settings_pages ) );

    if( !isset( $settings_pages[ $current_tab ] ) )
      $current_tab = key( $settings_pages );

    if( $settings_pages[ $current_tab ]->display_as_form() ) {
      echo '<form novalidate="novalidate" method="post">';
    }

    $this->show_tabs( $current_tab );

    $settings_pages[ $current_tab ]->display();

    if( $settings_pages[ $current_tab ]->display_as_form() ) {
      if( $this->displayed_form_submit == false )
        echo  '<p class="submit"><input type="submit" value="' . __( "Save Changes", "wpep" ) . '" class="button button-primary" id="submit" name="wpep_settings_save"></p>';

      echo '</form>';
    }

    echo '</div>';
  }

  public function show_tabs( $current = 'homepage' ) {
    echo '<h2 class="wpep-navigation wpep-navigation-primary" data-wpep-navigation="settings">';
    foreach( $this->get_settings_pages() as $tab => $setting_page ){
      $class = ( $tab == $current ) ? ' wpep-navigation-item-active' : '';
      echo '<a data-wpep-tab="' . $tab . '" class="wpep-navigation-item' . $class . '" href="' . wpep_admin_url_settings( [ 'tab' => $tab ] ) . '">';
      echo    '<span>' . $setting_page->get_name() . '</span>';
      echo '</a>';

    }
    echo '</h2>';

    echo '<div class="wpep-clearfix"></div>';
  }

  /**
   * @return Settings\Skeleton[]
   */
  public function get_settings_pages() {
    if( !is_array( $this->_settingsPages ) ) {
      $this->_settingsPages = [];

      $page_instances = [
        new Settings\Dashboard(),
        new Settings\General(),
        new Settings\IndexPage()
      ];

      if( in_array( WPEP_POST_TYPE_COURSE, wpep_content_type()->library_post_types() ) )
        $page_instances = array_merge( $page_instances, [ new Settings\CoursePage() ] );

      $page_instances = array_merge( $page_instances, [
        new Settings\LearnerProfile(),
        new Settings\LearnerEngagement()
      ]);

      if( !empty( WPEP_Controller::instance()->addOns->get_add_on_settings_list() ) )
        $page_instances[] = new Settings\AddOns();

      foreach( $page_instances as $current_instance )
        $this->_settingsPages[ $current_instance->get_alias() ] = $current_instance;

      $this->_settingsPages = apply_filters( 'wpep_administration_settings_pages', $this->_settingsPages );
    }

    return $this->_settingsPages;
  }

}