<?php

namespace WPEP\Admin\Coordinator;

use WPEP\Controller as WPEP_Controller;

class Statistics {

  /**
   * @var array
   */
  protected $_statisticsPages;

  public function filter_by_user_role() {
    $user_filters[] = [
      'user_role' => false
    ];

    if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
      $user_filters = get_user_meta( get_current_user_id(), 'wpep_statistics_filter_settings', true );

    $user_filters = apply_filters( 'wpep_administration_statistics_user_filters', $user_filters );

    return isset(  $user_filters['user_role' ] ) && $user_filters['user_role'] !== '' ? $user_filters['user_role'] : false;
  }

  private function _get_cap() {
    return apply_filters( 'wpep_administration_statistics_user_cap', WPEP_CAP_MANAGE_STATISTICS );
  }

  public function _init() {
    if( !is_admin() )
      return;

    if( !current_user_can( $this->_get_cap() ) )
      return;

    if( !isset( $_GET['page'] ) )
      return;

    if( $_GET['page'] != 'wpep-statistics' )
      return;

    if( isset( $_POST['wpep_user_role_filter'] ) && current_user_can( WPEP_CAP_MANAGE_STATISTICS ) ) {
      update_user_meta( get_current_user_id(), 'wpep_statistics_filter_settings', [
        'user_role' => sanitize_text_field( $_POST['wpep_user_role_filter'] )
      ] );
    }

    if( !isset( $_POST['wpep_download_statistics'] ) )
      return;

    $current_statistics_page = $this->get_statistics_pages( $this->get_current_tab() );

    if( method_exists( $current_statistics_page, 'download' ) )
      $current_statistics_page->download( [
        'user_role' => $this->filter_by_user_role()
      ] );
  }

  public function _register_menu() {
    wpep_admin_register_menu(
      __("WPEP Statistics", 'wpep'),
      __( "Statistics", 'wpep'),
      $this->_get_cap(),
      "wpep-statistics",
      [ $this, "display" ]
    );
  }

  public function display() {
    echo '<div class="wrap" data-wpep-page="statistics">';

    if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
      echo $this->_get_user_role_filter_dropdown();
    else
      echo '<h2 class="wpep-title-page">' . __( "WPEP Statistics", 'wpep') . '</h2>';

    $current_statistics_page = $this->get_statistics_pages( $this->get_current_tab() );

    if( empty( $current_statistics_page ) ) {

      echo '<div class="wpep-notification wpep-notification-info">' . __( "There are no statistics available for your Platform", "wpep" ) . '</div>';

    } else {

      $this->show_tabs( $this->get_current_tab() );

      $current_statistics_page->display( [
        'user_role' => $this->filter_by_user_role()
      ] );

    }

    echo '</div>';
  }

  private function _get_user_role_filter_dropdown() {
    $role_filter = '';

    $editable_roles = array_reverse( get_editable_roles() );

    $role_filter .= '<form method="POST" id="wpep_user_role_filter_container">';
    $role_filter .=   '<h2 class="wpep-title-page">' . __( "WPEP Statistics", 'wpep') . '</h2>';

    $role_filter .=   '<select id="wpep_user_role_filter" name="wpep_user_role_filter">';

    $role_filter .= "\n\t<option value=''>" . __( "All Roles", "wpep" ) . "</option>";

    foreach ( $editable_roles as $role => $details ) {
      $name = translate_user_role($details['name'] );
      // preselect specified role
      if ( $this->filter_by_user_role() == $role ) {
        $role_filter .= "\n\t<option selected='selected' value='" . esc_attr( $role ) . "'>$name</option>";
      } else {
        $role_filter .= "\n\t<option value='" . esc_attr( $role ) . "'>$name</option>";
      }
    }

    $role_filter .=   '</select>';
    $role_filter .=   '<div class="clear"></div>';
    $role_filter .= '</form>';

    return $role_filter;
  }

  public function show_tabs( $current = 'homepage' ) {
    echo '<h2 class="wpep-navigation wpep-navigation-primary" data-wpep-navigation="statistics">';
    foreach( $this->get_statistics_pages() as $tab => $setting_page ){
      $class = ( $tab == $current ) ? ' wpep-navigation-item-active' : '';
        echo '<a data-wpep-tab="' . $tab . '" class="wpep-navigation-item' . $class . '" href="' . wpep_admin_url_statistics( [ 'tab' => $tab ] ). '">';
        echo    '<span>' . $setting_page->get_name() . '</span>';
        echo '</a>';
    }
    echo '</h2>';

      echo '<div class="wpep-clearfix"></div>';
  }

  public function get_current_tab() {
    $statistics_pages = $this->get_statistics_pages();

    $current_tab = ( isset( $_GET['tab'] ) ? $_GET['tab'] : key( $statistics_pages ) );

    if( !isset( $statistics_pages[ $current_tab ] ) )
      $current_tab = key( $statistics_pages );

    return $current_tab;
  }

  /**
   * @param bool $index
   * @return Statistics\Skeleton[]
   */
  public function get_statistics_pages( $index = false ) {
    if( !is_array( $this->_statisticsPages ) ) {
      $this->_statisticsPages = [];

      $objectInstances = [];

      if( wpep_content_type()->is_enabled( WPEP_POST_TYPE_COURSE ) ) {

        if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) )
          $objectInstances[] = new Statistics\Dashboard();

        $objectInstances[] = new Statistics\ContentTypes();

        if( ( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) || current_user_can( WPEP_CAP_MANAGE_MEMBERS ) ) ) {
          $objectInstances[] = new Statistics\Users();

         // @todo add the new admin here
         // if( wpep_get_setting_flag( 'enable-course-timing-option' ) )
         //   $objectInstances[] = new Statistics\UserNotifications();

          if( wpep_has_sid_support() )
            $objectInstances[] = new Statistics\GAReport();
        }
      }

      if( current_user_can( WPEP_CAP_MANAGE_STATISTICS ) && ( defined( "WPEP_BETA_TESTING" ) || defined( "WPEP_BETA_TESTING_TEACHER_NOTIFICATIONS" ) ) )
        $objectInstances[] = new Statistics\Emails();

      foreach( $objectInstances as $current_instance )
        $this->_statisticsPages[ $current_instance->get_alias() ] = $current_instance;

      $this->_statisticsPages = apply_filters( 'wpep_administration_statistics_pages', $this->_statisticsPages );
    }

    return ( $index == false ? $this->_statisticsPages : $this->_statisticsPages[$index] );
  }

}
