<?php

namespace WPEP;

class ContentTaxonomy {

  /**
   * @var ContentTaxonomy
   */
  protected static $_instance;

  public static function instance() {
    if( self::$_instance === null )
      self::$_instance = new self();

    return self::$_instance;
  }

  public function setup() {
    add_action( 'init', [ $this, '_register_category' ], WPEP_TAXONOMY_REGISTRATION_PRIORITY );

    if( !is_admin() )
      return;

    add_action( 'admin_menu', [ $this, '_register_category_menu' ] );
    add_filter( 'parent_file', [ $this, '_verify_category_menu' ] );
    add_action( 'restrict_manage_posts', [ $this, '_content_type_filtering' ] );
  }

  public function data_export_array() {
    return [
      WPEP_TAXONOMY_CATEGORY => $this->_get_taxonomy_list( WPEP_TAXONOMY_CATEGORY )
    ];
  }

  public function data_import_array( $import_data, $id_mapping ) {
    foreach( [ WPEP_TAXONOMY_CATEGORY ] as $taxonomy ) {
      if( !isset( $import_data[ $taxonomy ] ) )
        continue;

      $id_mapping[ $taxonomy ] = $this->_import_taxonomy_list( $import_data[ $taxonomy ], $taxonomy );
    }

    return $id_mapping;
  }

  public function data_asset_map() {
    return [
      WPEP_TAXONOMY_CATEGORY => [
        'meta' => [
          'icon' => '',
          'url'  => ''
        ]
      ]
    ];
  }

  public function _register_category() {
    $labels = [
      'name'                       => __('WPEP Categories', 'wpep'),
      'singular_name'              => __('Plan Category', 'wpep'),
      'menu_name'                  => __('Categories', 'wpep'),
      'all_items'                  => __('All Items', 'wpep'),
      'parent_item'                => __('Parent Item', 'wpep'),
      'parent_item_colon'          => __('Parent Item:', 'wpep'),
      'new_item_name'              => __('New Category', 'wpep'),
      'add_new_item'               => __('Add New Category', 'wpep'),
      'edit_item'                  => __('Edit Category', 'wpep'),
      'update_item'                => __('Update Category', 'wpep'),
      'view_item'                  => __('View Category', 'wpep'),
      'separate_items_with_commas' => __('Separate items with commas', 'wpep'),
      'add_or_remove_items'        => __('Add or remove items', 'wpep'),
      'choose_from_most_used'      => __('Choose from the most used', 'wpep'),
      'popular_items'              => __('Popular Categories', 'wpep'),
      'search_items'               => __('Search Categories', 'wpep'),
      'not_found'                  => __('Not Found', 'wpep'),
    ];
    $args   = [
      'labels'            => $labels,
      'hierarchical'      => true,
      'public'            => true,
      'show_ui'           => true,
      'show_in_menu'      => WPEP_ADMIN_BASE_MENU_SLUG,
      'show_admin_column' => true,
      'show_in_nav_menus' => true,
      'show_tagcloud'     => false,
      'rewrite'             => [
        'slug'  => wpep_get_setting( 'rewrite-slug-categories', WPEP_TAXONOMY_CATEGORY )
      ],
      'capabilities'          => array(
        'manage_terms' => 'manage_' . WPEP_POST_CAP_COURSE . '_terms',
        'edit_terms'   => 'edit_'   . WPEP_POST_CAP_COURSE . '_terms',
        'delete_terms' => 'delete_' . WPEP_POST_CAP_COURSE . '_terms',
        'assign_terms' => 'assign_' . WPEP_POST_CAP_COURSE . '_terms',
      ),
    ];

    register_taxonomy(WPEP_TAXONOMY_CATEGORY, [], $args);
  }

  public function _register_category_menu() {
    wpep_admin_register_menu(
      __( 'All Categories', "wpep" ),
      __( 'All Categories', "wpep" ),
      'manage_' . WPEP_POST_CAP_COURSE . '_terms',
      'edit-tags.php?taxonomy=' . WPEP_TAXONOMY_CATEGORY,
      false
    );
  }

  public function _verify_category_menu( $parent_file ) {
    if( get_current_screen() == null )
      return $parent_file;

    if( isset( get_current_screen()->taxonomy ) && get_current_screen()->taxonomy == WPEP_TAXONOMY_CATEGORY )
      return WPEP_ADMIN_BASE_MENU_SLUG;

    return $parent_file;
  }

  public function _content_type_filtering( $post_type ) {
    if( !isset( wpep_content_type()->libraryContentTypes[$post_type] ) )
      return;

    // Retrieve taxonomy terms
    $terms = get_terms( WPEP_TAXONOMY_CATEGORY );

    // Display filter HTML
    echo '<select name="' . WPEP_TAXONOMY_CATEGORY . '" id="' . WPEP_TAXONOMY_CATEGORY . '" class="postform">';
    echo '<option value="">' . esc_html__( 'Show All Categories', 'wpep' ) . '</option>';
    foreach ( $terms as $term )
      echo '<option value="' . $term->slug . '" ' .
                    selected( ( isset( $_GET[ WPEP_TAXONOMY_CATEGORY ] ) && ( $_GET[ WPEP_TAXONOMY_CATEGORY ] == $term->slug ) ), true, false ) .
           '>' . $term->name . '</option>';
    echo '</select>';
  }

  private function _get_taxonomy_list( $taxonomy, $parent = 0, $post_id = false ) {
    $response = [];

    $term_query = [ 'taxonomy' => $taxonomy, 'hide_empty' => false, 'parent' => $parent ];

    if( $post_id !== false ) {
      $term_query['include'] = wp_get_post_terms( $post_id, $taxonomy, [ 'fields' => 'ids' ] );

      if( empty( $term_query['include'] ) )
        return $response;
    }

    $course_category = get_terms( $term_query );

    if( is_wp_error( $course_category ) )
      return $response;

    foreach( $course_category as $category ) {
      $response[$category->term_id] = [
        'term_id'   => $category->term_id,
        'name'      => $category->name,
        'slug'      => $category->slug,
        'children'  => $this->_get_taxonomy_list( $taxonomy, $category->term_id, $post_id ),
        'meta'      => get_term_meta( $category->term_id )
      ];
    }

    return $response;
  }

  /**
   * @param $import_data
   * @param $taxonomy
   * @param int $parent_id
   * @return array
   */
  private function _import_taxonomy_list( $import_data, $taxonomy, $parent_id = 0 ) {
    $response = [];

    foreach( $import_data as $category ) {
      $term_information = wp_insert_term(
        $category['name'], // the term
        WPEP_TAXONOMY_CATEGORY, // the taxonomy
        [
          'slug'    => $category['slug'],
          'parent'  => $parent_id
        ]
      );

      if( is_wp_error( $term_information ) ) {
        $term_information = [ 'term_id' => $term_information->error_data['term_exists'] ];
      } else {
        if( !is_array( $term_information ) )
          continue;
      }

      $response[ $category['term_id' ] ] = $term_information['term_id'];

      if( isset( $category['meta'] ) && !empty( $category['meta'] ) ) {
        foreach( $category['meta'] as $category_meta_key => $category_meta_value )
          add_term_meta( $term_information['term_id'], $category_meta_key, $category_meta_value );
      }

      if( !empty( $category['children'] ) )
        $response += $this->_import_taxonomy_list( $category['children'], $taxonomy, $term_information['term_id'] );
    }

    return $response;
  }

}