<?php

namespace WPEP\ContentType;

use WPEP\ContentType\Course\Container\Library as Course_Library;
use WPEP\ContentType\Course\Container\Data as Course_Data;

class Course extends Skeleton\Library {

  public $data_post_ignored_meta = [ '_edit_lock', '_edit_last', '_oembed_*', 'wpep_prerequisites' ];

  use Course_Library,
      Course_Data;

  /**
   * @var Course\Admin
   */
  public $admin;

  /**
   * @var Course\Block
   */
  public $block;

  /**
   * @var Course\Delivery
   */
  public $delivery;

  /**
   * @var Course\PreAuth
   */
  public $preAuth;

  /**
   * @var Course\Privacy
   */
  public $privacy;

  /**
   * @var Course\Request
   */
  public $request;

  /**
   * @var Course\UserProfile
   */
  public $userProfile;

  /**
   * @var Course\Integration\Discussion
   */
  public $discussionIntegration;

  /**
   * @var Course\Container\PostDependency[]
   */
  protected $_post_dependencies = [];

  public function __construct() {
    $this->admin       = new Course\Admin();
    $this->block       = new Course\Block();
    $this->delivery    = new Course\Delivery();
    $this->preAuth     = new Course\PreAuth();
    $this->privacy     = new Course\Privacy();
    $this->request     = new Course\Request();
    $this->userProfile = new Course\UserProfile();

    $this->discussionIntegration = new Course\Integration\Discussion();

    add_action( 'delete_post', [ $this, "_on_delete_post" ] );
    add_action( 'delete_user', [ $this, "_on_delete_user" ] );
  }

  public function init() {
    if( wpep_content_type()->is_enabled( WPEP_POST_TYPE_ASSESSMENT ) )
      $this->_post_dependencies[ WPEP_POST_TYPE_ASSESSMENT ] = wpep_content_type()->standardContentTypes[ WPEP_POST_TYPE_ASSESSMENT ];

    $this->preAuth->init();
    $this->admin->init();
  }

  public function _on_delete_post( $post_id ) {
    if( get_post_type( $post_id ) != WPEP_POST_TYPE_COURSE )
      return;

    global $wpdb;

    $lesson_ids = $wpdb->get_col(
      'SELECT id FROM ' . $wpdb->prefix . 'wpep_section_lesson WHERE post_id = ' . $post_id,
      0
    );

    if( !empty ( $lesson_ids ) ) {
      $wpdb->query( "DELETE FROM " . $wpdb->prefix . 'wpep_section_lesson WHERE id IN ( ' . implode( ',', $lesson_ids ). ' ) ' );
      $wpdb->query( "DELETE FROM " . $wpdb->prefix . 'wpep_section_lesson_completed WHERE lesson_id IN ( ' . implode( ',', $lesson_ids ). ' ) ' );
    };

    $wpdb->delete( $wpdb->prefix . 'wpep_section', [ 'post_id' => $post_id ] );
    $wpdb->delete( $wpdb->prefix . 'wpep_course_user_data', [ 'course_id' => $post_id ] );
  }

  public function _on_delete_user( $user_id ) {
    global $wpdb;

    $wpdb->delete( $wpdb->prefix . 'wpep_section_lesson_completed', [ 'user_id' => $user_id ] );
    $wpdb->delete( $wpdb->prefix . 'wpep_course_user_data', [ 'user_id' => $user_id ] );
  }

  /**
   * @return Course\Integration\Statistics
   */
  public function get_statistics_instance() {
    return new Course\Integration\Statistics();
  }

  /**
   * @return Course\Integration\LearningJourney
   */
  public function get_learning_journey_instance() {
    return new Course\Integration\LearningJourney( $this );
  }

  public function get_post_supports() {
    return [ 'title', "custom-fields" ];
  }

  public function get_post_type() {
    return WPEP_POST_TYPE_COURSE;
  }

  public function get_singular_name() {
    return 'Course';
  }

  public function get_plural_name() {
    return 'Courses';
  }

  /**
   * @param \WPEP\Admin\Coordinator\Settings\IndexPage $indexPageSettings
   */
  public function display_settings_fields( $indexPageSettings ) {
    $indexPageSettings->display_table_start();

    $indexPageSettings->display_table_select_field(
      'Courses Per Row',
      'index-page-courses-per-row',
      wpep_content_primary_row_options( WPEP_POST_TYPE_COURSE )
    );

    $indexPageSettings->display_table_checkbox_field( __( "Enable Image Hover" ), 'index-page-enable-hover-image', __( "Enable a different image on the presentation card on the Index Page on hover.", "wpep" ) );

    $indexPageSettings->display_table_checkbox_field( __( 'Progress Bars', "wpep" ), 'index-page-progress-bars', __( "Display Progress Bars under each course to hint the student which courses are completed.", "wpep" ) );

    $indexPageSettings->display_table_separator( __( "Button", "wpep" ) );

    $indexPageSettings->display_table_text_translated_field ( __('Start Text', "wpep"),  'opt-course-start-text', __( 'Personalize the Course Start Button Text', "wpep" ) );

    $indexPageSettings->display_table_text_translated_field ( __('Continue Text', "wpep"),  'opt-course-continue-text', __( 'Personalize the Course Continue Button Text', "wpep" ) );

    $indexPageSettings->display_table_text_translated_field ( __('Coming Soon', "wpep"),  'opt-course-coming-soon', __( 'Personalize the Coming Soon Button Text, when an Course is going to be available soon.', "wpep" ) );

    $indexPageSettings->display_table_color_field( __('Text Colour', "wpep"),  'index-page-start-text-color', __( 'Personalize the Button Text Colour', "wpep" ) );
    $indexPageSettings->display_table_color_field( __('Text Background Colour', "wpep"),  'index-page-start-text-color-background', __( 'Personalize the Button Text Background Colour', "wpep" ) );

    $indexPageSettings->display_table_separator( __( "Ribbon", "wpep" ) );

    $indexPageSettings->display_table_color_field( __('Text Colour', "wpep"),  'index-page-ribbon-text-color', __( 'Personalize the Ribbon Text Colour', "wpep" ) );
    $indexPageSettings->display_table_color_field( __('Background Colour', "wpep"),  'index-page-ribbon-text-color-background', __( 'Personalize the Ribbon  Background Colour', "wpep" ) );

    $indexPageSettings->display_table_end();
  }

  public function get_settings_defaults() {
    return [
      'index-page-courses-per-row'     => 3,
      'index-page-enable-hover-image'  => 0,
      'index-page-progress-bars'       => 0,
      'opt-course-start-text'          => 'START',
      'opt-course-continue-text'       => 'Continue',
      'opt-course-coming-soon'         => 'Coming Soon',
      'index-page-start-text-color'              => '',
      'index-page-start-text-color-background'   => '',
      'index-page-ribbon-text-color'             => '',
      'index-page-ribbon-text-color-background'  => '',
    ];
  }

  public function standard_get_post_meta_fields() {
    return $this->admin->get_meta_fields();
  }

  public function standard_get_post_meta_fields_context() {
    return 'side';
  }

  /**
   * @return mixed
   */
  public function get_cap_name() {
    return WPEP_POST_CAP_COURSE;
  }

}