<?php

namespace WPEP\ContentType\Course;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\CourseType as Entity_CourseType;

class Delivery {

  /**
   * @var Entity_CourseType\Skeleton
   */
  protected $currentDelivery = false;

  public $top_level_request = false;
  public $current_course_id = false;
  public $current_course_sidebar_option = false;
  
  public $current_prerequisite_mode    = null;
  public $current_prerequisites        = [];
  public $current_prerequisites_locked = false;

  private $_forced_sidebar_disable = false;

  public function __construct() {
    if( !WPEP_Controller::instance()->request->is_request_type( 'frontend' ) )
      return;

    add_action( 'init', [ $this, '_action_init' ], 15 );
  }

  public function get_current_course_id() {
    return $this->current_course_id;
  }

  public function _action_init() {
    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\ContentType\Course\Delivery()->_action_init() - start' );

    $this->current_course_id = wpep_get_current_post_id();

    if( get_post_type( $this->current_course_id ) != WPEP_POST_TYPE_COURSE )
      return;

    if( isset( $_GET['wpep-course-repeat'] ) && is_user_logged_in() ) {
      if( wp_verify_nonce( $_GET['wpep-course-repeat'], 'wpep-repeat_' . $this->current_course_id ) === false )
        wp_nonce_ays( 'wpep_course_repeat_trigger' );

      WPEP_Controller::instance()->userActivity->course_progress_reset( $this->current_course_id, get_current_user_id() );

      $this->_refresh_course_page();
    }

    if( is_user_logged_in() && isset( $_GET['wpep-login'] ) )
      $this->_refresh_course_page();

    if( wpep_post_has_member_magnet_option( $this->current_course_id, get_current_user_id() ) ) {
      if( !in_array( wpep_course_type_from_id( $this->current_course_id ), wpep_course_type_member_magnet_integrated() )  ) {
        if( !isset( $_GET['wpep-login'] ) ) {
          wpep_redirect( wpep_post_member_magnet_link( $this->current_course_id ) );
          exit;
        }
      }
    } elseif( wpep_post_has_member_magnet_option( $this->current_course_id, 0 ) && current_user_can( WPEP_CAP_EDIT_COURSE ) ) {
      WPEP_Controller::instance()->notifications->add_author_notification(
        __( "This course is protected by a Member Magnet, logout or visit the page in incognito to try it out.", "wpep" ),
        'info'
      );
    }

    $course_theme = get_post_meta( $this->current_course_id, 'course_theme', true );

    if( $course_theme !== '' && !WPEP_Controller::instance()->templateStyle->custom_style_delivered )
      WPEP_Controller::instance()->templateStyle->theme = $course_theme;

    if( wpep_course_has_the_content_filter() )
      add_filter( 'the_content', [ $this, 'the_content_filter' ], 1 );

    remove_filter ('the_content', 'wpautop');

    if( function_exists( 'gutenberg_wpautop' ) )
      remove_filter( 'the_content', 'gutenberg_wpautop', 8 );

    $this->top_level_request = true;

    $this->init();

    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\ContentType\Course\Delivery()->_action_init() - end' );
  }
  
  public function init( $current_course_id = false ) {
    if( $current_course_id != false ) {
      if( $this->current_course_id == $current_course_id
          && wpep_course_type_from_id( $current_course_id ) == $this->currentDelivery->get_alias() )
        return;

      $this->current_course_id = $current_course_id;

      if( get_post_type( $this->current_course_id ) != WPEP_POST_TYPE_COURSE )
        return;
    }

    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\ContentType\Course\Delivery()->init() - start' );

    if( $this->currentDelivery !== false )
      $this->close();

    $this->currentDelivery = Entity_CourseType::instance()->get_coordinator( $this->current_course_id );

    if( $this->currentDelivery == false )
      wp_die( __( "There is something wrong with this course, please check contact the owner and notify him." ), "wpep" );

    $this->currentDelivery->top_level_request = $this->top_level_request;

    if( wpep_post_has_member_magnet_option( $this->current_course_id, get_current_user_id() ) )
      wpep_current_request_setup_member_magnet();

    $this->_setup_prerequisites();
    $this->before_currently_deliver_init();

    $this->currentDelivery->init();

    if( isset( $_GET['wpep-course-repeat'] ) && is_user_logged_in() ) {
      if( wp_verify_nonce( $_GET['wpep-course-repeat'], 'wpep-repeat_' . $this->current_course_id ) === false )
        wp_nonce_ays( 'wpep_course_repeat_trigger' );

      WPEP_Controller::instance()->userActivity->course_progress_reset( $this->current_course_id, get_current_user_id() );

      $this->_refresh_course_page();
    }

    add_filter( 'wpep_single_content_type_alias', [ $this->currentDelivery, 'get_alias' ], 10 );
    add_action( 'wpep_single_content_sidebar', [ $this, 'action_wpep_single_sidebar' ], 10 );

    if( WPEP_DEV_ENABLE_BLACK_FIRE )
      \BlackfireProbe::addMarker( 'WPEP\ContentType\Course\Delivery()->init() - end' );
  }

  public function close() {
    remove_filter( 'wpep_single_content_type_alias', [ $this->currentDelivery, 'get_alias' ], 10 );

    $this->currentDelivery->close();

    unset( $this->currentDelivery );

    remove_action( 'wpep_single_content_sidebar', [ $this, 'action_wpep_single_sidebar' ], 10 );
    remove_filter( 'wpep_single_content_sidebar_active', [ $this, 'single_content_sidebar_active_filter' ], 10 );
    remove_filter( 'wpep_single_content_type_attr_classes', [ $this, 'single_content_type_classes_filter' ], 10 );

    if( $this->_forced_sidebar_disable )
      remove_filter( 'wpep_single_content_sidebar_active', '__return_false', 1000 );
  }

  public function the_content_filter( $content ) {
    if( post_password_required() )
      return $content;

    if( get_post_type() !== WPEP_POST_TYPE_COURSE )
      return $content;

    if( $this->currentDelivery == false )
      return $content;

    if( $this->current_prerequisites_locked )
      return $this->_get_prerequisites_content();

    return $this->get_current_course_content();
  }

  public function get_current_course_content() {
    $content_before = "";
    $content_after  = "";

    if( $this->top_level_request && wpep_post_coming_soon( $this->current_course_id ) ) {
      if( !current_user_can( WPEP_CAP_EDIT_COURSE ) )
        return wpep_format_content( wpep_get_setting( 'course-coming-soon-student-notification' ) );

      $content_before .= '<div class="wpep-notification wpep-notification-info">' . __( "This course is set to Coming Soon, students may not access it.", "wpep" ) . '</div>';
    }

    if( wpep_get_setting( 'enable-progress-repeat-support' )
        && get_post_meta( $this->current_course_id, 'wpep_progress_repeat_enabled', true ) == 1 ) {
      $content_after .= wpep_template_get( 'partial/course-progress-repeat.php', [
        'course_id' => $this->current_course_id
      ] );
    }

    if( wpep_has_engagement_embed_support()
        && (
          !isset( $_GET['wpep-embed'] )
            || wpep_get_setting_flag( 'user-content-embed-box-inside-embeds' )
        ) ) {
      $embed_post_id = wpep_get_current_post_id();

      if( $embed_post_id === 0 )
        $embed_post_id = get_the_ID();

      if( $embed_post_id != 0 ) {
        $is_active = get_post_meta( $embed_post_id,'wpep_embed_status', true );

        if( $is_active == '' )
          $is_active = wpep_get_setting_flag( 'user-content-embed-box-course-status' );

        if( $is_active ) {
          $content_after .= WPEP_Controller::instance()->template->get_template('partial/user-engagement-embed.php', [
            'embed_post_id' => $embed_post_id
          ] );
        }
      }
    }

    if( $this->currentDelivery->has_next_suggestions_after_completion == false )
      $content_after .= $this->_get_recommended_follow_up_content();

    if( wpep_user_course_is_completed( $this->get_current_course_id(), get_current_user_id() ) ) {
      $content_after .= wpep_prepare_tracking_code(
        get_post_meta( $this->get_current_course_id(), 'wpep_tracking_code_completed', true ),
        'course-completed'
      );
    }

    return apply_filters( 'wpep_single_content_course_before_html', $content_before, $this->current_course_id ) .
           $this->currentDelivery->content() .
           apply_filters( 'wpep_single_content_course_after_html', is_string( $content_after ) ? $content_after : "", $this->current_course_id );
  }

  /**
   * @return Entity_CourseType\Skeleton
   */
  public function getCurrentDeliveryMethod() {
    return $this->currentDelivery;
  }

  public function getCurrentDeliveryAlias() {
    return ( $this->currentDelivery == false ? '' : $this->currentDelivery->get_alias() );
  }

  public function before_currently_deliver_init() {
    if( $this->top_level_request === false && !$this->currentDelivery->has_sidebar_content() ) {
      add_filter( 'wpep_single_content_sidebar_active', '__return_false', 1000 );
      $this->_forced_sidebar_disable = true;
    }

    $this->current_course_sidebar_option = ( $this->current_course_sidebar_option == false ? get_post_meta( $this->current_course_id, 'wpep_sidebar_position', true ) : $this->current_course_sidebar_option );

    if( $this->getCurrentDeliveryAlias() == WPEP_COURSE_TYPE_CHALLENGE
        && in_array( $this->current_course_sidebar_option, [ 'default', 'none' ] ) )
      $this->current_course_sidebar_option = 'left';

    $do_sidebar_filter = $this->current_course_sidebar_option !== '' && $this->current_course_sidebar_option != 'default';

    if( $do_sidebar_filter ) {
      add_filter( 'wpep_single_content_sidebar_active', [ $this, 'single_content_sidebar_active_filter' ], 10 );
      add_filter( 'wpep_single_content_type_attr_classes', [ $this, 'single_content_type_classes_filter' ], 10 );
    }
  }

  public function action_wpep_single_sidebar( $course_id = false ) {
    if( !wpep_content_primary_sidebar_active() )
      return;

    $course_id = ( $course_id == false ? get_the_ID() : $course_id );

    if( $course_id != $this->current_course_id )
      return;

    echo '<div class="course-sidebar-content" role="complementary">';
    echo    '<div class="wpep-widget-area">';

    do_action( 'wpep_single_sidebar_before', $course_id );

    if ( is_active_sidebar('wpep-sidebar') && $this->top_level_request )
      dynamic_sidebar('wpep-sidebar');

    do_action( 'wpep_single_sidebar_after', $course_id );

    echo    '</div>';
    echo '</div>';
  }

  public function single_content_sidebar_active_filter( $status ) {
    if( $this->current_course_sidebar_option == 'disabled' )
      return false;

    if( $this->current_course_sidebar_option !== '' && is_active_sidebar('wpep-sidebar') )
      return true;

    return $status;
  }

  public function single_content_type_classes_filter( $classes = [] ) {
    $classes['sidebar-position'] = 'sidebar-position-' . $this->current_course_sidebar_option;

    return $classes;
  }

  private function _get_prerequisites_content() {
    WPEP_Controller::instance()->notifications->add_notification(
      sprintf( wpep_get_setting( 'prerequisite-required-notification' ),
        '<a href="' . get_permalink( $this->current_course_id ). '"><strong>' . get_the_title( $this->current_course_id ) . '</strong></a>'
      ),
      'warning'
    );

    return wpep_content_primary_course_list_by_ids( $this->current_prerequisites );
  }

  private function _setup_prerequisites() {
    $this->current_prerequisites = apply_filters( "wpep_single_course_prerequisites_list", get_post_meta( $this->current_course_id, 'wpep_prerequisite' ) );

    if( empty( $this->current_prerequisites ) )
      return;

    if( current_user_can( WPEP_CAP_EDIT_COURSE ) ) {
      $course_string = '';

      foreach( $this->current_prerequisites as $prerequisite ) {
        $permalink = get_the_permalink( $prerequisite );

        if( $permalink === false )
          continue;

        $course_string .= ( $course_string == '' ? '' : ', ' );
        $course_string .= '<a href="' . $permalink . '">';
        $course_string .=   get_the_title( $prerequisite );
        $course_string .=   ' ( ' . wpep_get_percentage( WPEP_Controller::instance()->userActivity->course_progress_percentage( $prerequisite, get_current_user_id() ) ) . ' )';
        $course_string .= '</a>';
      }

      if( $course_string == '' )
        return;

      WPEP_Controller::instance()->notifications->add_author_notification(
        sprintf( __( 'This course has the following prerequisites : %s'), $course_string ),
        'info'
      );

      return;
    }

    $this->current_prerequisite_mode = wpep_get_prerequisite_mode( get_post_meta( $this->current_course_id, WPEP_PREREQUISITE_MODE_POST_META, true ) );

    $completed_prerequisites = [];

    foreach( $this->current_prerequisites as $prerequisite ) {
      if( !wpep_user_course_is_completed( $prerequisite, get_current_user_id(), true ) )
        continue;

      $completed_prerequisites[] = $prerequisite;
    }

    if( $this->current_prerequisites_locked )
      $this->current_course_sidebar_option = 'disabled';

    $requirement = ( $this->current_prerequisite_mode[WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT] === 'all' ? count( $this->current_prerequisites ) : $this->current_prerequisite_mode[WPEP_PREREQUISITE_FIELD_COMPLETION_REQUIREMENT] );

    $this->current_prerequisites_locked = ( $requirement != 0 && $requirement > count( $completed_prerequisites )  );

    if( $this->current_prerequisites_locked )
      return;

    if( !$this->current_prerequisite_mode[ WPEP_PREREQUISITE_FIELD_SHOW_NOTIFICATION ] )
      return;

    $course_string = '';

    foreach( $this->current_prerequisites as $prerequisite ) {
      if( in_array( $prerequisite, $completed_prerequisites ) )
        continue;

      $permalink = get_the_permalink( $prerequisite );

      if( $permalink === false )
        continue;

      $course_string .= ( $course_string == '' ? '' : ', ' );
      $course_string .= '<a href="' . $permalink . '">';
      $course_string .=   get_the_title( $prerequisite );
      $course_string .=   ' ( ' . wpep_get_percentage( WPEP_Controller::instance()->userActivity->course_progress_percentage( $prerequisite, get_current_user_id() ) ) . ' )';
      $course_string .= '</a>';
    }

    if( $course_string !== '' )
      WPEP_Controller::instance()->notifications->add_notification(
        sprintf( wpep_get_setting_translated('prerequisite-suggested-notification'), $course_string ),
        'info'
        );
  }

  private function _get_recommended_follow_up_content() {
    if( !apply_filters( "wpep_single_course_has_next_suggestions", $this->currentDelivery->has_next_suggestions ) )
      return '';

    $recommended_courses = get_posts([
      'post_type'   => WPEP_POST_TYPE_COURSE,
      'numberposts' => -1,
      'meta_key'    => 'wpep_prerequisite',
      'meta_value'  => $this->current_course_id,
      'fields'      => 'ids'
    ]);

    if( empty( $recommended_courses ) )
      return '';

    $content  = '';
    $content .= '<div class="wpep-follow-up-content wpep-follow-up-content-on-course-' . $this->currentDelivery->get_alias() . '">';
    $content .=   '<h2>' . wpep_get_setting_translated('prerequisite-next-courses-title' ). '</h2>';
    $content .=   wpep_content_primary_course_list_by_ids( $recommended_courses, [
      'lock_by_prerequisites' => 0
    ] );
    $content .= '</div>';

    return $content;
  }

  private function _refresh_course_page() {
    $redirect = wpep_course_permalink( $this->current_course_id );

    if( wpep_has_embed_support() && isset( $_GET['wpep-embed'] ) )
      $redirect = add_query_arg( 'wpep-embed', 1, $redirect );

    $redirect = remove_query_arg( 'wpep-course-repeat', $redirect );

    wpep_redirect( $redirect );
    exit;
  }

}