<?php

namespace WPEP\ContentType\Course;

class Privacy {

  public function __construct() {
    add_filter( 'wp_privacy_personal_data_exporters', [ $this, 'register_exporters' ], 7 );
  }

  /**
   * Integrate this exporter implementation within the WordPress core exporters.
   *
   * @param array $exporters List of exporter callbacks.
   * @return array
   */
  public function register_exporters( $exporters = [] ) {
    $exporters[ 'wpep-courses' ] = [
      'exporter_friendly_name' => __( "Course Information", "wpep" ),
      'callback'               => [ $this, '_export_information' ],
    ];

    return $exporters;
  }

  public function _export_information( $email_address, $page ) {
    $number = 10;
    $page   = (int) $page;

    $data_to_export = [];

    $user = get_user_by( 'email' , $email_address );

    if ( false === $user ) {
      return array(
        'data' => $data_to_export,
        'done' => true,
      );
    }

    $started_course_list    = wpep_controller()->userActivity->statistics->get_started_course_list( $user->ID, false, false );
    $completed_course_ids   = wpep_controller()->userActivity->statistics->get_completed_course_list( $user->ID, false );
    $bookmarked_course_list = wpep_course_has_bookmarks() ? wpep_controller()->userActivity->statistics->get_bookmarked_course_list( $user->ID ) : [];

    $post_query = new \WP_Query( [
      'posts_per_page' => $number,
      'paged'          => $page,
      'post_type'      => WPEP_POST_TYPE_COURSE,
      'numberposts'    => -1,
      'orderby'        => 'post__in',
      'order'          => 'ASC',
      'post__in'       => array_keys( $started_course_list ),
      'post_status'    => 'any',
    ] );

    foreach ( (array) $post_query->posts as $post ) {
      $post_data_to_export = [
        [
          'name'  => __( "Course" ),
          "value" => get_the_title( $post )
        ],
        [
          'name'  => __( 'Started At' ),
          'value' => wpep_format_date_with_time( $started_course_list[ $post->ID ] )
        ],
      ];

      if( in_array( $post->ID, $bookmarked_course_list ) ) {
        $post_data_to_export[] = [
          'name'  => __( "Is Bookmarked" ),
          'value' => __( "Yes" )
        ];
      }

      if( in_array( $post->ID, $completed_course_ids ) ) {
        $post_data_to_export[] = [
          'name'  => __( "Completed at" ),
          'value' => wpep_format_date_with_time( wpep_controller()->userActivity->get_course_user_data_value( WPEP_USER_COURSE_COMPLETED_AT, $post->ID, false, false, $user->ID ) )
        ];
      } else {
        $post_data_to_export[] = [
          'name'  => __( 'Progress' ),
          'value' => wpep_get_percentage( wpep_user_course_progress_percentage( $post->ID, $user->ID ) )
        ];
      }

      if( wpep_course_type_from_id( $post->ID ) != WPEP_COURSE_TYPE_ONE_OFF ) {
        $course_information = wpep_controller()->userActivity->get_course_sections( $post->ID, $user->ID, false );

        foreach( $course_information as $course_section ) {
          if(
            isset( $course_section['options']['question_answers'] )
            && !empty( $course_section['options']['question_answers'] )
            && count( $course_section['options']['question_answers'] ) >= 2
          ) {
            $user_selection = ( isset( $course_section['user_data']['learning_path'] ) ? $course_section['user_data']['learning_path'] : - 1 );

            if( $user_selection == -1 )
              continue;

            if( !isset( $course_section['options']['question_answers'][$user_selection] ) )
              continue;

            $post_data_to_export[] = [
              'name'  => $course_section['title'],
              'value' => $course_section['options']['question_answers'][$user_selection]
            ];

          }

          foreach( $course_section['lessons'] as $course_lesson) {
            if( $course_lesson['completed_at'] == 0 )
              continue;

            $post_data_to_export[] = [
              'name'  => $course_lesson['title'],
              'value' => wpep_format_date_with_time( $course_lesson['completed_at'] )
            ];
          }
        }
      }

      $data_to_export[] = array(
        'group_id'    => 'wpep-courses',
        'group_label' => __( 'Courses' ),
        'item_id'     => "post-{$post->ID}",
        'data'        => $post_data_to_export,
      );
    }

    $done = $post_query->max_num_pages <= $page;

    return array(
      'data' => $data_to_export,
      'done' => $done,
    );
  }

}