<?php

namespace WPEP\ContentType;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class EBook extends Skeleton\Library {

  /**
   * @return EBook\Statistics
   */
  public function get_statistics_instance() {
    return new EBook\Statistics();
  }

  /**
   * @return EBook\LearningJourney
   */
  public function get_learning_journey_instance() {
    return new EBook\LearningJourney( $this );
  }

  public function get_post_supports() {
    return [ 'title', 'thumbnail', "custom-fields" ];
  }

  public function get_post_type() {
    return WPEP_POST_TYPE_E_BOOK;
  }

  public function get_singular_name() {
    return 'EBook';
  }

  public function get_plural_name() {
    return 'EBooks';
  }

  /**
   * @param \WPEP\Admin\Coordinator\Settings\IndexPage $indexPageSettings
   */
  public function display_settings_fields( $indexPageSettings ) {
    $indexPageSettings->display_table_start();

    $indexPageSettings->display_table_select_field(
      __( 'Items Per Row', "wpep" ),
      'index-page-ebooks-per-row',
      wpep_content_primary_row_options()
    );

    $indexPageSettings->display_table_checkbox_field( __( 'Enable Redirect', "wpep" ),  'ebook-redirect-display', __( 'In case you want to protect your e-books with a membership plugin that is not integrated with WPEP.', "wpep" ) );

    $indexPageSettings->display_table_editor_field( __( 'Redirect Message', "wpep" ), 'ebook-redirect-message', __( 'This message will be displayed if the redirect method is enabled, after 3 seconds the user is redirected to the respective ebook link.', "wpep" ) );

    $indexPageSettings->display_table_text_translated_field( __( 'Redirect Button', "wpep" ), 'ebook-redirect-button' );

    $indexPageSettings->display_table_end();
  }

  public function get_settings_defaults() {
    return [
      'index-page-ebooks-per-row'  => 3,
      'ebook-redirect-display'     => 0,
      'ebook-redirect-message'     => "Please wait a few moments...",
      'ebook-redirect-button'      => "Don't wait",
    ];
  }

  public function standard_get_post_meta_fields() {
    return [
      [
        'name' => __( "Enter the Download URL for you Ebook here", "wpep" ),
        'id'   => '_ebook_link',
        'type' => 'text',
      ],
      [
        'name'    => __('Button Text', "wpep"),
        'id'      => 'wpep_post_button_text',
        'type'    => 'text',
        'desc'    => __( 'The Button Text on the Index Page, leave this empty for default behaviour.', "wpep" )
      ],
      [
        'name'    => __('Button Color', "wpep"),
        'id'      => 'wpep_post_button_color',
        'type'    => 'colorpicker',
        'desc'    => __( 'Choose a different color for the button, leave this empty for default behaviour.', "wpep" )
      ],
      [
        'name'    => __('Button Text Color', "wpep"),
        'id'      => 'wpep_post_button_text_color',
        'type'    => 'colorpicker',
        'desc'    => __( 'Choose a different text color for the button, leave this empty for default behaviour.', "wpep" )
      ],
    ];
  }

  /**
   * @return mixed
   */
  public function get_cap_name() {
    return WPEP_POST_CAP_E_BOOK;
  }

  public function init() {
    if( wpep_controller()->request->is_request_type( 'admin' ) )
      return;

    add_filter('init', [$this, '_inner_filter_init'], 15);
    add_filter('the_content', [$this, '_inner_filter_the_content'], 1);
  }

  public function get_grid_content_options() {
    $per_row_options = [ '' => __( "Default", "wpep" ) ] + wpep_content_primary_row_options() + [ 0 => __( "Do not display", "wpep" ) ];

    return [
      'ebooks'         => [
        'type'          => 'multiple_select',
        'options'       => [],
        'description'   => __( "If you want to Display specific E-Book Tabs only leave the Categories field empty and disable Category Navigation.", "wpep" ),
        'attributes'    => [
          'data-wpep-select-search' => json_encode( [
            'action'    => 'wpep_admin_post_list_search',
            'post_type' => $this->get_post_type()
          ] )
        ]
      ],
      'ebooks_per_row' => [
        'type'    => 'select',
        'options' => $per_row_options
      ],
    ];
  }

  public function process_grid_content_options( $filter_proxy_list, $attributes ) {
    $ebooks_per_row       = isset( $attributes['ebooks_per_row'] ) ? intval( $attributes['ebooks_per_row'] )  : '';
    $ebook_list_ids       = isset( $attributes['ebooks'] )         ? $attributes['ebooks']  : '';

    if( $ebooks_per_row !== '' )
      $filter_proxy_list['wpep_primary_content_grid_ebooks_per_row'] = function( $current_per_row ) use ( $ebooks_per_row ) {
        return $ebooks_per_row;
      };

    if( $ebook_list_ids !== '' ) {
      $filter_proxy_list['wpep_primary_content_grid_categories'] = [];

      $filter_proxy_list['wpep_primary_content_ebook_list_args'] = function ( $args ) use ( $ebook_list_ids ) {
        $args['post__in'] = wp_parse_id_list( $ebook_list_ids );

        return $args;
      };
    }

    return $filter_proxy_list;
  }

  public function library_output_grid_content_html( $course_categories = [], $post_list = [] ) {
    $items_per_row = apply_filters( 'wpep_primary_content_grid_ebooks_per_row', wpep_get_setting( 'index-page-ebooks-per-row' ) );

    if( $items_per_row == 0 )
      return;

    if( empty( $post_list ) ) {
      $post_list = $this->library_get_grid_post_list( $course_categories );

      if( empty( $post_list ) )
        return;
    }

    echo '<div class="wpep-ebook-grid-items-container" data-wpep-content-type="' . $this->get_post_type() . '" data-items-per-row="' . $items_per_row . '">';

    foreach( $post_list as $key => $value)
      echo wpep_controller()->template->get_template( 'primary/item-ebook.php', [
        'item'          => $value
      ] );

    echo '<div class="wpep-clearfix"></div>';
    echo '</div>';
  }

  public function library_allow_content() {
    return apply_filters( 'wpep_primary_content_grid_ebooks_per_row', wpep_get_setting( 'index-page-ebooks-per-row' ) ) != 0;
  }

  public function library_get_search_details( $current_details, $post ) {
    $thumbnail_id = get_post_thumbnail_id( $post->ID );

    if( $thumbnail_id !== '' )
      $current_details['thumbnail'] = get_post( $thumbnail_id, 'full' )->guid;

    $current_details['permalink'] = apply_filters(
      'wpep_primary_content_item_link',
      ( wpep_get_setting_flag( 'ebook-redirect-display' ) ? get_permalink( $post ) : wpep_controller()->actionTracking->get_event_url( WPEP_ACTION_TRACKING_EVENT_TYPE_OPEN_LINK, $post->ID ) ),
      $post->ID
    );

    return $current_details;
  }

  public function _inner_filter_init() {
    if( get_post_type( wpep_get_current_post_id() ) != WPEP_POST_TYPE_E_BOOK )
      return;

    if( wpep_post_has_member_magnet_option( wpep_get_current_post_id(), get_current_user_id() ) ) {
      if( !isset( $_GET['wpep-login'] ) ) {
        wpep_redirect( wpep_post_member_magnet_link( wpep_get_current_post_id() ) );
        exit;
      }

      wpep_current_request_setup_member_magnet();

      return;
    }

    if( ( is_user_logged_in() && isset( $_GET['wpep-login'] ) ) || wpep_get_setting_flag( 'ebook-redirect-display' ) == false ) {
      wpep_redirect( wpep_controller()->actionTracking->get_event_url( WPEP_ACTION_TRACKING_EVENT_TYPE_OPEN_LINK, wpep_get_current_post_id() ) );
      exit;
    }
  }

  public function _inner_filter_the_content( $content ) {
    if( post_password_required() )
      return $content;

    if( get_post_type() !== WPEP_POST_TYPE_E_BOOK )
      return $content;

    $link = wpep_controller()->actionTracking->get_event_url( WPEP_ACTION_TRACKING_EVENT_TYPE_OPEN_LINK, wpep_get_current_post_id() );

    return '<div class="wpep-ebook-redirect-container">' .
                wpautop( wpep_get_setting_translated( 'ebook-redirect-message' ) ) .
                '<p class="wpep-link-container"><a href="' . $link . '">' . wpep_get_setting_translated( 'ebook-redirect-button' ) . '</a></p>' .
                '<script type="text/javascript">setTimeout( function() { window.location.href = "' . $link . '"; }, 3000 );</script>' .
           '</div>';
  }

  public function data_user_activity_report( $args ) {
    $response = parent::data_user_activity_report( $args );

    $args['type'] = WPEP_ACTION_TRACKING_TYPE_OPEN_LINK;
    $args['post_type'] = $this->get_post_type();

    return $response + [
      __( "E-Books Downloaded", "wpep" ) => WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( $args )
    ];
  }

}