<?php

namespace WPEP\ContentType;

use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class Offer extends Skeleton\Library {

  /**
   * @return Offer\Statistics
   */
  public function get_statistics_instance() {
    return new Offer\Statistics();
  }

  /**
   * @return Offer\LearningJourney
   */
  public function get_learning_journey_instance() {
    return new Offer\LearningJourney( $this );
  }

  public function get_post_supports() {
    return [ 'title', 'thumbnail', "custom-fields" ];
  }

  public function get_post_type() {
    return WPEP_POST_TYPE_OFFER;
  }

  public function get_singular_name() {
    return 'Offer';
  }

  public function get_plural_name() {
    return 'Offers';
  }

  /**
   * @param \WPEP\Admin\Coordinator\Settings\IndexPage $indexPageSettings
   */
  public function display_settings_fields( $indexPageSettings ) {
    $indexPageSettings->display_table_start();

    $indexPageSettings->display_table_separator( __( "Frontend", "wpep" ) );

    $indexPageSettings->display_table_select_field(
      __( 'Offers Per Row', "wpep" ),
      'index-page-offers-per-row',
      wpep_content_primary_row_options()
    );

    $indexPageSettings->display_table_separator( __( "Administration", "wpep" ) );

    $indexPageSettings->display_table_attributes_field(
      __( "Default Attributes", "wpep"),
      'offer-information-map-default',
      __( "When creating a new offer, these fields will be the default displayed, optionally you can even have default values prepared if you want.", "wpep" ),
      [],
      wpep_get_setting( 'offer-information-map-default' )
    );

    $indexPageSettings->display_table_end();
  }

  public function get_settings_defaults() {
    return [
      'offer-information-map-default'     => [
        'Discount Offered' => '',
        'Duration'         => '',
        'Promo Code'       => ''
      ],
      'index-page-offers-per-row'               => 3,
    ];
  }

  public function standard_get_post_meta_fields() {
    return [
      [
        'name'        => __( 'Sub Title', "wpep" ),
        'id'          => 'offer_subtitle',
        'type'        => 'text',
        'description' => __( 'The sub-title is present under the title, this field is frequently used to display the offer type, for example "Monthly Membership".', "wpep" )
      ],
      [
        'name'        => __( 'Description', "wpep" ),
        'id'          => 'offer_description',
        'type'        => 'textarea',
        'description' => __( 'Text only offer description.', "wpep" )
      ],
      [
        'name'        => __( 'Link', "wpep" ),
        'id'          => 'offer_link',
        'type'        => 'text',
        'description' => __( 'The link where people will be taken when they redeem the offer.', "wpep" )
      ],
      [
        'name'        => __( 'Link Text', "wpep" ),
        'id'          => 'offer_link_text',
        'type'        => 'text',
        'description' => __( 'The text displayed on the button.', "wpep" )
      ],
      [
        'name'        => __( 'Highlighted Information', "wpep" ),
        'id'          => '_offer_information_map',
        'type'        => 'attributes',
        'description' =>
          sprintf(
            '<p>' . __( 'An offer is usually accompanied by a few key details, such as the discount value, the discount coupon, and even the discount availability.', "wpep" ) . '</p>' .
            '<p>' . __( "We allow each offer to have different highlighted information blocks, if you're using an standard format, you can %s", "wpep" ) . '</p>'
            ,
            '<a href="' . wpep_admin_url_settings( [ 'tab'  => 'index_page', 'tab_inner' => $this->get_post_type() ] ). '">' . __( "manage the Offers Settings", "wpep" ) . '</a>'
          ),
        'default'     => wpep_get_setting( 'offer-information-map-default' )
      ],
      [
        'name'    => __('Button Color', "wpep"),
        'id'      => 'wpep_post_button_color',
        'type'    => 'colorpicker',
        'desc'    => __( 'Choose a different color for the button, leave this empty for default behaviour.', "wpep" )
      ],
      [
        'name'    => __('Button Text Color', "wpep"),
        'id'      => 'wpep_post_button_text_color',
        'type'    => 'colorpicker',
        'desc'    => __( 'Choose a different text color for the button, leave this empty for default behaviour.', "wpep" )
      ],
    ];
  }

  /**
   * @return mixed
   */
  public function get_cap_name() {
    return WPEP_POST_CAP_OFFER;
  }

  public function is_publicly_queryable() {
    return false;
  }

  public function library_get_search_details( $current_details, $post ) {
    $current_details['permalink'] = apply_filters( 'wpep_primary_content_item_link', get_permalink( $post ) , $post->ID );

    $thumbnail_id = get_post_thumbnail_id( $post->ID );

    if( $thumbnail_id !== '' )
      $current_details['thumbnail'] = get_post( $thumbnail_id, 'full' )->guid;

    return $current_details;
  }

  public function get_grid_content_options() {
    $per_row_options = [ '' => __( "Default", "wpep" ) ] + wpep_content_primary_row_options() + [ 0 => __( "Do not display", "wpep" ) ];

    return [
      'offers'         => [
        'type'          => 'multiple_select',
        'options'       => [],
        'description'   => __( "If you want to Display specific Offers Tabs only leave the Categories field empty and disable Category Navigation.", "wpep" ),
        'attributes'    => [
          'data-wpep-select-search' => json_encode( [
            'action'    => 'wpep_admin_post_list_search',
            'post_type' => $this->get_post_type()
          ] )
        ]
      ],
      'offers_per_row' => [
        'type'    => 'select',
        'options' => $per_row_options
      ],
    ];
  }

  public function process_grid_content_options( $filter_proxy_list, $attributes ) {
    $offers_per_row       = isset($attributes['offers_per_row'])       ? intval( $attributes['offers_per_row'] )  : '';
    $offer_list_ids       = isset($attributes['offers'])               ? $attributes['offers']  : '';

    if( $offer_list_ids !== '' ) {
      $filter_proxy_list['wpep_primary_content_grid_categories'] = [];

      $filter_proxy_list['wpep_primary_content_offer_list_args'] = function ($args) use ($offer_list_ids) {
        $args['post__in'] = wp_parse_id_list( $offer_list_ids );

        return $args;
      };
    }

    if( $offers_per_row !== '' )
      $filter_proxy_list['wpep_primary_content_grid_offers_per_row'] = function( $current_per_row ) use ( $offers_per_row ) {
        return $offers_per_row;
      };

    return $filter_proxy_list;
  }

  public function library_output_grid_content_html( $course_categories = [], $post_list = [] ) {
    $items_per_row = apply_filters( 'wpep_primary_content_grid_offers_per_row', wpep_get_setting( 'index-page-offers-per-row' ) );

    if( $items_per_row == 0 )
      return;

    if( empty( $post_list ) ) {
      $post_list = $this->library_get_grid_post_list( $course_categories );

      if( empty( $post_list ) )
        return;
    }

    echo '<div class="wpep-offer-grid-items-container" data-wpep-content-type="' . $this->get_post_type() . '" data-items-per-row="' . $items_per_row . '">';

    foreach( $post_list as $key => $current_offer ) {
      $thumbnail_id = get_post_thumbnail_id( $current_offer->ID );

      $link_text = get_post_meta( $current_offer->ID, 'offer_link_text', true );

      if( $link_text == '' )
        $link_text = __( 'Redeem Offer', 'wpep' );

      echo wpep_controller()->template->get_template( 'primary/item-offer.php', [
        'offer_id'        => $current_offer->ID,
        'image'           => ( $thumbnail_id !== '' ? get_post( $thumbnail_id, 'full' )->guid : '' ),
        'title'           => get_the_title( $current_offer ),
        'sub_title'       => get_post_meta( $current_offer->ID, 'offer_subtitle', true ),
        'description'     => get_post_meta( $current_offer->ID, 'offer_description', true ),
        'link'            => apply_filters( 'wpep_primary_content_item_link', wpep_controller()->actionTracking->get_event_url( WPEP_ACTION_TRACKING_EVENT_TYPE_OPEN_LINK, $current_offer->ID ), $current_offer->ID ),
        'link_text'       => apply_filters( 'wpep_primary_content_item_link_title', $link_text, $current_offer->ID ),
        'information_map' => get_post_meta( $current_offer->ID, '_offer_information_map', true )
      ] );
    }
    echo '<div class="wpep-clearfix"></div>';
    echo '</div>';
  }

  public function library_allow_content() {
    return apply_filters( 'wpep_primary_content_grid_offers_per_row', wpep_get_setting( 'index-page-offers-per-row' ) ) != 0;
  }

  public function data_user_activity_report( $args ) {
    $response = parent::data_user_activity_report( $args );

    $args['type'] = WPEP_ACTION_TRACKING_TYPE_OPEN_LINK;
    $args['post_type'] = $this->get_post_type();

    return $response + [
      __( "Offers Redeemed", "wpep" ) => WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( $args )
    ];
  }

}