<?php
namespace WPEP\Coordinator\Ajax;

use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;

class Admin {

  public function __construct() {
    if( !is_admin() )
      return;

    add_action('wp_ajax_wpep_admin_platform_health_export', [ $this, 'platform_health_export' ] );
    add_action('wp_ajax_wpep_admin_menu_notifications', [ $this, 'menu_notifications' ] );
    add_action('wp_ajax_wpep_admin_disable_legacy_comments', [ $this, 'disable_legacy_comments' ] );
    add_action('wp_ajax_wpep_admin_update_course_user_data', [ $this, 'update_course_user_data' ] );
    add_action('wp_ajax_wpep_admin_save_post', [ $this, 'wpep_admin_save_post_ajax_coordinator' ] );
    add_action('wp_ajax_wpep_admin_user_icon_list', [ $this, 'user_icon_list' ] );
    add_action('wp_ajax_wpep_admin_post_list_search', [ $this, 'post_list_search' ] );
    add_action('wp_ajax_wpep_admin_post_learning_journey_search', [ $this, 'post_learning_journey_search' ] );
    add_action('wp_ajax_wpep_admin_learning_journey_date_modify_list', [ $this, 'learning_journey_date_modify_list' ] );
    add_action('wp_ajax_wpep_admin_user_list_search', [ $this, 'user_list_search' ] );

    add_action('wp_ajax_wpep_admin_plugin_get_plugin_status', [ $this, 'get_plugin_status' ] );
    add_action('wp_ajax_wpep_admin_plugin_action_install', [ $this, 'install_store_product' ] );
    add_action('wp_ajax_wpep_admin_plugin_action_activate', [ $this, 'activate_plugin' ] );
    add_action('wp_ajax_wpep_admin_plugin_action_deactivate', [ $this, 'deactivate_plugin' ] );

    if( isset( $_GET['wpep-editor-display'] ) )
      add_action( 'admin_init', [ $this, 'wpep_admin_editor'], 100 );
  }

  public function platform_health_export() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => sprintf( __( "Current user cannot %s.", "wpep" ), WPEP_CAP_MANAGE_SETTINGS )
      ] ) );

    $response = '';

    $response .= '---------- ' . __( "Service Status Report", "wpep" ) . ' ----------' . "\n\n";

    foreach( WPEP_Controller::instance()->platformHealth->get_service_status_report() as $service_key => $service_information ) {
      $response .=  wpep_format_report_data_to_line( $service_key, [
          'status'  => $service_information['status'],
          'type'    => 'service_status_report',
          'value'   => ( $service_information['status'] === 'success' ? true : ( $service_information['status'] === '' ? 'Not Configured' : false ) ),
          'label'   => $service_information['title']
        ] ) . "\n";
    }

    foreach( WPEP_Controller::instance()->platformHealth->get_report() as $report_key => $report_group ) {
      $response .=  "\n";

      $response .=  '---------- ' . $report_group['name'] . ' ----------' . "\n\n";

      foreach( $report_group['entries'] as $report_entry_key => $report_entry_data )
        $response .=  wpep_format_report_data_to_line( $report_entry_key, $report_entry_data ) . "\n";
    }

    exit( json_encode( [
      'status'   => 'ok',
      'content'  => $response,
      'lang'     => [
        'copy'    => __( "Copy to Clipboard", "wpep" ),
        'copied'  => __( "Copied to Clipboard", "wpep" )
      ]
    ] ) );
  }

  public function menu_notifications() {
    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => sprintf( __( "Current user cannot %s.", "wpep" ), WPEP_CAP_MANAGE_SETTINGS )
      ] ) );

    $total = WPEP_Controller::instance()->platformHealth->get_service_status_notification_count();

    exit( json_encode( [
      'status'  => 'ok',
      'total'   => $total,
      'map'     => [
        'admin.php?page=wpep-platform-health' => $total
      ]
    ] ) );
  }

  public function disable_legacy_comments() {
    if( !wp_verify_nonce( $_POST[ 'nonce' ], 'wpep_disable_legacy_comments' ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Invalid Nonce.", "wpep" )
      ] ) );

    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => sprintf( __( "Current user cannot %s.", "wpep" ), WPEP_CAP_MANAGE_SETTINGS )
      ] ) );

    wpep_set_setting( 'enable-course-comments', 0 );

    exit( json_encode( [
      'status'  => 'success',
    ] ) );
  }

  public function update_course_user_data() {
    $key = $_POST['data_key'];

    if( !wp_verify_nonce( $_POST[ 'nonce' ], 'wpep-user-data-' . $key ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Invalid Nonce.", "wpep" )
      ] ) );

    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS )
      && !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => sprintf( __( "Current user cannot %s or %s.", "wpep" ), WPEP_CAP_MANAGE_SETTINGS, WPEP_CAP_MANAGE_MEMBERS )
      ] ) );

    $key   = sanitize_text_field( $_POST['data_key'] );
    $value = sanitize_text_field( $_POST['data_value'] );
    $course_id = intval( $_POST['course_id'] );
    $user_id   = intval( $_POST['user_id'] );

    if( in_array( $key, [ WPEP_USER_COURSE_STARTED, WPEP_USER_COURSE_COMPLETED_AT, WPEP_USER_COURSE_LATEST_INTERACTION ] ) )
      $value = strtotime( $value );

    wpep_controller()->userActivity->set_course_user_data( $key, $value, $course_id, 0,0, $user_id );

    exit( json_encode( [
      'status'  => 'ok'
    ] ) );
  }

  public function wpep_admin_save_post_ajax_coordinator() {
    if( !current_user_can( WPEP_CAP_EDIT_COURSE )
        && !current_user_can( WPEP_CAP_EDIT_ASSESSMENT ) )
      exit( json_encode( [
          'status'  => 'error',
          'error'   => __( "Current user cannot manage_options.", "wpep" )
      ] ) );

    if( !isset( $_POST['post_id'] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Post ID has not been provided.", "wpep" )
      ] ) );

    $post_id = intval( $_POST['post_id'] );

    $entity_id_map = ( isset( $_POST['wpep_entity_id_map'] ) ? $_POST['wpep_entity_id_map'] : [] );
    $entity_id_map = apply_filters( 'wpep_admin_save_post_ajax', $entity_id_map, $post_id );

    exit( json_encode( [
      'status'          => 'ok',
      'entity_id_map'   => $entity_id_map
    ] ) );
  }

  public function wpep_admin_editor() {
    $content = ( isset( $_POST['content'] ) ? wpep_utilities_sanitize_input( $_POST['content'] ) : '' );

    if( $content == ''
        && (
        current_user_can( WPEP_CAP_EDIT_COURSE )
        || current_user_can( WPEP_CAP_EDIT_ASSESSMENT )
        || current_user_can( WPEP_CAP_MANAGE_SETTINGS )
      ) ) {
      global $wpdb;

      if( isset( $_POST['assessment_id'] ) ) {
        $assessment_type = ( isset( $_POST['assessment_type'] ) ? sanitize_title( $_POST['assessment_type'] ) : 'success' );

        $content = ( $assessment_type == 'success'
          ? get_post_meta( intval( $_POST['assessment_id'] ), 'wpep_success_message', true )
          : get_post_meta( intval( $_POST['assessment_id'] ), 'wpep_failed_message', true )
        );

        if( $content == '' && $assessment_type == 'success' )
          $content = wpep_get_setting_translated( 'assessment-success-message' );

        if( $content == '' && $assessment_type == 'failed' )
          $content = wpep_get_setting_translated( 'assessment-status-failed' );

      } else if( isset( $_POST['lesson_id'] ) ) {
        $content = $wpdb->get_col('SELECT `content` FROM ' . $wpdb->prefix . 'wpep_section_lesson WHERE id = ' . intval( $_POST['lesson_id'] ) );
      } else if( isset( $_POST['section_id'] ) ) {
        $content = $wpdb->get_col('SELECT `content` FROM ' . $wpdb->prefix . 'wpep_section WHERE id = ' . intval( $_POST['section_id'] ) );
      }

      if( is_array( $content ) )
        $content = array_shift( $content );

      if( !is_string( $content ) )
        $content = '';
    }

    $editor_args = [];

    if( isset( $_POST['editor_class'] ) )
      $editor_args['editor_class'] = sanitize_text_field( $_POST['editor_class'] );

    wpep_editor( $content, $_POST['id'], $_POST['name'], $editor_args );

    exit;
  }

  public function user_icon_list() {
    echo json_encode( wpep_get_admin_icon_selection_list() );
    exit;
  }

  public function post_list_search() {
    if( !isset( $_POST[ 'search' ] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Missing search term.", "wpep" )
      ] ) );

    if( !isset( $_POST[ 'post_type'] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Missing search post_type.", "wpep" )
      ] ) );

    $search     = sanitize_text_field( $_POST[ 'search' ] );
    $post_type  = $_POST[ 'post_type' ];

    if( !isset( wpep_content_type()->contentTypes[ $post_type ] )
        && !in_array( $post_type, [ 'page', 'post' ] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Invalid post_type provided.", "wpep" )
      ] ) );


    $post_list = wpep_get_post_list_assoc( $post_type, 'ID', 'post_title', 'post_title', [], $search );

    if( empty( $post_list ) )
      exit( json_encode( [
        'status'  => 'ok',
        'results' => []
      ] ) );

    $search_results = [];

    foreach( $post_list as $post_id => $post_title )
      $search_results[] = [
        'text'  => $post_title . ' #' . $post_id,
        'id'    => $post_id
      ];

    echo json_encode( [
      'status'  => 'ok',
      'results' => $search_results
    ] );
    exit;
  }

  public function post_learning_journey_search() {
    if( !isset( $_POST[ 'search' ] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Missing search term.", "wpep" )
      ] ) );

    if( !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => sprintf( __( "Current user cannot %s.", "wpep" ), WPEP_CAP_MANAGE_SETTINGS )
      ] ) );

    $search     = sanitize_text_field( $_POST[ 'search' ] );
    $post_types = array_keys( wpep_content_type()->get_learning_journey_instances() );

    $post_list = wpep_get_post_list( $post_types, 'post_title', [], $search );

    if( empty( $post_list ) )
      exit( json_encode( [
        'status'  => 'ok',
        'results' => []
      ] ) );

    $search_results = [];

    foreach( $post_list as $post_information ) {
      $text = '';

      if( isset( wpep_content_type()->contentTypes[ $post_information->post_type ] ) )
        $text .= wpep_content_type()->contentTypes[ $post_information->post_type ]->get_singular_name() . ' - ';

      $text .= $post_information->post_title . ' #' . $post_information->ID;

      $search_results[] = [
        'text'  => $text,
        'id'    => $post_information->ID
      ];
    }

    echo json_encode( [
      'status'  => 'ok',
      'results' => $search_results
    ] );
    exit;
  }

  public function learning_journey_date_modify_list() {
    echo json_encode( wpep_learning_journey_drip_options() );
    exit;
  }

  public function user_list_search() {
    if( !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Your account is not allowed to search trough users.", "wpep" )
      ] ) );

    if( !isset( $_POST[ 'search' ] ) || !isset( $_POST[ 'nonce' ] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Missing search term or nonce.", "wpep" )
      ] ) );

    if( wp_verify_nonce( $_POST[ 'nonce' ], 'wpep_admin_user_list_search' ) === false )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Invalid Nonce.", "wpep" )
      ] ) );

    $search     = sanitize_text_field( $_POST[ 'search' ] );

    $user_object_list = get_users( [
      'number'         => 15,
      'search'         => '*' . $search . '*',
      'search_columns' => [ 'user_login', 'user_nicename', 'user_email' ],
    ] );

    if( empty( $user_object_list ) )
      exit( json_encode( [
        'status'  => 'ok',
        'results' => []
      ] ) );

    $search_results = [];

    foreach( $user_object_list as $user_object )
      $search_results[] = [
        'text'  => wpep_content_user_profile_format_name( $user_object ) . ' #' . $user_object->ID,
        'id'    => $user_object->ID
      ];

    echo json_encode( [
      'status'  => 'ok',
      'results' => $search_results
    ] );
    exit;
  }

  public function install_store_product() {
    $store_product_id = isset( $_POST['store_product_id'] ) ? $_POST['store_product_id'] : false;

    if ( $store_product_id == false
        || ! isset( $_POST['nonce'] )
        || ! wp_verify_nonce( $_POST['nonce'], 'wpep-install-plugin_' . $store_product_id )
        || !current_user_can( 'install_plugins'))
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "An unexpected error has happened, please refresh the page and try again", "wpep" )
      ] ) );

    $addon_list = wpep_controller()->adminController->addOnsSection->get_available_addon_list();

    if( !isset( $addon_list[ $store_product_id ] ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "The provided store_id doesn't match any addon.", "wpep" )
      ] ) );

    if( !in_array( $store_product_id, wpep_controller()->adminController->license->get_license_bundled_products() ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "The Bundle License cannot be used to download this addon / plugin, please download it from the codestore.", "wpep" )
      ] ) );

    $addon_information = $addon_list[ $store_product_id ];

    unset( $addon_list ); // Cleanup Memory

    set_current_screen();

    $license_and_updates = new \WPEP_Integration_License_And_Updates(
      $addon_information->title,
      $addon_information->id,
      false,
      '0.0.0',
      false
    );

    $license_data = $license_and_updates->api->activate_license();

    if( !isset( $license_data->license ) || $license_data->license != 'valid' )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "The Bundle License cannot be validated, please download the plugin from the My Account section.", "wpep" )
      ] ) );

    $request_params = $license_and_updates->api->get_remote_request_params();
    $request_params['body'] = $license_and_updates->api->get_request_params( 'get_version' );

    $request = wp_remote_post( WPEP_STORE_LINK, $request_params );

    if ( is_wp_error( $request ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Plugin Information cannot be fetched from server, please download the plugin from the My Account section.", "wpep" )
      ] ) );

    $request = json_decode( wp_remote_retrieve_body( $request ) );

    if( !isset( $request->package ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "Plugin Package cannot be fetched from server, please download the plugin from the My Account section.", "wpep" )
      ] ) );

    if( !class_exists( '\Plugin_Upgrader' ) )
      require_once ABSPATH . 'wp-admin/includes/class-wp-upgrader.php';

    if( !class_exists( '\WPEP_Silent_Upgrader_Skin') )
      require_once WPEP_BASE_PATH . '/lib/_internal/Silent_Upgrader.php';

    try {
      ob_start();
      $installer = new \Plugin_Upgrader( new \WPEP_Silent_Upgrader_Skin() );
      $installation_response = $installer->install( $request->package );
      ob_end_clean();
      if( is_wp_error( $installation_response ) )
        exit( json_encode( [
          'status'  => 'error',
          'error'   => $installation_response->get_error_message()
        ] ) );

      wp_cache_flush();
    } catch ( \Exception $exception ) {
      exit( json_encode( [
        'status'  => 'error',
        'error'   => $exception->getMessage()
      ] ) );
    }

    // Flush the cache and return the newly installed plugin basename.
    if ( !$installer->plugin_info() )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "The installation has failed, please install the addon manually as we cannot do it for you at the moment.", "wpep" )
      ] ) );

    exit( json_encode( [
      'status'              => 'ok'
    ] ) );
  }

  public function get_plugin_status() {
    if ( ! isset( $_POST[ 'plugin_alias' ] ) || !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "An unexpected error has happened, please refresh the page and try again", "wpep" )
      ] ) );

    $addon = wpep_controller()->addOns->get_add_on( $_POST[ 'plugin_alias' ] );

    $response = [];

    $response['available_operation_html'] = wpep_controller()->adminController->addOnsSection->get_addon_status_operations(
      sanitize_text_field( $_POST[ 'plugin_alias' ] ),
      $addon
    );

    $response['status_html'] = wpep_controller()->adminController->addOnsSection->get_addon_status_text( $addon );

    exit( json_encode( $response ) );
  }

  public function activate_plugin() {
    $plugin_identifier = isset( $_POST['plugin'] ) ? $_POST['plugin'] : false;

    if ( $plugin_identifier == false
          || ! isset( $_POST['nonce'] )
          || ! wp_verify_nonce( $_POST['nonce'], 'activate-plugin_' . $plugin_identifier )
          || !current_user_can( 'activate_plugins') )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "An unexpected error has happened, please refresh the page and try again", "wpep" )
      ] ) );

    $activate = activate_plugin( $plugin_identifier );

    if ( is_wp_error( $activate ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => $activate->get_error_message()
      ] ) );

    exit( json_encode( [
      'status'              => 'ok'
    ] ) );
  }

  public function deactivate_plugin() {
    $plugin_identifier = isset( $_POST['plugin'] ) ? $_POST['plugin'] : false;

    if ( $plugin_identifier == false
        || ! isset( $_POST['nonce'] )
        || ! wp_verify_nonce( $_POST['nonce'], 'deactivate-plugin_' . $plugin_identifier )
        || !current_user_can( 'activate_plugins') )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => __( "An unexpected error has happened, please refresh the page and try again", "wpep" )
      ] ) );

    $deactivate = deactivate_plugins( $plugin_identifier );

    if ( is_wp_error( $deactivate ) )
      exit( json_encode( [
        'status'  => 'error',
        'error'   => $deactivate->get_error_message()
      ] ) );

    exit( json_encode( [
      'status'              => 'ok'
    ] ) );
  }

}