<?php

namespace WPEP\Coordinator\Authentication;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class Request {

  public function __construct() {
    add_action('wp_ajax_wpep_auth_request', [ $this, 'ajax_request_logged' ] );
    add_action('wp_ajax_nopriv_wpep_auth_request', [ $this, 'ajax_request' ] );
  }

  public function ajax_request() {
    if( !isset( $_POST['auth_information'] ) || !isset( $_POST['auth_type'] ) )
      exit ( json_encode( [
        'status'              => 'error',
        'notification'        => __( "Invalid Request, the page will reload", "wpep" ),
        'notification_class'  => 'wpep-notification wpep-notification-warning',
        'refresh'             => 1
      ] ) );

    $auth_type        = $_POST['auth_type'];
    $auth_information = $_POST['auth_information'];

    $allow_registration = ( wpep_get_setting_flag( 'user-auth-enable-registration' ) || wpep_get_setting_flag( 'enable-post-member-magnet-option' ) );
    $allow_registration = apply_filters( 'wpep_auth_allow_registration', $allow_registration );

    if( !isset( $auth_information['wpep_auth_nonce'] ) || wp_verify_nonce( $auth_information['wpep_auth_nonce'], 'wpep_auth_nonce' ) == false )
      exit ( json_encode( [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-auth-notification-page-expired' ),
        'notification_class'  => 'wpep-notification wpep-notification-warning',
        'refresh'             => 1
      ] ) );

    if( $auth_type == 'login' ) {
      if( $auth_information['login_identifier'] == '' || $auth_information['password'] == '' )
        exit ( $this->_auth_request_response( [
          'status'              => 'error',
          'notification'        => wpep_get_setting_translated( 'user-auth-notification-login-details-required' ),
          'notification_class'  => 'wpep-notification'
        ], $auth_information, $auth_type ) );

      $user = wp_signon( [
        'user_login'    => $auth_information['login_identifier'],
        'user_password' => $auth_information['password'],
        'remember'      => true
      ], is_ssl() );

      if ( is_wp_error( $user ) ) {
        exit ( $this->_auth_request_response( [
          'status'              => 'error',
          'notification'        => ( isset( $user->errors['incorrect_password'] )
            ? ( wpep_get_setting_translated( "user-auth-notification-password-incorrect" ) . ' <span class="reset-password-text" data-field-type-switch="reset_password">' . wpep_get_setting_translated( 'user-auth-open-tab-reset-password' ) . '</span>' )
            : $user->get_error_message()
          ),
          'notification_class'  => 'wpep-notification wpep-notification-warning'
        ], $auth_information, $auth_type ) );
      } else {
        wp_set_current_user( $user->ID );

        exit ( $this->_auth_request_response( [
          'status'              => 'ok',
          'notification'        => wpep_get_setting_translated( 'user-auth-notification-logged-in' ),
          'notification_class'  => 'wpep-notification wpep-notification-success',
          'refresh'             => 1,
          'refresh_delay'       => 1000
        ], $auth_information, $auth_type ) );
      }

    } else if( $auth_type == 'register' && $allow_registration ) {
      if( wpep_get_setting_flag( 'user-service-agreement-enabled' )
            && ( !isset( $auth_information['service_agreement'] ) || intval( $auth_information['service_agreement'] ) != 1 ) ) {
        exit ( $this->_auth_request_response( [
          'status'              => 'error',
          'notification'        => wpep_get_setting( 'user-service-agreement-error' ),
          'notification_class'  => 'wpep-notification wpep-notification-danger'
        ], $auth_information, $auth_type ) );
      }

      if( wpep_get_setting_flag( 'user-privacy-agreement-enabled' )
            && ( !isset( $auth_information['privacy_agreement'] ) || intval( $auth_information['privacy_agreement'] ) != 1 ) ) {
        exit ( $this->_auth_request_response( [
          'status'              => 'error',
          'notification'        => wpep_get_setting( 'user-privacy-agreement-error' ),
          'notification_class'  => 'wpep-notification wpep-notification-danger'
        ], $auth_information, $auth_type ) );
      }

      $user_id = wp_create_user( $auth_information['username'], $auth_information['password'], $auth_information['email_address'] );

      if( is_wp_error( $user_id ) ) {
        exit ( $this->_auth_request_response( [
          'status'              => 'error',
          'notification'        => $user_id->get_error_message(),
          'notification_class'  => 'wpep-notification wpep-notification-warning'
        ], $auth_information, $auth_type ) );
      } else {
        $first_name = ( isset( $auth_information['first_name'] ) ? sanitize_text_field( $auth_information['first_name'] ) : '' );
        $last_name  = ( isset( $auth_information['last_name'] ) ? sanitize_text_field( $auth_information['last_name'] ) : '' );

        if( $first_name !== '' && $last_name !== '' ) {
          wp_update_user( [
            'ID'           => $user_id,
            'first_name'   => $first_name,
            'last_name'    => $last_name,
            'display_name' => ( $first_name == '' && $last_name == '' ? $auth_information['username'] : trim( $first_name . ' ' . $last_name ) )
          ] );
        }

        if( wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) )
          update_user_meta( $user_id, WPEP_USER_META_AGREEMENT_PRIVACY_DATE, date( "Y-m-d" ) );
        if( wpep_get_setting_flag( 'user-service-agreement-enabled' ) )
          update_user_meta( $user_id, WPEP_USER_META_AGREEMENT_TOS_DATE, date( "Y-m-d" ) );

        $user = get_user_by( 'id', $user_id );

        wp_set_current_user( $user_id );
        wp_set_auth_cookie( $user_id );
        do_action( 'wp_login', $user->user_login, $user );

        exit ( $this->_auth_request_response( [
          'status'              => 'ok',
          'notification'        => wpep_get_setting_translated( 'user-auth-notification-account-registered' ),
          'notification_class'  => 'wpep-notification wpep-notification-success',
          'refresh'             => 1,
          'refresh_delay'       => 2000
        ], $auth_information, $auth_type ) );
      }
    } else if( $auth_type == 'reset_password' ) {
      $response = wpep_retrieve_password( $auth_information['login_identifier'] );

      if( is_wp_error( $response ) ) {
        exit ( $this->_auth_request_response( [
          'status'              => 'error',
          'notification'        => $response->get_error_message(),
          'notification_class'  => 'wpep-notification wpep-notification-warning'
        ], $auth_information, $auth_type ) );
      } else {
        exit ( json_encode( [
          'status'              => 'ok',
          'notification'        => wpep_get_setting_translated( 'user-auth-notification-password-reset' ),
          'notification_class'  => 'wpep-notification wpep-notification-success',
          'refresh'             => 1,
          'refresh_delay'       => 5000
        ] ) );
      }
    }

    exit ( $this->_auth_request_response( [
      'status'              => 'error',
      'notification'        => __( "An unexpected error has just occured, please try again in a few moments.", "wpep" ),
      'notification_class'  => 'wpep-notification wpep-notification-danger',
      'refresh'             => 1
    ], $auth_information, $auth_type ) );
  }

  private function _auth_request_response( $response, $auth_information, $auth_type ) {
    if( !isset( $response['status'] ) || $response['status'] != 'ok' )
      return json_encode( $response );

    $post_id = ( isset( $auth_information[ 'wpep_post_id'] ) ? intval( $auth_information[ 'wpep_post_id'] ) : 0 );
    $lead_segment = ( isset( $auth_information[ 'wpep_lead_segment'] ) ? $auth_information[ 'wpep_lead_segment'] : ( $post_id !== 0 ? 'lead_magnet' : 'registration_form' ) );

    if( $auth_type == 'register' ) {
      WPEP_Entity_ActionTracking::instance()->track_event( $post_id, get_current_user_id(), WPEP_ACTION_TRACKING_TYPE_REGISTRATION, $lead_segment );

      if( $post_id !== 0 && wpep_post_has_member_magnet_option( $post_id ) ) {
        $custom_redirect = trim( get_post_meta( $post_id, 'wpep_member_magnet_option_registration_redirect_url', true ) );

        if( $custom_redirect !== '' ) {
          wpep_user_course_started_event( $post_id, get_current_user_id() );

          $response = [
            'status'       => 'ok',
            'redirect'     => $custom_redirect,
            'redirect_top' => 1
          ];
        }
      }
    } elseif( $auth_type === 'reset_password' ) {
      WPEP_Entity_ActionTracking::instance()->track_event( $post_id, get_current_user_id(), WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, $lead_segment );
    } elseif( $auth_type === 'login' ) {
      WPEP_Entity_ActionTracking::instance()->track_event( $post_id, get_current_user_id(), WPEP_ACTION_TRACKING_TYPE_LOGIN, $lead_segment );
    }

    if( isset( $auth_information[ 'success_response_type' ] )
      && $auth_information[ 'success_response_type' ] !== ''
      && in_array( $auth_type, [ 'login', 'register' ] ) ) {
      if( $auth_information[ 'success_response_type' ] == 'redirect' && isset( $auth_information[ 'success_response_redirect' ] ) ) {
        $response = [
          'status'       => 'ok',
          'redirect'     => $auth_information[ 'success_response_redirect' ]
        ];
      } elseif( $auth_information[ 'success_response_type' ] == 'redirect-to-public-profile' ) {
        $response = [
          'status'       => 'ok',
          'redirect'     => wpep_user_profile_public_link()
        ];
      }

    }

    return json_encode( apply_filters( "wpep_auth_request_response_json", $response, $auth_information, $auth_type ) );
  }

  public function ajax_request_logged() {
    exit ( json_encode( [
      'notification'        => wpep_get_setting_translated( 'user-auth-notification-already-logged-in' ),
      'notification_class'  => 'wpep-notification wpep-notification-success',
      'refresh'             => 1,
      'refresh_delay'       => 2000
    ] ) );
  }


}