<?php

namespace WPEP\Coordinator\Content;

use WPEP\Controller as WPEP_Controller;

class Search {

  public function __construct() {
    add_shortcode( 'wpep_search', [ $this, '_display_form_from_attributes' ] );
    add_action( 'widgets_init', [ $this, '_register_widget' ], 10 );

    add_action('wp_ajax_wpep_search', [ $this, '_display_search_results' ] );
    add_action('wp_ajax_nopriv_wpep_search', [ $this, '_display_search_results' ] );
  }

  public function _display_form_from_attributes( $attributes ) {
    return WPEP_Controller::instance()->template->get_template( "search/form.php", [
      'post_type_list'     => ( isset( $attributes['post_type'] )          ? $attributes['post_type']          : '' ),
      'include_categories' => ( isset( $attributes['include_categories'] ) ? $attributes['include_categories'] : '' ),
      'exclude_categories' => ( isset( $attributes['exclude_categories'] ) ? $attributes['exclude_categories'] : '' )
    ] );
  }

  public function _register_widget() {
    register_widget('\WPEP\Coordinator\Content\Search\Widget' );
  }

  public function _display_search_results() {
    if( !isset( $_POST[ 'wpep_search_term' ] ))
      exit( json_encode( [
        'status'    => 'ok',
        'response'  => '<p class="wpep-notification wpep-notification-danger">' . __( "Search term not included", "wpep" ) . '</p>'
      ] ) );

    // Post Types & Search Term are sanitized by the final function.

    exit( json_encode( [
      'status'    => 'ok',
      'response'  => WPEP_Controller::instance()->template->get_template(
        'search/results.php', [
          'search_result_list' => $this->get_as_array(
            $_POST['wpep_search_term'],
            ( isset( $_POST['wpep_search_post_types'] ) ? $_POST['wpep_search_post_types'] : [] ),
            wpep_get_setting( 'search-results-number' ),
            [
              'include_categories'  => ( isset( $_POST['wpep_search_include_categories'] ) ? $_POST['wpep_search_include_categories'] : '' ),
              'exclude_categories'  => ( isset( $_POST['wpep_search_exclude_categories'] ) ? $_POST['wpep_search_exclude_categories'] : '' )
            ]
          )
        ]
      )
    ] ) );
  }

  /**
   * @param $keyword
   * @param array $post_types
   * @param int $result_entries_count
   * @param array $search_rules
   * @return array
   */
  public function get_as_array( $keyword, $post_types = [], $result_entries_count = 10, $search_rules = [] ) {
    $post_list = $this->get_as_posts( $keyword, $post_types, $result_entries_count, $search_rules );
    $response  = [];

    foreach( $post_list as $post ) {
      $response[ $post->ID ] = [
        'ID'        => $post->ID,
        'post_type' => $post->post_type,
        'title'     => $post->post_title,
        'thumbnail' => wpep_get_placeholder_image(),
        'permalink' => false,
        'thumbnail_ribbon_text' => '',
      ];

      $response[ $post->ID ] = wpep_content_type()->library_get_search_details( $response[ $post->ID ], $post );
    }

    return $response;
  }

  /**
   * @param $keyword
   * @param array $post_types
   * @param int $result_entries_count
   * @param array $search_rules
   * @return array
   */
  public function get_as_posts( $search, $post_types = [], $result_entries_count = 10, $search_rules = [] ) {
    if( is_string( $post_types ) && $post_types !== '' )
      $post_types = explode( ",", $post_types );

    if( !is_array( $post_types ) || empty( $post_types ) ) {
      $post_types = wpep_content_type()->library_post_types();
    } else {
      foreach( $post_types as $k => $post_type )
        if( !in_array( $post_type, wpep_content_type()->library_post_types() ) )
          unset( $post_types[ $k ] );
    }

    $search = sanitize_text_field( $search );

    $query_args = [
      'post_type'   => $post_types,
      'post_status' => 'publish',
      'meta_query'  => [
        'wpep_search' => [
          'relation' => 'OR',
          [
            'key'     => 'wpep_show_on_index',
            'value'   => '1',
            'compare' => '=',
          ],
          [
            'key'     => 'wpep_show_on_index',
            'compare' => 'NOT EXISTS'
          ]
        ]
      ],
      'paged'          => false,
      'posts_per_page' => $result_entries_count
    ];

    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      $query_args['post_status'] = wpep_post_monitored_post_status_list();

    if( isset( $search_rules['include_categories'] ) && $search_rules['include_categories'] !== '' ) {
      $include_categories = $search_rules['include_categories'];

      if( !is_array( $include_categories ) )
        $include_categories = explode( ",", $include_categories );

      $include_categories = array_map( 'absint', $include_categories );

      $query_args['tax_query'] = [
        'wpep_search_include_categories'  => [
          'taxonomy'         => WPEP_TAXONOMY_CATEGORY,
          'field'            => 'id',
          'terms'            => $include_categories,
          'operator'         => 'IN'
        ]
      ];
    } else if( isset( $search_rules['exclude_categories'] ) && $search_rules['exclude_categories'] !== '' ) {
      $exclude_categories = $search_rules['exclude_categories'];

      if( !is_array( $exclude_categories ) )
        $exclude_categories = explode( ",", $exclude_categories );

      $exclude_categories = array_map( 'absint', $exclude_categories );

      $query_args['tax_query'] = [
        'wpep_search_exclude_categories'  => [
          'taxonomy'         => WPEP_TAXONOMY_CATEGORY,
          'field'            => 'id',
          'terms'            => $exclude_categories,
          'operator'         => 'NOT IN'
        ]
      ];
    }

    if( $search != '' ) {
      $query_args['s'] = $search;
      unset( $query_args['orderby'] );
      unset( $query_args['order'] );
    }

    return get_posts( $query_args );
  }

}
