<?php

namespace WPEP\Coordinator\Content\UserProfile;

use WPEP\Controller as WPEP_Controller;
use WPEP\Container\FilterProxy as FilterProxy;

class Delivery {

  use FilterProxy;

  public function __construct() {
    add_shortcode( 'wpep_profile', [ $this, 'primary' ] );
    add_shortcode( 'wpep_public_profile_link', [ $this, 'public_profile_link'] );

    if( wpep_get_setting_flag( 'user-profile-custom-tabs-enabled' ) )
      add_filter( 'wpep_content_user_profile_tab_list', [ $this, '_custom_profile_tabs' ], 150, 2 );

    if( wpep_controller()->request->is_request_type( 'admin' ) )
      return;

    add_action( 'init', [ $this, '_init' ] );
  }

  public function get_tab_list( $user_id = false ) {
    $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

    if( $user_id == 0 && !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return [];

    $tab_list = [];

    if( wpep_get_setting_flag( 'user-profile-tab-my-content-enabled' )
        && wpep_controller()->contentLibrary->is_integrated()
        && wpep_user_profile_current_user_can_view( $user_id ) ) {
      $tab_list[ 'my_content' ] = [
        'title'   => wpep_get_setting_translated( 'user-profile-tab-my-content' ),
        'content' => '[wpep_index display_header="0" categories_per_row="1" category_navigation="0" display_restriction="access" pagination_per_page="0"]'
      ];
    }

    $tab_list = apply_filters( 'wpep_content_user_profile_tab_list', $tab_list, $user_id );

    if( wpep_get_setting_flag( 'enable-student-supervisor' ) ) {
      if( user_can( $user_id, WPEP_CAP_SUPERVISE_STUDENTS )
            && ( $user_id == get_current_user_id() || current_user_can( WPEP_CAP_MANAGE_MEMBERS ) ) )
        $tab_list['supervised_user_list'] = $this->_supervised_user_list_tab( $user_id );
      elseif( current_user_can(WPEP_CAP_MANAGE_SETTINGS ) )
        $tab_list['supervised_user_list'] = [
          'title'   => wpep_get_setting( 'user-profile-tab-my-students' ),
          'content' => '<div class="wpep-notification wpep-notification-info">' .
                          __( "Admin Notification : Student Supervisors will see their students in this tab, users that don't have the Supervisor Role will not see this tab.", "wpep" ) .
                       '</div>'
        ];
    }

    $order_list = wpep_get_setting( 'user-profile-tab-ordering' );

    if( empty( $order_list ) )
      return $tab_list;

    $response = [];

    foreach( $order_list as $current_tab_index ) {
      if( !isset( $tab_list[$current_tab_index] ) )
        continue;

      $response[ $current_tab_index ] = $tab_list[ $current_tab_index ];

      unset( $tab_list[ $current_tab_index ] );
    }

    return $tab_list + $response;
  }

  public function primary($attributes, $content = null) {
    $user_information = WPEP_Controller::instance()->content->userProfile->get_display_user();

    if( $user_information == false )
      return WPEP_Controller::instance()->authentication->get_form( $attributes );

    WPEP_Controller::instance()->templateStyle->match_request_layout_options( $attributes );

    return WPEP_Controller::instance()->template->get_template(
      'user-profile.php', [
        'user_information' => $user_information,
        'include_header'   => false,
        'include_footer'   => false
      ]
    );
  }

  public function public_profile_link( $attributes, $content = '' ) {
    $content = ( is_string( $content ) ? $content : '' );
    $link    = wpep_user_profile_public_link( get_current_user_id() );

    if( $link === false )
      return '';

    if( $content !== '' )
      return '<a href="' . $link . '">' . $content . '</a>';

    return $link;
  }

  public function _init() {
    $current_post_id = wpep_get_current_post_id();

    if( $current_post_id != 0 ) {
      $post            = get_post($current_post_id);

      if( strpos( $post->post_content, '[wpep_profile' ) !== false && get_current_user_id() != 0 )
        WPEP_Controller::instance()->assets->has_enqueue_media_support = true;
    }
  }

  public function _custom_profile_tabs( $tabs, $user_id ) {
    $custom_tabs = wpep_get_setting( 'user-profile-custom-tabs' );

    if( !is_array( $custom_tabs ) )
      return $tabs;

    if( empty( $custom_tabs ) )
      return $tabs;

    if( !wpep_user_profile_current_user_can_view( $user_id ) )
      return $tabs;

    $current_tabs = [];

    foreach( $custom_tabs as $tab_title => $tab_content ) {
      $current_tabs[ 'custom_' . sanitize_title( $tab_title ) ] = [
        'title'         => $tab_title,
        'content'       => wpep_format_content( $tab_content )
      ];
    }

    return (
    wpep_get_setting( 'user-profile-custom-tabs-position' ) === 'left' ?
      $current_tabs + $tabs
      : $tabs + $current_tabs
    );
  }

  public function _supervised_user_list_tab( $user_id ) {
    $response = [
      'title'   => wpep_get_setting( 'user-profile-tab-my-students' ),
      'content' => ''
    ];

    $supervised_users = get_users( [
      'meta_key'    => WPEP_USER_META_SUPERVISOR_USER_ID,
      'meta_value'  => $user_id,
      'meta_compare'=> '=',
      'number'      => -1
    ] );

    if( empty( $supervised_users ) ) {
      $response['content'] = wpep_format_content( wpep_get_setting( 'user-profile-tab-my-students-content-pre' ) );
    } else {
      $response['content'] = wpep_template_get( 'partial/user-profile-supervisor-tab.php', [
        'user_id'   => $user_id,
        'user_list' => $supervised_users
      ] );
    }

    return $response;
  }

}