<?php

namespace WPEP\Coordinator\Content\UserProfile;

use WPEP\Controller as WPEP_Controller;

class PublicProfile {

  public $pagename_identifier = 'wpep-user-profile';
  public $user_identifier     = 'wpep-user-profile';

  private $_user_setup = false;

  public function __construct() {
    if( !wpep_public_profiles_enabled() )
      return;

    $user_slug = wpep_get_setting( 'user-profile-public-slug' );

    if( $user_slug !== '' )
      $this->user_identifier = $user_slug;

    add_action( 'init', [ $this, '_add_rewrite_rules' ] );
    add_filter( 'query_vars', [ $this, '_add_query_vars' ] );
    add_filter( 'status_header', [ $this, '_status_header' ], 50 );
    add_filter( 'body_class', [ $this, '_body_class' ], 50 );

    add_filter( 'wpep_current_request_template', [ $this, '_wpep_current_request_template' ] );
  }

  public function _add_query_vars( $query_vars ) {
    $query_vars[] = $this->user_identifier;

    return $query_vars;
  }

  public function _add_rewrite_rules() {
    add_rewrite_rule(
      $this->user_identifier . '/([^/]+)/?',
      'index.php?pagename=' . $this->pagename_identifier . '&' . $this->user_identifier . '=$matches[1]',
      'top'
    );
  }

  public function _status_header( $response ) {
    if( !$this->_is_valid_public_profile_request() )
      return $response;

    return 'HTTP/1.1 200 OK';
  }

  public function _body_class( $class ) {
    $class = array_diff( $class, [ 'error404', 'error-404' ] );

    $class[] = 'wpep-public-user-profile';

    return $class;
  }

  private function _profile_coordinator() {
    return wpep_controller()->content->userProfile;
  }

  public function _wpep_current_request_template( $template ) {
    if( !$this->_is_valid_public_profile_request() )
      return $template;

    add_filter( 'wp_title', [ $this, 'wp_title' ], 50 );
    add_filter( 'pre_get_document_title', [ $this, 'wp_title' ], 50 );
    add_action( 'wp_head', [ $this, 'html_head_tags' ], 5 );

    if( isset( $_GET[ 'wpep-login' ] ) ) {
      add_filter( 'wpep_user_profile_authentication_form_view', [ $this, '_authentication_form_view' ], 200 );
    }

    // Remove Yoast
    if( class_exists( '\WPSEO_Frontend' ) ) {
      $wp_thing = \WPSEO_Frontend::get_instance();
      remove_action('wp_head',array($wp_thing,'head'),1);
    }

    return 'user-profile.php';
  }

  public function wp_title( $title ) {
    return $this->_get_title();
  }

  private function _get_title() {
    return sprintf( wpep_get_setting_translated( 'user-profile-heading' ), wpep_content_user_profile_format_name( $this->_profile_coordinator()->get_display_user() ) );
  }

  public function _authentication_form_view() {
    return 'login';
  }

  public function html_head_tags() {
    $current_user = $this->_profile_coordinator()->get_display_user();

    if( $current_user == false )
      return;

    $description = wpep_user_profile_description_excerpt( $current_user->ID );

    echo '<meta name="description" content="' . $description . '">' . "\n";
    echo '<meta property="og:url"         content="' . wpep_user_profile_public_link( $current_user ) . '" />' . "\n";
    echo '<meta property="og:type"        content="profile" />' . "\n";
    echo '<meta property="og:title"       content="' . $this->_get_title() . '" />' . "\n";
    echo '<meta property="og:description" content="' . $description . '" />' . "\n";
    echo '<meta property="og:image"       content="' . wpep_user_profile_picture_url( $current_user->ID ). '" />' . "\n";
  }

  private function _set_current_request_user() {
    $user = get_user_by( 'login', $this->get_profile_username() );

    if( $user == false )
      $user = get_user_by( 'email', $this->get_profile_username() );

    if( $user == false )
      $user = get_user_by( 'slug', $this->get_profile_username() );

    if( $user == false ) {
      $this->_profile_coordinator()->set_display_user( false );
      return;
    }

    if( wpep_user_profile_is_public( $user->ID ) ) {
      $this->_profile_coordinator()->set_display_user( $user );
      return;
    }

    if( wpep_user_profile_current_user_can_view( $user->ID ) ) {
      $this->_profile_coordinator()->set_display_user( $user );
      return;
    }

    $this->_profile_coordinator()->set_display_user( false );
  }

  private function get_profile_username() {
    return urldecode( get_query_var( $this->user_identifier ) );
  }

  private function _is_valid_public_profile_request() {
    global $wp_query;

    if( $wp_query === null )
      return false;

    if( get_query_var( 'pagename' ) != $this->pagename_identifier )
      return false;

    if( $this->get_profile_username() == '' )
      return false;

    if( !$this->_user_setup ) {
      $this->_set_current_request_user();
      $this->_user_setup = true;
    }

    if( $this->_profile_coordinator()->get_display_user() == false )
      return false;

    return true;
  }

}