<?php

namespace WPEP\Coordinator;

use WPEP\Container\DataWalker;

class DataSystem {

  use DataWalker;

  private $_storage = [];

  /**
   * @var DataSystem\UserActivity
   */
  public $userActivity;

  public function __construct() {
    $this->userActivity = new DataSystem\UserActivity();
  }

  public function setup() {
    add_action( 'wpep_cron_daily', [ $this, 'wpep_cron_daily' ] );
  }

  public function wpep_cron_daily() {
    if( wpep_get_setting( 'data-system-enable-background-backups' ) == false )
      return;

    ignore_user_abort(false);

    $this->create_backup();

    $vault_item_expiration_days = intval( wpep_get_setting( 'data-system-backups-cap' ) );

    if( $vault_item_expiration_days == 0 )
      return;

    $vault_items = $this->get_backup_list();
    $allow_deletion = false;

    foreach( $vault_items as $vault_item ) {
      $vault_age_days = ( ( time() - strtotime( $vault_item->post_date ) ) / ( 60 * 60 * 24 ) );
      if( $vault_age_days <= $vault_item_expiration_days ) {
        $allow_deletion = true;
        continue;
      }
      if( !$allow_deletion )
        continue;
      wp_delete_attachment( $vault_item->ID );
    }

  }

  public function data_import_array( $content_array ) {
    if( isset( $content_array['settings' ] ) )
      $this->update_settings( $content_array['settings'] );

    if( isset( $content_array['users'] ) )
      $this->import_users( $content_array['users'] );

    if( isset( $content_array['taxonomies' ] ) ) {
      $content_array += ['taxonomies'];

      unset( $content_array['taxonomies'] );
    }

    $id_mapping = [];

    $id_mapping += wpep_content_taxonomy()->data_import_array( $content_array, $id_mapping );
    $post_id_mapping = wpep_content_type()->data_import_array( $content_array, $id_mapping );

    $id_mapping += $post_id_mapping;

    $post_id_mapping_plain = [];

    foreach( $post_id_mapping as $post_type => $post_ids )
      $post_id_mapping_plain += $post_ids;

    wpep_content_type()->data_import_array_post_connection( $content_array, $post_id_mapping_plain );

    $content_array = apply_filters( 'wpep_data_system_import_array', $content_array, $id_mapping, $post_id_mapping_plain );

    return $content_array;
  }

  public function create_backup() {
    if( !class_exists( '\ZipArchive' ) )
      return false;

    set_time_limit( 1200 );
    @ini_set('memory_limit', '2048M');

    $current_vault_file_path = $this->data_export_array_to_zip( $this->data_export_array() );

    $attachment = [
      'post_mime_type' => 'application/zip',
      'guid'           => $current_vault_file_path,
      'post_parent'    => 0,
      'post_title'     => sprintf( __( "WPEP Platform Backup %s", "wpep" ), wpep_format_date_with_time() ),
    ];

    $new_attachment_id = wp_insert_attachment( $attachment, '/vault-' . time() . '.zip', 0 );
    update_post_meta( $new_attachment_id, '_wp_attached_file', $current_vault_file_path );
    update_post_meta( $new_attachment_id, '_wpep_exported', wpep_format_date_with_time() );

    if( !function_exists( 'wp_generate_attachment_metadata' ) )
      require ( ABSPATH . 'wp-admin/includes/image.php' );

    $attach_data = wp_generate_attachment_metadata( $new_attachment_id, get_attached_file( $new_attachment_id ) );

    wp_update_attachment_metadata( $new_attachment_id,  $attach_data );

    return true;
  }

  /**
   * @param $data_export_array
   * @param string $prefix
   * @return string $current_vault_file_path
   */
  public function data_export_array_to_zip( $data_export_array, $prefix = 'vault-' ) {
    $current_vault_destination = wpep_get_upload_dir() . '/backups';
    $current_vault_file_path = $current_vault_destination . '/' . $prefix . time() . '.zip';

    if( !is_dir( $current_vault_destination ) ) {
      mkdir($current_vault_destination, 0755, true);

      file_put_contents(
        $current_vault_destination . '/.htaccess',
        "Options -Indexes" . "\n" .
        "deny from all" . "\n" .
        "<FilesMatch '\.(jpg|jpeg|png|gif|mp3|ogg)$'>" . "\n" .
        "Order Allow,Deny" . "\n" .
        "Allow from all" . "\n" .
        "</FilesMatch>"
      );
    }
    
    $zip_archive = new \ZipArchive();
    $zip_archive->open( $current_vault_file_path, \ZipArchive::CREATE );

    $zip_archive->addFromString( 'data.json', json_encode( $data_export_array, JSON_PRETTY_PRINT ) );

    $attachment_list = $this->data_walker_get_assets( $data_export_array, $this->data_asset_map() );

    if( !empty( $attachment_list ) && is_array( $attachment_list ) ) {
      $attachment_meta_data_list = [];

      foreach( $attachment_list as $attachment_id => $attachment_url ) {
        if( !is_numeric( $attachment_id ) )
          continue;

        foreach( get_post_meta( $attachment_id ) as $attachment_post_meta_key => $attachment_post_meta_value ) {
          if( strpos( $attachment_post_meta_key, 'wpep_' ) !== 0 )
            continue;

          if( !isset( $attachment_meta_data_list[$attachment_id] ) || !is_array( $attachment_meta_data_list[$attachment_id] ) )
            $attachment_meta_data_list[$attachment_id] = [];

          $attachment_meta_data_list[ $attachment_id ][ $attachment_post_meta_key ] = $attachment_post_meta_value;
        }
      }

      $zip_archive->addFromString(
        'attachments.json',
        json_encode( [
          'base_url' => get_site_url(),
          'map'      => $attachment_list,
          'meta_map' => $attachment_meta_data_list
        ], JSON_PRETTY_PRINT ) );

      foreach( $attachment_list as $attachment_id => $attachment_url ) {
        $attached_file = get_attached_file( $attachment_id );
        $attached_file_content = @file_get_contents( ( $attached_file == false ? $attachment_url : $attached_file ) );

        if( $attached_file_content == false )
          continue;

        $zip_archive->addFromString(
          $attachment_id . ( substr( $attachment_url, strrpos( $attachment_url, '.', -1 ), strlen( $attachment_url ) ) ),
          $attached_file_content
        );
      }
    }

    $zip_archive->close();

    return $current_vault_file_path;
  }

  public function get_backup_list() {
    return get_posts( [
      'post_type'   => 'attachment',
      'meta_key'    => '_wpep_exported',
      'meta_compare'=> 'EXISTS',
      'numberposts' => -1,
      'orderby'     => 'date',
      'order'       => 'DESC'
    ] );
  }

  public function data_export_array() {
    $response = [];

    $response += wpep_content_taxonomy()->data_export_array();
    $response += wpep_content_type()->data_export_array();

    $response['users'] = $this->get_users();
    $response['settings'] = $this->get_settings();

    $response = apply_filters( 'wpep_data_system_export_array', $response );

    return $response;
  }

  public function data_asset_map() {
    $response = [];

    $response += wpep_content_taxonomy()->data_asset_map();
    $response += wpep_content_type()->data_asset_map();

    return $response;
  }

  public function get_users( $refresh = false ) {
    if( isset( $this->_storage['users'] ) && $refresh == false )
      return $this->_storage['users'];

    $response = [];

    $users = get_users( [ 'paged' => -1, 'orderby' => 'ID' ] );

    foreach( $users as $user ) {
      $response[ $user->ID ] = [
        'id'            => $user->ID,
        'username'      => $user->user_login,
        'email_address' => $user->user_email,
        'first_name'    => $user->first_name,
        'last_name'     => $user->last_name,
        'user_status'   => $user->user_status,
        'description'   => get_the_author_meta( 'description', $user->ID )
      ];
    }

    $this->_storage['users'] = $response;

    return $this->_storage['users'];
  }

  public function get_user_ids_mapped_by_email( $refresh = false ) {
    if( isset( $this->_storage['user_email_map'] ) && $refresh == false )
      return $this->_storage['user_email_map'];

    $this->_storage['user_email_map'] = wpep_utility_map_to_array_assoc( $this->get_users(), 'email_address', 'id' );

    return $this->_storage['user_email_map'];
  }

  public function get_settings() {
    global $wpdb;

    $response = [];

    $option_list = $wpdb->get_col( 'SELECT option_name FROM ' . $wpdb->options . ' WHERE option_name like "wpep_%"' );

    foreach( $option_list as $option_key )
      $response[$option_key] = get_option( $option_key, '' );

    unset( $response['wpep_migration_version'] );

    return $response;
  }

  public function update_settings( $settings ) {
    foreach( $settings as $option_key => $option_value )
      update_option( $option_key, $option_value );
  }

  public function import_users( $import_users ) {
    $current_users = wpep_utility_map_to_array_assoc( $this->get_users(), 'username', 'email_address' );

    foreach( $import_users as $import_user_id => $import_user ) {
      if( in_array( $import_user['email_address'], $current_users ) )
        continue;

      if( isset( $current_users->$import_user['username'] ) )
        $import_user['username'] = $import_user['email_address'];

      if( isset( $current_users->$import_user['username'] ) )
        continue;

      $user_id = wp_create_user( $import_user['username'], wp_generate_password(), $import_user['email_address'] );

      wp_update_user( [
        'ID'          => $user_id,
        'first_name'  => $import_user['first_name'],
        'last_name'   => $import_user['last_name'],
        'description' => $import_user['description']
      ] );

      $this->_storage['user'][ $import_user_id ] = [
        'id'            => $user_id,
        'username'      => $import_user['username'],
        'email_address' => $import_user['email_address'],
        'first_name'    => $import_user['first_name'],
        'last_name'     => $import_user['last_name'],
        'user_status'   => 'active',
        'description'   => $import_user['description']
      ];
    }
  }

}