<?php

namespace WPEP\Coordinator\DataSystem;

use WPEP\Controller as WPEP_Controller;
use WPEP\Entity\ActionTracking as WPEP_Entity_ActionTracking;

class UserActivity {

  private $_chart_map_by_time_slots_data = [];

  /**
   * @param $time_slots
   * @return array|null
   */
  public function get_chart_map_by_time_slots( $time_slots ) {
    return $this->_chart_map_by_time_slots( $time_slots, [ 'content_types', 'user_registration', 'authentication', 'emails' ] );
  }

  /**
   * @param $time_slots
   * @return array|null
   */
  public function get_user_activity_chart_map_by_time_slots( $time_slots ) {
    return $this->_chart_map_by_time_slots( $time_slots, [ 'user_registration', 'authentication', 'authentication_by_segment', 'authentication_by_post_id' ] );
  }

  private function _chart_map_by_time_slots( $time_slots, $options = [] ) {
    if( count( $time_slots ) < 2 )
      return null;

    if( in_array( 'authentication_by_segment', $options ) )
      $this->_chart_map_by_time_slots_data['authentication_segments'] = WPEP_Entity_ActionTracking::instance()->get_unique_cols_by_args( 'segment', [
        'type'      => [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_REGISTRATION, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD ],
        'time_from' => $time_slots[0],
        'time_to'   => array_reverse( $time_slots )[0]
      ] );

    if( in_array( 'authentication_by_post_id', $options ) )
      $this->_chart_map_by_time_slots_data['authentication_posts'] = WPEP_Entity_ActionTracking::instance()->get_unique_cols_by_args( 'post_id=>post_title', [
        'type'      => [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_REGISTRATION, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD ],
        'time_from' => $time_slots[0],
        'time_to'   => array_reverse( $time_slots )[0]
      ] );

    $response = [];

    $all_empty = true;

    $first_slot_key = key( $time_slots );
    $current_slot = $time_slots[ key( $time_slots ) ];
    unset( $time_slots[ $first_slot_key ] );

    foreach( $time_slots as $next_slot ) {
      $current_time_slot_data = [];

      if( in_array( 'content_types', $options ) ) {
        foreach( wpep_content_type()->contentTypes as $contentType ) {
          $content_type_data = $contentType->data_user_activity_report( [
            'time_from' => $current_slot,
            'time_to'   => $next_slot
          ] );

          if( empty( $content_type_data ) )
            continue;

          $current_time_slot_data += $content_type_data;
        }
      }

      if( in_array( 'user_registration', $options ) )
        $current_time_slot_data = $this->_chart_map_by_time_slots_user_registration( $current_time_slot_data, $current_slot, $next_slot );

      if( in_array( 'authentication', $options ) )
        $current_time_slot_data = $this->_chart_map_by_time_slots_authentication( $current_time_slot_data, $current_slot, $next_slot );

      if( in_array( 'authentication_by_segment', $options ) && !empty( $this->_chart_map_by_time_slots_data['authentication_segments'] ) )
        $current_time_slot_data = $this->_chart_map_by_time_slots_authentication_by_segment( $current_time_slot_data, $current_slot, $next_slot );

      if( in_array( 'authentication_by_post_id', $options ) && !empty( $this->_chart_map_by_time_slots_data['authentication_posts'] ) )
        $current_time_slot_data = $this->_chart_map_by_time_slots_authentication_by_post_id( $current_time_slot_data, $current_slot, $next_slot );

      if( in_array( 'emails', $options ) )
        $current_time_slot_data = $this->_chart_map_by_time_slots_emails( $current_time_slot_data, $current_slot, $next_slot );

      if( !empty( $current_time_slot_data ) )
        $all_empty = false;

      ksort( $current_time_slot_data );

      $response[ $current_slot ] = $current_time_slot_data;

      $current_slot = $next_slot;
    }

    return ( $all_empty ? null : $response );
  }

  private function _chart_map_by_time_slots_user_registration( $current_time_slot_data, $time_from, $time_to ) {
    $current_time_slot_data[ 'User Registration' ] = WPEP_Controller::instance()->userActivity->get_registration_count( [
      'time_from' => $time_from,
      'time_to'   => $time_to
    ] );

    return $current_time_slot_data;
  }

  private function _chart_map_by_time_slots_authentication( $current_time_slot_data, $time_from, $time_to ) {

    foreach( [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, WPEP_ACTION_TRACKING_TYPE_REGISTRATION ] as $event_type ) {
      $current_time_slot_data[ 'Authentication - ' . $this->_authentication_name_by_event( $event_type ) ] = WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( [
        'type'      => $event_type,
        'time_from' => $time_from,
        'time_to'   => $time_to
      ] );
    }

    return $current_time_slot_data;
  }

  private function _chart_map_by_time_slots_authentication_by_segment( $current_time_slot_data, $time_from, $time_to ) {
    $data = WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( [
      'group_by'  => [ 'type', 'segment' ],
      'type'      => [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, WPEP_ACTION_TRACKING_TYPE_REGISTRATION ],
      'segment'   => $this->_chart_map_by_time_slots_data['authentication_segments'],
      'time_from' => $time_from,
      'time_to'   => $time_to
    ] );

    foreach( [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, WPEP_ACTION_TRACKING_TYPE_REGISTRATION ] as $event_type ) {
      foreach( $this->_chart_map_by_time_slots_data['authentication_segments'] as $segment )
        $current_time_slot_data[ 'Authentication - ' . $this->_authentication_name_by_event( $event_type ). ' - ' . $segment ] = 0;
    }

    foreach( $data as $information ) {
      $current_time_slot_data[ 'Authentication - ' . $this->_authentication_name_by_event( $information->type ). ' - ' . $information->segment ] = $information->entries;
    }

    return $current_time_slot_data;
  }

  private function _chart_map_by_time_slots_authentication_by_post_id( $current_time_slot_data, $time_from, $time_to ) {
    $data = WPEP_Entity_ActionTracking::instance()->get_event_count_by_args( [
      'group_by'   => [ 'type', 'post_id' ],
      'type'       => [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, WPEP_ACTION_TRACKING_TYPE_REGISTRATION ],
      'post_id'    => array_keys( $this->_chart_map_by_time_slots_data['authentication_posts'] ),
      'time_from'  => $time_from,
      'time_to'    => $time_to
    ] );

    foreach( [ WPEP_ACTION_TRACKING_TYPE_LOGIN, WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD, WPEP_ACTION_TRACKING_TYPE_REGISTRATION ] as $event_type ) {
      foreach( $this->_chart_map_by_time_slots_data['authentication_posts'] as $post_id => $post_title )
        $current_time_slot_data[ 'Authentication - ' . $this->_authentication_name_by_event( $event_type ). ' - ' . $post_title ] = 0;
    }

    foreach( $data as $information )
      $current_time_slot_data[ 'Authentication - ' . $this->_authentication_name_by_event( $information->type ). ' - ' . $this->_chart_map_by_time_slots_data['authentication_posts'][$information->post_id] ] = $information->entries;

    return $current_time_slot_data;
  }

  private function _authentication_name_by_event( $event_type ) {
    if( $event_type === WPEP_ACTION_TRACKING_TYPE_LOGIN )
      return 'Login';

    if( $event_type === WPEP_ACTION_TRACKING_TYPE_REGISTRATION )
      return 'Register';

    if( $event_type === WPEP_ACTION_TRACKING_TYPE_RESET_PASSWORD )
      return 'Reset Password';

    return '???';
  }

  private function _chart_map_by_time_slots_emails( $current_time_slot_data, $time_from, $time_to ) {
    if( defined( "WPEP_BETA_TESTING" ) || defined( "WPEP_BETA_TESTING_TEACHER_NOTIFICATIONS" ) ) {
      $current_time_slot_data[ 'Email Notifications' ] = WPEP_Controller::instance()->email->get_delivery_count( [
        'time_from' => $time_from,
        'time_to'   => $time_to
      ] );
    }

    return $current_time_slot_data;
  }

}