<?php
namespace WPEP\Coordinator;

use WPEP\Controller as WPEP_Controller;
use WPEP\ContentType as WPEP_ContentType;
use WPEP\Posts;

class Hooks {

  public function setup() {
    add_action( 'wpep_deliver_csv_export_file', [ $this, 'download_table_csv' ], 10, 3 );
    add_action( 'widgets_init', [ $this, 'register_widget_area' ], 10 );
    add_filter( 'pre_get_avatar', [ $this, 'pre_get_avatar' ], 20, 3 );

    if( wpep_string_ends_with( wpep_get_current_url(), 'async-upload.php' )
      || ( isset( $_POST[ 'action' ] ) && $_POST['action'] == 'query-attachments' ) ) {
      if( isset( $_COOKIE[ 'wpep_upload_post_id' ] ) && ( $_COOKIE[ 'wpep_upload_post_id' ] != '0' ) ) {
        add_filter( 'ajax_query_attachments_args', [ $this, 'handle_frontend_attachments_query' ] );
        add_filter( 'user_has_cap', [ $this, 'user_has_cap'], 100, 4 );
        add_action( 'add_attachment', [ $this, 'add_attachment' ] );
      }
    } else {
      add_filter( 'ajax_query_attachments_args', [ $this, 'handle_backend_attachments_query' ] );
    }

    if( !is_admin() && wpep_get_setting_flag( 'disable-admin-bar-for-students' ) )
      add_action( 'after_setup_theme', [ $this, '_disable_admin_bar_for_students' ] );
  }

  public function register_widget_area() {
    register_sidebar( [
      'name'          => __('WPEP Sidebar', "wpep" ),
      'id'            => 'wpep-sidebar',
      'before_widget' => '<div id="%1$s" class="wpep-widget %2$s">',
      'after_widget'  => '</div>',
      'before_title'  => '<h1 class="wpep-widget-title">',
      'after_title'   => '</h1>',
    ] );
  }

  public function add_attachment( $post_ID ) {
    if( ! is_user_logged_in() )
      return;

    if( !isset( $_COOKIE[ 'wpep_upload_post_id' ] ) )
      return;

    $post = ( !is_numeric( $_COOKIE[ 'wpep_upload_post_id' ] ) ? false : get_post( intval( $_COOKIE[ 'wpep_upload_post_id' ] ) ) );

    if( $post !== null ) {
      wp_update_post( [
        'ID'          => $post_ID,
        'post_parent' => intval( $_COOKIE[ 'wpep_upload_post_id' ] )
      ] );
    }
    
    if( isset( $post->ID ) )
      update_post_meta( $post_ID, 'wpep_uploaded_file_parent_id', $post->ID );
    else
      update_post_meta( $post_ID, 'wpep_uploaded_file_parent_id', sanitize_text_field( $_COOKIE[ 'wpep_upload_post_id' ] ) );

    update_post_meta( $post_ID, 'wpep_uploaded_file_user_id', get_current_user_id() );
  }

  public function user_has_cap( $capabilities, $caps, $args, $user_object ) {
    if( ! is_user_logged_in() )
      return $capabilities;

    $capabilities['upload_files'] = true;

    return $capabilities;
  }

  public function handle_frontend_attachments_query( $query ) {
    if( ! is_user_logged_in() )
      return $query;

    if( current_user_can( 'manage_options' ) )
      return $this->handle_backend_attachments_query( $query );

    $user_id = get_current_user_id();

    $query['author'] = $user_id;

    return $query;
  }

  public function handle_backend_attachments_query( $query ) {
    if( ! is_user_logged_in() )
      return $query;
    
    if( !isset( $query['meta_query'] ) )
      $query['meta_query'] = [];

    $query['meta_query']['wpep_uf_uid'] = [
      'relation' => 'OR',
      [
        'key'     => 'wpep_uploaded_file_user_id',
        'value'   => get_current_user_id(),
        'compare' => '=',
      ],
      [
        'key'     => 'wpep_uploaded_file_user_id',
        'value'   => '',
        'compare' => 'NOT EXISTS',
      ],
    ];

    return $query;
  }

  public function pre_get_avatar( $default_avatar, $id_or_email, $args ) {
    if( !is_numeric( $id_or_email ) )
      return $default_avatar;

    $profile_picture = get_user_meta( $id_or_email, WPEP_USER_META_PROFILE_PICTURE, true );

    if( $profile_picture == '' )
      return $default_avatar;

    return sprintf(
      "<img alt='%s' src='%s' class='avatar' height='%d' width='%d' %s/>",
      esc_attr( $args['alt'] ),
      esc_url( $profile_picture ),
      (int) $args['height'],
      (int) $args['width'],
      $args['extra_attr']
    );
  }

  public function _disable_admin_bar_for_students() {
    if( current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      return;
    
    show_admin_bar(false);
  }

  public function download_table_csv( $name = 'download',
                                      $head_row_data = [],
                                      $body_row_list = [] ) {
    if( empty( $head_row_data ) || empty( $body_row_list ) )
      return;

    $name          = sanitize_title( $name );
    $head_row_data = $this->_get_csv_head( $head_row_data );

    header('Content-Type: application/csv');
    header('Content-Disposition: attachment; filename="'. $name .'.csv";');

    $output_stream = fopen('php://output', 'w');

    fputcsv( $output_stream, $head_row_data , ';' );

    foreach ( $body_row_list as $body_row_item ) {
      fputcsv( $output_stream, $this->_get_csv_body_row( $body_row_item, $head_row_data ), ';' );
    }

    fclose( $output_stream );

    exit;
  }

  private function _get_csv_head( $head_row_data ) {
    foreach( $head_row_data as $key => $value ) {
      if( is_string( $value ) )
        continue;

      if( isset( $value['type'] ) && $value['type'] == 'link-item' ) {
        unset( $head_row_data[ $key ] );
        continue;
      }

      $head_row_data[ $key ] = (string)( isset( $value['value_raw'] ) ? $value['value_raw'] : $value['value'] );
    }

    return $head_row_data;
  }

  private function _get_csv_body_row( $body_row_data, $head_row_data ) {
    $resp = [];

    foreach( $head_row_data as $head_row_key => $head_row_value ) {
      $current_body_row_data = $body_row_data[ $head_row_key ];

      $resp[ $head_row_key ] = (string) ( is_string( $current_body_row_data )
        ? $current_body_row_data
        : ( isset( $current_body_row_data['value_raw'] ) ? $current_body_row_data['value_raw'] : $current_body_row_data['value'] ) );
    }

    return $resp;
  }

}