<?php

namespace WPEP\Coordinator;

use Braintree\Xml;
use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;
use WPEPCronController as WPEP_Cron;

class PlatformHealth {

  private $_service_status_report = [];
  private $_notification_count_transient_name = 'wpep_platform_health_error_detection';

  public function get_service_status_notification_count() {
    $error_count = ( isset( $_GET['page'] ) && $_GET['page'] == 'wpep-platform-health' ? false : get_transient( $this->_notification_count_transient_name ) );

    if( $error_count !== false )
      return intval( $error_count );

    $error_count = 0;

    foreach( WPEP_Controller::instance()->platformHealth->get_service_status_report() as $service_status_information ) {
      if( !isset( $service_status_information['status'] ) )
        continue;

      if( $service_status_information['status'] !== 'danger'
        && $service_status_information['status'] !== 'error' )
        continue;

      $error_count++;
    }

    set_transient( $this->_notification_count_transient_name, $error_count, HOUR_IN_SECONDS );

    return $error_count;
  }

  public function delete_service_status_notification_count_transient() {
    delete_transient( $this->_notification_count_transient_name );
  }

  private function _is_database_fix_required() {
    foreach( WPEP_Controller::instance()->adminController->migration->get_integrations() as $integration ) {
      if( $integration->is_migration_required() )
        return false;

      if( !$integration->is_valid_table_structure() )
        return true;
    }

    global $wpdb;

    $table_assessment_question_answer_columns = wpep_utility_map_to_array_assoc(
      $wpdb->get_results( 'DESCRIBE `' . $wpdb->prefix . 'wpep_assessment_question_answer' . '`'),
      'Field',
      'Type'
    );

    if( !isset( $table_assessment_question_answer_columns['question_id'] ) )
      return true;

    $table_section_lesson_columns = wpep_utility_map_to_array_assoc(
      $wpdb->get_results( 'DESCRIBE `' . $wpdb->prefix . 'wpep_section_lesson' . '`'),
      'Field',
      'Type'
    );

    if( !isset( $table_section_lesson_columns['post_id'] ) )
      return true;

    $table_assessment_user_data_columns = wpep_utility_map_to_array_assoc(
      $wpdb->get_results( 'DESCRIBE `' . $wpdb->prefix . 'wpep_assessment_user_data' . '`'),
      'Field',
      'Type'
    );

    if( $table_assessment_user_data_columns['value'] != 'longtext' )
      return true;

    return false;
  }

  public function get_service_status_report() {
    if( !empty( $this->_service_status_report ) )
      return $this->_service_status_report;

    $user_role_list = WPEP_Controller::instance()->capabilities->get_role_list();

    $database_fix_required = $this->_is_database_fix_required();

    $response = [];

    $response[ 'database_table_integrity' ] = [
      'title'             => __( "Database Table Integrity", "wpep" ),
      'operation_text'    => __( "Resolve", "wpep" ),
      'operation'         => wpep_admin_url_platform_health( [ 'tab' => 'dashboard', 'operation' => 'resolve-table-integrity-issues' ]  ),
      'operation_allowed' => $database_fix_required,
      'status'            => ( !$database_fix_required ? 'success' : 'danger' )
    ];

    $response[ 'user_roles_integrity' ] = [
      'title'             => __( "User Roles", "wpep" ),
      'operation_text'    => __( "Refresh", "wpep" ),
      'operation'         => wpep_admin_url_platform_health( [ 'tab' => 'dashboard', 'operation' => 'refresh-user-roles' ]  ),
      'operation_allowed' => true,
      'status'            => ( !in_array( 'error', $user_role_list ) ? 'success' : 'danger' )
    ];

    $response[ 'rewrite_rules' ] = [
      'title'             => __( "Rewrite Rules ( Post Permalinks )", "wpep" ),
      'operation_text'    => __( "Reset", "wpep" ),
      'operation'         => wpep_admin_url_platform_health( [ 'tab' => 'dashboard', 'operation' => 'reset-permalinks' ]  ),
      'operation_allowed' => true,
      'status'            => ( $this->_rewrite_rules_ok()  ? 'success' : 'danger' )
    ];

    $response[ 'cron_jobs' ] = [
      'title'             => __( "Cron Jobs - Not required", "wpep" ),
      'operation_text'    => '',
      'operation'         => '',
      'operation_allowed' => false,
      'status'            => 'success'
    ];

    if( WPEP_Cron::has_jobs() ) {
      $cron_jobs_status = WPEP_Cron::get_jobs_status();

      if( $cron_jobs_status === false ) {
        $response[ 'cron_jobs' ] = [
            'title'             => __( "Cron Jobs - Reset Required", "wpep" ),
            'operation_allowed' => true,
            'operation_text'    => __( "Reset", "wpep" ),
            'operation'         => wpep_admin_url_platform_health( [ 'tab' => 'dashboard', 'operation' => 'reset-cron-jobs' ]  ),
            'status'            => 'danger'
          ] + $response[ 'cron_jobs' ];
      } elseif( $cron_jobs_status === null ) {
        $response[ 'cron_jobs' ] = [
            'title'             => __( "Cron Jobs - Status Unknown", "wpep" ),
            'status'            => 'warning'
          ] + $response[ 'cron_jobs' ];
      } elseif( !WPEP_Cron::is_job_list_executing() ) {
        $response[ 'cron_jobs' ] = [
            'title'             => __( "Cron Jobs - Not Executing", "wpep" ),
            'operation_allowed' => true,
            'operation_text'    => __( "Learn More", "wpep" ),
            'operation'         => WPEP_DOC_CRON_JOBS_NOT_WORKING,
            'status'            => 'danger'
          ] + $response[ 'cron_jobs' ];
      } elseif( ! ( defined( 'DISABLE_WP_CRON' ) && DISABLE_WP_CRON ) ) {
        $response[ 'cron_jobs' ] = [
            'title'             => __( "Cron Jobs - WP Triggered", "wpep" ),
            'operation_allowed' => true,
            'operation_text'    => __( "Learn More", "wpep" ),
            'operation'         => WPEP_DOC_CRON_JOBS_FROM_SERVER,
            'status'            => 'info'
          ] + $response[ 'cron_jobs' ];
      } else {
        $response[ 'cron_jobs' ] = [
            'title'             => __( "Cron Jobs - Server Triggered", "wpep" ),
            'operation_allowed' => false,
            'status'            => 'success'
          ] + $response[ 'cron_jobs' ];
      }
    }

    $response[ 'custom_post_permalinks' ] = [
      'title'             => __( "Custom Post Permalink Slugs", "wpep" ),
      'operation_text'    => __( "Settings", "wpep" ),
      'operation'         => wpep_admin_url_settings( [ 'tab' => 'general', 'tab_inner' => 'url' ] ),
      'operation_allowed' => true,
      'status'            => ( !WPEP_Settings::instance()->has_changed_setting( ['content-type-courses-rewrite-slug', 'content-type-video-rewrite-slug', 'content-type-ebook-rewrite-slug', 'content-type-offer-rewrite-slug', 'content-type-assessment-rewrite-slug', 'rewrite-slug-categories' ] )  ? '' : 'success' )
    ];

    $response[ 'custom_css_by_file' ] = [
      'title'             => __( "Custom CSS Delivered by File", "wpep" ),
      'operation_text'    => __( "Deliver on Page", "wpep" ),
      'operation'         => wpep_admin_url_platform_health( [ 'tab' => 'dashboard', 'operation' => 'set-custom-css-by-html' ]  ),
      'operation_allowed' => true,
      'status'            => 'success'
    ];

    if( !wpep_get_setting_flag( 'wpep-custom-css-by-file' ) ) {
      $response[ 'custom_css_by_file' ] = [
        'title'             => __( "Custom CSS Delivered on Page", "wpep" ),
        'operation_text'    => __( "Deliver on File", "wpep" ),
        'operation'         => wpep_admin_url_platform_health( [ 'tab' => 'dashboard', 'operation' => 'set-custom-css-by-file' ]  ),
        'status'            => 'info'
        ] + $response[ 'custom_css_by_file' ];
    } else if( wpep_get_custom_style_file() == false ) {
      $response[ 'custom_css_by_file' ] = [
        'operation_text'    => __( "Deliver on Page", "wpep" ),
        'title'             => __( "Custom CSS cannot be delivered from file", "wpep" ),
        'status'            => 'danger'
        ] + $response[ 'custom_css_by_file' ];
    }

    $response[ 'course_use_content_filter' ] = [
      'title'             => __( "Deliver Course Content by Filtering - Disabled", "wpep" ),
      'operation_text'    => __( "Enable", "wpep" ),
      'operation'         => wpep_admin_url_platform_health( [ 'tab' => 'dashboard', 'operation' => 'set-course-content-delivery-by-filter' ]  ),
      'operation_allowed' => true,
      'status'            => 'info'
    ];

    if( wpep_get_setting_flag( 'course-use-the-content-filter' ) ) {
      $response[ 'course_use_content_filter' ]['title']   = __( "Deliver Course Content by Filtering", "wpep" );
      $response[ 'course_use_content_filter' ]['status']  = 'success';
      $response[ 'course_use_content_filter' ]['operation'] = wpep_admin_url_platform_health( [ 'tab' => 'dashboard', 'operation' => 'set-course-content-delivery-by-raw' ]  );
      $response[ 'course_use_content_filter' ]['operation_text'] = __( "Disable", "wpep" );
    }


    $response[ 'user_public_profiles' ] = [
      'title'             => __( "User Public Profiles", "wpep" ),
      'operation_text'    => __( "Configure", "wpep" ),
      'operation'         => wpep_admin_url_settings( [ 'tab' => 'learner_profile', 'tab_inner' => 'public' ] ),
      'operation_allowed' => true,
      'status'            => ''
    ];

    if( wpep_public_profiles_enabled() ) {
      $response[ 'user_public_profiles' ] = [
          'operation_text'     => __( "Settings", "wpep" ),
          'operation'         => wpep_admin_url_settings( [ 'tab' => 'learner_profile', 'tab_inner' => 'public' ] ),
          'status'            => 'success'
        ] + $response[ 'user_public_profiles' ];
    }

    $response[ 'course_assessments' ] = [
      'title'             => __( "Course Assessments", "wpep" ),
      'operation_text'    => ( wpep_assessments_enabled() ? __( "Settings", "wpep" ) : __( "Configure", "wpep" ) ),
      'operation'         => wpep_admin_url_settings( [ 'tab' => 'assessments' ] ),
      'operation_allowed' => true,
      'status'            => ( wpep_assessments_enabled() ? 'success' : '' )
    ];

    $response[ 'course_timing_options' ] = [
      'title'             => __( "Course Timing Options", "wpep" ),
      'operation_text'    => ( wpep_get_setting_flag( 'enable-course-timing-option' ) ? __( "Disable", "wpep" ) : __( "Enable", "wpep" ) ),
      'operation'         => ( wpep_get_setting_flag( 'enable-course-timing-option' )
        ? wpep_admin_url_platform_health( [ 'tab' => 'dashboard', 'operation' => 'disable-course-timing-options' ]  )
        : wpep_admin_url_platform_health( [ 'tab' => 'dashboard', 'operation' => 'enable-course-timing-options' ]  )
      ),
      'operation_allowed' => true,
      'status'            => ( wpep_get_setting_flag( 'enable-course-timing-option' ) ? 'success' : '' )
    ];

    $this->_service_status_report = $response;

    return $response;
  }

  private function _rewrite_rules_ok() {
    $standardContentTypes = wpep_content_type()->standardContentTypes;

    foreach( $standardContentTypes as $standardContentType ) {
      if( !$standardContentType->is_publicly_queryable() )
        continue;

      $tested_post    = get_posts( [
        'post_type'   => $standardContentType->get_post_type(),
        'numberposts' => 1
      ] );

      if( empty( $tested_post ) )
        continue;

      $tested_post = array_shift( $tested_post );

      if( $tested_post->ID != wpep_get_post_id_by_url( get_permalink( $tested_post ) ) )
        return false;
    }

    return true;
  }

  public function get_report() {
    $active_plugins   = $this->_get_active_plugin_list();
    $inactive_plugins = $this->_get_inactive_plugin_list();

    $response = [];

    $response['wordpress_environment'] = [
      'name'    => 'WordPress Environment',
      'type'    => 'alert',
      'entries' => $this->_get_wordpress_environment( $active_plugins, $inactive_plugins )
    ];

    if( WPEP_Cron::has_jobs() ) {
      $response['cron_jobs'] = [
        'name'    => 'Cron Jobs',
        'type'    => 'alert',
        'entries' => $this->_get_cron_jobs_report()
      ];
    }

    $response['server_environment'] = [
      'name'    => 'Server Environment',
      'type'    => 'alert',
      'entries' => $this->_get_server_environment()
    ];

    $response['database_environment'] = [
      'name'    => 'Database Environment',
      'type'    => 'alert',
      'entries' => $this->_get_database_environment()
    ];

    $response['wpep_role_list'] = [
      'name'    => 'WPEP Role List Status',
      'type'    => 'alert',
      'entries' => $this->_get_user_role_list()
    ];

    $response['wpep_settings'] = [
      'name'    => 'WPEP Settings',
      'type'    => 'normal',
      'entries' => $this->_get_wpep_settings()
    ];

    $response['wpep_post_settings'] = [
      'name'    => 'WPEP Post Types Settings',
      'type'    => 'normal',
      'entries' => $this->_get_post_type_settings()
    ];

    if( in_array( WPEP_POST_TYPE_COURSE, wpep_content_type()->library_post_types() ) ) {
      $response['wpep_course_data'] = [
        'name'    => 'WPEP User Course Data',
        'type'    => 'normal',
        'entries' => $this->_get_course_data_integrity()
      ];
    }

    $response['license_status'] = [
      'name'    => 'License Status & Verification',
      'type'    => 'alert',
      'entries' => $this->_get_license_status_list()
    ];

    $response['active_plugin_list'] = [
      'name'    => 'Active Plugins - ' . count( $active_plugins ) . ' / ' . ( count( $active_plugins ) + count( $inactive_plugins ) ),
      'type'    => 'alert',
      'entries' => $active_plugins
    ];

    $response['inactive_plugin_list'] = [
      'name'    => 'Inactive Plugins - ' . count( $inactive_plugins ) . ' / ' . ( count( $active_plugins ) + count( $inactive_plugins ) ),
      'type'    => 'alert',
      'entries' => $inactive_plugins
    ];

    return $response;
  }

  private function _get_wordpress_environment( $active_plugins = [], $inactive_plugins = [] ) {
    $wp_memory_limit = wpep_let_to_num( WP_MEMORY_LIMIT );
    if ( function_exists( 'memory_get_usage' ) ) {
      $wp_memory_limit = max( $wp_memory_limit, wpep_let_to_num( @ini_get( 'memory_limit' ) ) );
    }

    $has_wp_cron          = ! ( defined( 'DISABLE_WP_CRON' ) && DISABLE_WP_CRON );
    $has_wp_debug         = ( defined( 'WP_DEBUG' ) && WP_DEBUG );
    $has_wp_debug_log     = ( defined( 'WP_DEBUG_LOG' ) && WP_DEBUG_LOG );

    $my_theme = wp_get_theme();

    return [
      'home_url'                  => get_option( 'home' ),
      'site_url'                  => get_option( 'siteurl' ),
      'wpep_version'              => WPEP_VERSION,
      'wp_version'                => get_bloginfo( 'version' ),
      'active_theme'              => sprintf( __( "%s on version %s", "wpep" ), $my_theme->get( 'Name' ), $my_theme->get( 'Version' ) ),
      'wp_multisite'              => is_multisite(),
      'wp_memory_limit'           => [
        'status'  => ( $wp_memory_limit < 67108864 ? 'warning' : 'success' ),
        'type'    => 'storage_size',
        'value'   =>  $wp_memory_limit
      ],
      'wp_cron'                   => [
        'status'  => ( $has_wp_cron ? 'info' : 'success' ),
        'value'   => $has_wp_cron
      ],
      'wp_debug'                  => [
        'status'  => ( $has_wp_debug ? 'info' : 'success' ),
        'value'   => $has_wp_debug
      ],
      'wp_debug_log'              => [
        'status'  => ( $has_wp_debug_log ? 'info' : 'success' ),
        'value'   => $has_wp_debug_log
      ],
      'language'  => get_locale(),
      'active_plugins' => [
        'status'  => ( count( $active_plugins ) > 50 ? 'danger' : 'success' ),
        'value'   => count( $active_plugins )
      ]
    ];
  }

  private function _get_server_environment() {
    global $wpdb;

    // Figure out cURL version, if installed.
    $curl_version = '';
    if ( function_exists( 'curl_version' ) ) {
      $curl_version = curl_version();
      $curl_version = $curl_version['version'] . ', ' . $curl_version['ssl_version'];
    }

    $mysql_version = ( ! empty( $wpdb->is_mysql ) ? $wpdb->db_version() : '' );

    $post_max_size   = wpep_let_to_num( ini_get( 'post_max_size' ) );
    $max_upload_size = wp_max_upload_size();
    $server_info     = $_SERVER['SERVER_SOFTWARE'];

    return [
      'server_info'               => [
        'status'  => ( $server_info == '' ? 'info' : 'success' ),
        'value'   => $server_info
      ],
      'php_version'               => [
        'status'  => ( version_compare( phpversion(), '5.6', '<' ) ? 'danger' : ( version_compare( phpversion(), '7.1', '<' ) ? 'warning' : 'success' ) ),
        'value'   => phpversion()
      ],
      'php_post_max_size'         => [
        'status'  => ( $post_max_size >= 64000000 ? 'success' : 'warning' ),
        'type'    => 'storage_size',
        'value'   => $post_max_size
      ],
      'php_max_execution_time'    => [
        'status'  => ( ini_get( 'max_execution_time' ) >= 30 ? 'success' : 'warning' ),
        'type'    => 'php_var',
        'value'   => ini_get( 'max_execution_time' )
      ],
      'php_max_input_vars'        => [
        'status'  => ( ini_get( 'max_input_vars' ) >= 3000 ? 'success' : ( ini_get( 'max_input_vars' ) >= 1000 ? 'info' : 'warning' ) ),
        'type'    => 'php_var',
        'value'   => ini_get( 'max_input_vars' )
      ],
      'max_upload_size'           => [
        'status'  => ( $max_upload_size >= 16000000 ? 'success' : 'warning' ),
        'type'    => 'storage_size',
        'value'   => $max_upload_size
      ],
      'mysql_version'             => [
        'status'  => ( version_compare( $mysql_version, '5.6', '<' ) ? ( version_compare( $mysql_version, '5.2', '<' ) ? 'danger' : 'info' ) : 'success' ),
        'value'   => $mysql_version
      ],
      'curl_version'              => [
        'status'  => ( $curl_version == '' ? 'danger' : 'success' ),
        'value'   => $curl_version
      ],
      'file_get_contents'         => [
        'status'  => ( function_exists( 'file_get_contents' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( function_exists( 'file_get_contents' ) ? 'Available' : 'Not Available' )
      ],
      'file_put_contents' => [
        'status'  => ( function_exists( 'file_put_contents' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( function_exists( 'file_put_contents' ) ? 'Available' : 'Not Available' )
      ],
      'allow_url_fopen' => [
        'status'  => ( ini_get('allow_url_fopen') ? 'success' : 'warning' ),
        'type'    => 'php_var',
        'value'   => ( ini_get('allow_url_fopen') ? 'Available' : 'Not Available' )
      ],
      'imagettftext'      => [
        'status'  => ( function_exists( 'imagettftext' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( function_exists( 'imagettftext' ) ? 'Available' : 'Not Available' )
      ],
      'imagejpeg'         => [
        'status'  => ( function_exists( 'imagejpeg' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( function_exists( 'imagejpeg' ) ? 'Available' : 'Not Available' )
      ],
      'soapclient' => [
        'status'  => ( class_exists( 'SoapClient' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( class_exists( 'SoapClient' ) ? 'Available' : 'Not Available' )
      ],
      'gzip'   => [
        'status'  => ( is_callable( 'gzopen' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( is_callable( 'gzopen' ) ? 'Available' : 'Not Available' )
      ],
      'mbstring' => [
        'status'  => ( extension_loaded( 'mbstring' ) ? 'success' : 'warning' ),
        'type'    => 'php_function',
        'value'   => ( extension_loaded( 'mbstring' ) ? 'Available' : 'Not Available' )
      ]
    ];
  }

  private function _get_cron_jobs_report() {
    $response = [];

    foreach( WPEP_Cron::get_job_list_report() as $job_name => $data ) {
      $status = 'success';

      if( $data['scheduled'] === 0 )
        $status = 'danger';
      elseif( $data['scheduled'] < time() )
        $status = ( time() - $data['scheduled'] > HOUR_IN_SECONDS ? 'danger' : 'warning' );

      $response[ $job_name ] = [
        'status'  => $status,
        'value'   => ( $data['scheduled'] === 0 ? __( "Missing", "wpep" ) : ( wpep_format_date_with_time( $data['scheduled'] ) . ' - ' . ( $data['schedule_name' ] === false ? '???' : $data['schedule_name' ] ) ) )
      ];
    }

    return $response;
  }

  private function _get_database_environment() {
    global $wpdb;

    $response = [];

    foreach( WPEP_Controller::instance()->migrationLibrary->get_migration_instances() as $migration_instance ) {
      $response[ $migration_instance->namespace . '_migration' ] = [
        'status'  => ( $migration_instance->current_version == $migration_instance->current_version_available ? 'success' : 'danger' ),
        'type'    => 'version',
        'value'   => $migration_instance->current_version
      ];

      foreach( $migration_instance->get_table_list() as $table_name ) {
        $table_exists = $wpdb->get_var("SHOW TABLES LIKE '$table_name'") == $table_name;

        $response[ $table_name ] = [
          'status'  => ( $table_exists ? 'success' : 'danger' ),
          'type'    => 'table',
          'value'   => ( $table_exists ? 'ok' : 'missing' )
        ];
      }
    }

    return $response;
  }

  private function _get_wpep_settings() {
    $response = [
      'wpep_theme'                       => wpep_get_setting( 'wpep-theme' ),
      'primary_color'                    => wpep_get_setting( 'wpep-custom-color' ),
      'secondary_color'                  => wpep_get_setting( 'wpep-custom-color-secondary' ),
      'custom_css_by_file_enabled'       => $this->_bool_val( wpep_get_setting_flag( 'wpep-custom-css-by-file' ) ),
      'assessments_enabled'              => $this->_bool_val( wpep_assessments_enabled() ),
      'timing_options_enabled'           => $this->_bool_val( wpep_get_setting_flag( 'enable-course-timing-option' ) ),
      'internal_auth_status'             => wpep_get_auth_status_level_options( wpep_get_setting_flag( 'user-auth-status-level' ) ),
      'single_category_view_enabled'     => $this->_bool_val( wpep_get_setting_flag( 'index-page-single-category-primary-view' ) ),
      'lesson_incomplete_enabled'        => $this->_bool_val( wpep_get_setting_flag( 'enable-lesson-incomplete-option' ) ),
      'admin_course_save_alert_enabled'  => $this->_bool_val( wpep_get_setting_flag( 'admin-course-save-alert' ) ),
      'lesson_wp_editor_preload_enabled' => $this->_bool_val( wpep_get_setting_flag( 'lesson-wp-editor-preload' ) ),
      'lesson_open_on_add_enabled'       => $this->_bool_val( wpep_get_setting_flag( 'lesson-wp-editor-enabled-new' ) ),
      'wpep_editor_integration_enabled'  => $this->_bool_val( wpep_get_setting_flag( 'admin-editor-integration' ) ),
      'wpep_editor_setting'              => wpep_get_setting_flag( 'wpep-editor-setup' ),
      'new_course_sections'              => wpep_get_setting_flag( 'admin-course-preset-sections' ),
      'new_course_lessons'               => wpep_get_setting_flag( 'admin-course-preset-lessons' )
    ];

    return $response;
  }

  private function _get_post_type_settings() {
    $response = [];

    foreach( wpep_content_type()->libraryContentTypes as $libraryContentType ) {
      $alias = wpep_label_to_alias( $libraryContentType->get_plural_name() );

      $response[ $alias . '_created' ] = wpep_utility_array_sum( wp_count_posts( $libraryContentType->get_post_type() ) );
      $response[ $alias . '_archive' ] = $this->_bool_val( wpep_get_setting_flag( 'content-type-' . $libraryContentType->get_post_type() . '-enable-archive' ) );

      if( $libraryContentType->is_publicly_queryable() )
        $response[ $alias . '_slug' ]    = wpep_get_setting( 'content-type-' . $libraryContentType->get_post_type() . '-rewrite-slug' );
    }

    if( wpep_assessments_enabled() ) {
      $response[ 'assessments_created' ] = wpep_utility_array_sum( wp_count_posts( WPEP_POST_TYPE_ASSESSMENT ) );
      $response[ 'assessments_slug' ]    = wpep_get_setting( 'content-type-assessment-rewrite-slug' );
    }

    $response['categories_slug']    = wpep_get_setting( 'rewrite-slug-categories' );
    $response['categories_created'] = wp_count_terms( WPEP_TAXONOMY_CATEGORY );

    return $response;
  }

  private function _get_license_status_list() {
    $response = [];

    foreach( WPEP_Controller::instance()->licenseLibrary->get_license_list() as $integration_alias => $license_information ) {
      $response[$integration_alias] = [
        'type'   => 'table',
        'status' => ( $license_information['is_valid'] ? 'success' : 'danger' ),
        'value'  => ( $license_information['license'] === '' ? 'missing' : ( $license_information['is_valid'] ? 'valid' : 'invalid' ) . ' - ' . ( $license_information['last_verification'] == 0 ? 'never' : wpep_format_date_with_time( $license_information['last_verification'] ) ) )
      ];
    }

    return $response;
  }

  private function _get_user_role_list() {
    $response = [];

    foreach( WPEP_Controller::instance()->capabilities->get_role_list() as $role_key => $role_status )
      $response[$role_key] = [
        'type'   => 'user_role',
        'status' => ( $role_status == 'ok' ? 'success' : 'danger' ),
        'value'  =>  $role_status
      ];

    return $response;
  }

  private function _get_course_data_integrity() {
    $response = [];

    foreach( [
               WPEP_USER_COURSE_STARTED,
               WPEP_USER_COURSE_COMPLETED,
               WPEP_USER_COURSE_COMPLETED_AT,
               WPEP_USER_COURSE_IS_BOOKMARKED
             ] as $course_data_key ) {
      $response[ $course_data_key ] = [
        'type'  => 'table',
        'value' => count( WPEP_Controller::instance()->userActivity->get_course_user_data_value( $course_data_key, false, false, false, false, false ) )
      ];
    }

    $response[ WPEP_USER_COURSE_STARTED . '_users' ] = [
      'type'  => 'table',
      'value' => count(
        wpep_utility_map_to_array_assoc(
          WPEP_Controller::instance()->userActivity->get_course_user_data( WPEP_USER_COURSE_STARTED, false, false, false, false, false ),
          'user_id',
          'user_id'
        )
      )
    ];

    $response[ WPEP_USER_COURSE_COMPLETED . '_users' ] = [
      'type'  => 'table',
      'value' => count(
        wpep_utility_map_to_array_assoc(
          WPEP_Controller::instance()->userActivity->get_course_user_data( WPEP_USER_COURSE_COMPLETED, false, false, false, false, false ),
          'user_id',
          'user_id'
        )
      )
    ];

    return $response;
  }

  private function _get_active_plugin_list() {
    $response = [];

    foreach( get_plugins() as $plugin_path => $plugin_information ) {
      if( !is_plugin_active( $plugin_path ) )
        continue;

      $response[$plugin_information['Name']] = [
        'type'  => 'plugin',
        'value' => $plugin_information['Version']
      ];
    }

    return $response;
  }

  private function _get_inactive_plugin_list() {
    $response = [];

    foreach( get_plugins() as $plugin_path => $plugin_information ) {
      if( is_plugin_active( $plugin_path ) )
        continue;

      $response[$plugin_information['Name']] = [
        'type'  => 'plugin',
        'value' => $plugin_information['Version']
      ];
    }

    return $response;
  }

  private function _bool_val( $value ) {
    return !! $value;
  }

}