<?php

namespace WPEP\Coordinator;

use WPEP\Admin\Coordinator\Statistics\Courses\User;
use WPEP\Controller as WPEP_Controller;
use WPEP\Settings as WPEP_Settings;
use WPEP\Container\FilterProxy as FilterProxy;

class UserEngagement {

  use FilterProxy;

  /**
   * @var UserEngagement\Assessment
   */
  public $assessment;

  /**
   * @var UserEngagement\AutoLogin
   */
  public $autoLogin;

  /**
   * @var UserEngagement\Misc
   */
  public $misc;

  /**
   * @var UserEngagement\WeeklyRecap
   */
  public $weeklyRecap;

  /**
   * @var int|\WP_User
   */
  public $loop_user = 0;
  public $loop_courses_started = [];
  public $loop_courses_completed = [];
  public $loop_courses_recommended = [];
  public $loop_courses_updated = [];
  public $loop_lessons_completed_count = 0;

  private $_all_updated_courses = [];
  private $_temporary_shortcodes_registered = [];

  public function __construct() {
    $this->assessment  = new UserEngagement\Assessment();
    $this->autoLogin   = new UserEngagement\AutoLogin();
    $this->misc        = new UserEngagement\Misc();
    $this->weeklyRecap = new UserEngagement\WeeklyRecap();

    if( wpep_get_setting( 'user-delete-tag-email-marketing' ) !== '' )
      add_action( 'delete_user', [ $this, "_on_delete_user" ], 15 );

    if( wpep_get_setting_flag( 'user-account-delete-enabled' ) )
      add_action('wp_ajax_wpep_delete_account', [ $this, '_ajax_delete_account' ] );

    if( wpep_get_setting_flag( 'user-service-agreement-enabled' ) == true
        || wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) == true )
      add_action('wp_ajax_wpep_confirm_agreement', [ $this, '_ajax_confirm_agreement' ] );
  }

  public function _ajax_delete_account() {
    if( !isset( $_POST['user_id'] ) || !isset( $_POST['nonce'] ) )
      exit('');

    if( wp_verify_nonce( $_POST['nonce'], 'wpep-account-delete-' . $_POST['user_id'] ) === false )
      exit('');

    if( $_POST['user_id'] != get_current_user_id() && !current_user_can( WPEP_CAP_MANAGE_SETTINGS ) )
      exit('');

    if( user_can( $_POST['user_id'], WPEP_CAP_MANAGE_SETTINGS ) )
      exit( __( 'Admin Users cannot delete their own accounts from the User Profile, to prevent mistakes.', "wpep" ) );

    wp_delete_user( $_POST[ 'user_id' ] );

    exit( wpep_get_setting( 'user-account-delete-success-page', get_site_url() ) );
  }

  public function _ajax_confirm_agreement() {
    $form_information = $_POST['form_information'];

    if( !isset( $form_information['nonce'] ) )
      exit ( json_encode( [
        'status'              => 'error',
        'notification'        => __( "Invalid Request, the page will reload", "wpep" ),
        'notification_class'  => 'wpep-notification wpep-notification-warning',
        'refresh'             => 1
      ] ) );

    if( wp_verify_nonce( $form_information['nonce'], 'wpep-agreement-' . get_current_user_id() ) === false )
      exit ( json_encode( [
        'status'              => 'error',
        'notification'        => __( "Invalid Request, the page will reload", "wpep" ),
        'notification_class'  => 'wpep-notification wpep-notification-warning',
        'refresh'             => 1
      ] ) );

    if( wpep_get_setting_flag( 'user-service-agreement-enabled' )
      && ( !isset( $form_information['service_agreement'] ) || intval( $form_information['service_agreement'] ) != 1 ) ) {
      exit ( json_encode( [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-service-agreement-error' ),
        'notification_class'  => 'wpep-notification wpep-notification-danger'
      ] ) );
    }

    if( wpep_get_setting_flag( 'user-privacy-agreement-enabled' )
      && ( !isset( $form_information['privacy_agreement'] ) || intval( $form_information['privacy_agreement'] ) != 1 ) ) {
      exit ( json_encode( [
        'status'              => 'error',
        'notification'        => wpep_get_setting_translated( 'user-privacy-agreement-error' ),
        'notification_class'  => 'wpep-notification wpep-notification-danger'
      ] ) );
    }

    if( wpep_get_setting_flag( 'user-privacy-agreement-enabled' ) )
      update_user_meta( get_current_user_id(), WPEP_USER_META_AGREEMENT_PRIVACY_DATE, date( "Y-m-d" ) );
    if( wpep_get_setting_flag( 'user-service-agreement-enabled' ) )
      update_user_meta( get_current_user_id(), WPEP_USER_META_AGREEMENT_TOS_DATE, date( "Y-m-d" ) );

    exit ( json_encode( [
      'status'              => 'success',
      'notification'        => wpep_get_setting_translated( 'user-agreement-notification-success' ),
      'notification_class'  => 'wpep-notification wpep-notification-success',
      'refresh'             => 1
    ] ) );

  }

  public function _on_delete_user( $user_id ) {
    try {
      wpep_controller()->emailMarketing->tag_user( wpep_get_setting( 'user-delete-tag-email-marketing' ), $user_id );
    } catch ( \Exception $e ) {
      return false;
    }

    return true;
  }

  public function get_user_list() {
    return wpep_utility_map_to_array_assoc(
      WPEP_Controller::instance()->userActivity->get_course_user_data(
        WPEP_USER_COURSE_STARTED, false, false, false, false, false
      ),
      'user_id',
      'user_id'
    );

  }

  private function _pre_register_temporary_shortcodes() {
    if( !empty( $this->_temporary_shortcodes_registered ) )
      return;

    global $shortcode_tags;

    $this->_temporary_shortcodes_registered = $shortcode_tags;
    remove_all_shortcodes();
  }

  private function _register_temporary_content_shortcodes() {
    add_shortcode( 'video', function() {
      return '<img src="' . WPEP_EMAIL_CONTENT_PREVIEW_VIDEO . '"/>';
    });

    add_shortcode( 'wpep_video', function() {
      return '<img src="' . WPEP_EMAIL_CONTENT_PREVIEW_VIDEO . '"/>';
    });

    add_shortcode( 'audio', function() {
      return '<img src="' . WPEP_EMAIL_CONTENT_PREVIEW_AUDIO . '"/>';
    });

    add_shortcode( "user_name", function() {
      return wpep_content_user_profile_format_name( get_user_by( 'id', $this->loop_user->ID ) );
    } );
    add_shortcode( "name", function() {
      return wpep_content_user_profile_format_name( get_user_by( 'id', $this->loop_user->ID ) );
    } );
    add_shortcode( 'email_address', function() {
      return $this->loop_user->user_email;
    } );
    add_shortcode( "site_url", function() {
      return get_site_url();
    } );
    add_shortcode( "date", function() {
      return wpep_format_date();
    } );
    add_shortcode( "date_time", function() {
      return wpep_format_date_with_time();
    } );
    add_shortcode( "number_lessons_completed", function() {
      return $this->loop_lessons_completed_count;
    } );

    add_shortcode( "list_courses_updated", function() {
      if( empty( $this->loop_courses_updated ) )
        return '';

      return wpep_template_get( 'email-template/course-card-list.php', [
        'user_id'              => $this->loop_user->ID,
        'course_list'          => $this->loop_courses_updated,
        'course_object_date'   => 'post_modified'
      ] );
    } );
    add_shortcode( "list_courses_started", function() {
      if( empty( $this->loop_courses_started ) )
        return '';

      return wpep_template_get( 'email-template/course-card-list.php', [
        'user_id'              => $this->loop_user->ID,
        'course_list'          => $this->loop_courses_started,
        'course_user_data_key' => WPEP_USER_COURSE_STARTED
      ] );
    } );
    add_shortcode( "list_courses_completed", function() {
      if( empty( $this->loop_courses_completed ) )
        return '';

      return wpep_template_get( 'email-template/course-card-list.php', [
        'user_id'              => $this->loop_user->ID,
        'course_list'          => $this->loop_courses_completed,
        'course_user_data_key' => WPEP_USER_COURSE_COMPLETED_AT
      ] );
    } );
    add_shortcode( "list_courses_recommended", function() {
      if( empty( $this->loop_courses_recommended ) )
        return '';

      return wpep_template_get( 'email-template/course-card-list.php', [
        'user_id'              => $this->loop_user->ID,
        'course_list'          => $this->loop_courses_recommended
      ] );
    } );
  }

  private function _register_temporary_conditional_shortcodes() {
    add_shortcode( "condition_has_updated_courses", function( $atts, $content ) {
      return !empty( $this->loop_courses_updated ) ? do_shortcode( $content ) : '';
    } );

    add_shortcode( "condition_has_started_courses", function( $atts, $content ) {
      return !empty( $this->loop_courses_started ) ? do_shortcode( $content ) : '';
    } );
    add_shortcode( "condition_has_completed_courses", function( $atts, $content ) {
      return !empty( $this->loop_courses_completed ) ? do_shortcode( $content ) : '';
    } );
    add_shortcode( "condition_has_recommended_courses", function( $atts, $content ) {
      return !empty( $this->loop_courses_recommended ) ? do_shortcode( $content ) : '';
    } );

    add_shortcode( "condition_has_activity", function( $atts, $content ) {
      return !empty( $this->loop_courses_started ) || !empty( $this->loop_courses_completed ) || $this->loop_lessons_completed_count != 0 ? do_shortcode( $content ) : '';
    } );
    add_shortcode( "condition_no_activity", function( $atts, $content ) {
      return empty( $this->loop_courses_started ) && empty( $this->loop_courses_completed ) && $this->loop_lessons_completed_count == 0 ? do_shortcode( $content ) : '';
    } );
  }

  private function _remove_temporary_content_shortcodes() {
    global $shortcode_tags;

    foreach( $shortcode_tags as $shortcode_tag => $shortcode_func )
      if( strpos( $shortcode_tag, 'condition_' ) !== 0 )
        unset( $shortcode_tags[ $shortcode_tag ] );
  }

  private function _remove_temporary_shortcodes() {
    if( empty( $this->_temporary_shortcodes_registered ) )
      return;

    global $shortcode_tags;

    $shortcode_tags = $this->_temporary_shortcodes_registered;

    $this->_temporary_shortcodes_registered = [];
  }

  private function _set_recently_updated_courses() {
    $this->_filter_proxy_setup( [
      'wpep_primary_content_list_args'  => function( $args ) {
        $args['orderby']    = 'modified';
        $args['order']      = 'desc';

        if( !isset( $args['date_query'] ) )
          $args['date_query'] = [];

        $args['date_query'][] = [
          'column'  => 'post_modified',
          'after'   => date( 'Y-m-d H:i:s', strtotime( "-1 week" ) )
        ];

        return $args;
      }
    ] );

    $this->_all_updated_courses = wpep_content_type_course_instance()->library_get_grid_post_list();

    $this->_filter_proxy_remove();
  }

  public function send_email( $user_ids, $email_title, $email_content, $is_simple = false ) {
    if( $user_ids == 0 )
      return false;

    if( !is_array( $user_ids ) )
      $user_ids = [ $user_ids ];

    $this->_pre_register_temporary_shortcodes();

    $this->_register_temporary_conditional_shortcodes();

    $user_list = get_users( [
      'include' =>  $user_ids
    ] );

    $current_user_id = get_current_user_id();

    if( !$is_simple ) {
      if( strpos( $email_content, 'list_courses_started' ) === false
        && strpos( $email_content, 'condition_has_started_courses' ) === false )
        $this->loop_courses_started = null;

      if( strpos( $email_content, 'list_courses_completed' ) === false
        && strpos( $email_content, 'condition_has_completed_courses' ) === false )
        $this->loop_courses_completed = null;

      if( strpos( $email_content, 'list_courses_recommended' ) === false
          && strpos( $email_content, 'condition_has_recommended_courses' ) === false )
        $this->loop_courses_recommended = null;

      if( strpos( $email_content, 'number_lessons_completed' ) === false )
        $this->loop_lessons_completed_count = null;

      if( strpos( $email_content, 'list_courses_updated' ) !== false && empty( $this->_all_updated_courses ) )
        $this->_set_recently_updated_courses();
      else
        $this->_all_updated_courses = [];
    }

    foreach( $user_list as $user ) {
      wp_set_current_user( $user->ID );

      $this->loop_user = $user;

      if( !$is_simple ) {
        $past_started_courses = WPEP_Controller::instance()->userActivity->statistics->get_started_course_list( $user->ID, [ false, strtotime( "-1 week" ) ], true );

        $this->loop_courses_updated = [];

        if( $this->loop_courses_started !== null )
          $this->loop_courses_started   = WPEP_Controller::instance()->userActivity->statistics->get_started_course_list( $user->ID, [ strtotime( "-1 week" ), time() ], true );

        if( $this->loop_courses_completed !== null )
          $this->loop_courses_completed = WPEP_Controller::instance()->userActivity->statistics->get_completed_course_list( $user->ID, [ strtotime( "-1 week" ), time() ] );

        if( $this->loop_lessons_completed_count !== null )
          $this->loop_lessons_completed_count = WPEP_Controller::instance()->userActivity->statistics->get_completed_lesson_count( $this->loop_user->ID, strtotime( "-1 week" ), time() );

        if( $this->loop_courses_recommended !== null )
          $this->loop_courses_recommended = WPEP_Controller::instance()->userActivity->get_recommended_content( $this->loop_user->ID, 4 );

        if( !empty( $this->_all_updated_courses ) ) {
          foreach( $this->_all_updated_courses as $updated_course ) {
            if( !wpep_controller()->contentLibrary->has_access_to_post( $updated_course->ID ) )
              continue;

            if( !in_array( $updated_course->ID, $past_started_courses ) )
              continue;

            $this->loop_courses_updated[] = $updated_course;
          }
        }
      }

      $current_email_content = do_shortcode( $email_content );

      $current_email_content = wpautop( $current_email_content );

      $this->_register_temporary_content_shortcodes();

      $current_email_title   = do_shortcode( $email_title );
      $current_email_content = do_shortcode( $current_email_content );

      $this->_remove_temporary_content_shortcodes();

      WPEP_Controller::instance()->email->send( $user, $current_email_title, $current_email_content, false );
    }

    $this->_remove_temporary_shortcodes();

    wp_set_current_user( $current_user_id );

    $this->loop_courses_started = [];
    $this->loop_courses_completed = [];
    $this->loop_courses_recommended = [];
    $this->loop_lessons_completed_count = 0;

    return true;
  }
  
  public function get_available_shortcode_list( $full_list = true, $user_registration = false ) {
     $response = '<h2>' . __( "General ShortCodes") . '</h2>';
     $response .= '<ul>' .
                    '<li>[name] - ' . __( "The name of the person that receives this email.", "wpep" ) . '</li>' .
                    '<li>[email_address] - ' . __( "The email address of the person that receives this email.", "wpep" ) . '</li>';

     if( $user_registration )
       $response .= '<li>[password] - ' . __( "The password generated for this user account", "wpep" ) . '</li>';

     $response .=   '<li>[date] - ' . __( "The current date when the email is sent.", "wpep" ) . '</li>' .
                    '<li>[date_time] - ' . __( "The current date & time when the email is sent.", "wpep" ) . '</li>' .
                    '<li>[site_url] - ' . __( "The current website url.", "wpep" ) . '</li>';

     if( $user_registration )

     $response .= '</ul>';

     if( !$full_list )
       return $response;

     $response .= '<h2>' . __( "Content ShortCodes") . '</h2>';

     $response .= '<ul>' .
                    '<li>[number_lessons_completed] - ' . __( "The number of lessons completed this week.", "wpep" ) . '</li>' .
                    '<li>[list_courses_completed] - ' . __( "List of Courses Completed this Week.", "wpep" ) . '</li>' .
                    '<li>[list_courses_started] - '  . __( "List of Courses Started this Week.", "wpep" ) . '</li>';
    if( defined( "WPEP_BETA_TESTING" ) ) {
       $response .= '<li>[list_courses_recommended] - '  . __( "List of Courses Recommended, the User must have access to the course, and not start it yet.", "wpep" ) . '</li>' .
                    '<li>[list_courses_updated] - '  . __( "List of Courses that have been updated in the past week.", "wpep" ) . '</li>';
    }

     $response .= '</ul>';
     $response .= '<h2>' . __( "Condition ShortCodes, will display the content within, if the condition is met") . '</h2>';
     $response .= '<ul>' .
                    '<li>[condition_has_completed_courses] - ' . __( "The user has to complete at least 1 course.", "wpep" ) . '</li>' .
                    '<li>[condition_has_started_courses] - ' . __( "The user has to start at least 1 course.", "wpep" ) . '</li>';

    if( defined( "WPEP_BETA_TESTING" ) ) {
       $response .=  '<li>[condition_has_recommended_courses] - ' . __( "The user has at least 1 recommended course.", "wpep" ) . '</li>';
    }
     $response .=  '<li>[condition_has_activity] - ' . __( "The user had activity this week.", "wpep" ) . '</li>' .
                   '<li>[condition_no_activity] - ' . __( "The user didn't have any activity this week.", "wpep" ) . '</li>';

    if( defined( "WPEP_BETA_TESTING" ) ) {
       $response .=  '<li>[condition_has_updated_courses] - ' . __( "There is at least 1 course that has been updated in the past week.", "wpep" ) . '</li>';
    }

     $response .= '</ul>';
    
    return  $response;
  }

}