<?php

namespace WPEP\Entity;

class ActionTracking {

  protected static $_instance;

  public static function instance() {
    if( self::$_instance == null )
      self::$_instance = new self();

    return self::$_instance;
  }

  /**
   * @var \wpdb;
   */
  public $wpdb;
  public $table_action_tracking;
  private $_table_action_tracking = 'wpep_action_tracking';
  private $_table_action_tracking_field_whitelist = [ 'id', 'post_id', 'user_id', 'type', 'segment', 'created_at' ];

  public function __construct() {
    $this->_setup();
  }

  private function _setup() {
    global $wpdb;

    $this->wpdb = $wpdb;

    $this->table_action_tracking = $this->wpdb->prefix . $this->_table_action_tracking;
  }

  public function is_event_tracked( $post_id, $user_id, $type, $segment = false ) {
    return $this->get_first_tracked_event( $post_id, $user_id, $type, $segment ) !== null;
  }

  public function get_first_tracked_event( $post_id, $user_id, $type, $segment = false ) {
    $where_args = [
      'post_id' => $post_id,
      'user_id' => $user_id,
      'type'    => $type
    ];

    if( $segment !== false )
      $where_args['segment'] = $segment;

    $sql = 'SELECT * FROM ' . $this->table_action_tracking . ' ' . $this->_sql_where( $where_args, $this->_table_action_tracking_field_whitelist ) . ' LIMIT 1';

    return $this->wpdb->get_row( $sql );
  }

  public function track_event( $post_id, $user_id, $type, $segment ) {
    $post_id = absint( $post_id );
    $user_id = absint( $user_id );

    $this->wpdb->insert( $this->table_action_tracking, [
      'post_id'    => $post_id,
      'user_id'    => $user_id,
      'type'       => $type,
      'segment'    => $segment,
      'created_at' => current_time( 'mysql' )
    ] );
  }

  public function get_tracked_events_statistics_grouped_by_post( $where ) {
    $sql = 'SELECT post_id as post_id,
                   COUNT( id ) as total_entries,
                   MIN( created_at ) as first_created_at,
                   MAX( created_at ) as last_created_at
              FROM ' . $this->table_action_tracking;

    $sql .= ' ' . $this->_sql_where( $where, $this->_table_action_tracking_field_whitelist );

    $sql .= ' GROUP BY post_id';

    return wpep_utilities_map_object( $this->wpdb->get_results( $sql ), 'post_id' );
  }

  public function get_tracked_events( $where, $args = [] ) {
    $sql = 'SELECT SQL_CALC_FOUND_ROWS *
              FROM ' . $this->table_action_tracking;
    $sql .= ' ' . $this->_sql_where( $where, $this->_table_action_tracking_field_whitelist );

    $sql .= ' ORDER BY created_at DESC';

    if( isset( $args['limit'] ) && $args['limit'] != 0 )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    if( isset( $args['offset'] ) && $args['offset'] != 0 )
      $sql .= ' OFFSET ' . absint( $args[ 'offset' ] );

    if( !isset( $args[ 'count_total' ] ) || $args[ 'count_total' ] == false )
      return $this->wpdb->get_results( $sql );

    return [
      'results' => $this->wpdb->get_results( $sql ),
      'total'   => absint( $this->wpdb->get_var( 'SELECT FOUND_ROWS();' ) )
    ];
  }

  private function _sql_where( $where, $whitelist ) {
    $sql = 'WHERE 1 = 1';

    foreach( $where as $where_column_field => $where_value ) {
      if( !in_array( $where_column_field, $whitelist )
        || $where_value === null )
        continue;

      if( is_array( $where_value ) ) {
        foreach( $where_value as $where_comparison_field => $where_inner_value ) {
          if( $where_comparison_field === 'IN' )
            $sql .= ' AND ' . $where_column_field . ' IN ("' . implode( '","', array_map( 'esc_sql', $where_inner_value ) ) . '")';
          elseif( $where_comparison_field === 'NOT IN' )
            $sql .= ' AND ' . $where_column_field . ' NOT IN ("' . implode( '","', array_map( 'esc_sql', $where_inner_value ) ) . '")';
          elseif( in_array( $where_comparison_field, [ '=', '>', '<', '>=', '<=', '!=' ] ) )
            $sql .= ' AND ' . $where_column_field . ' ' . $where_comparison_field . ' "' . esc_sql( $where_inner_value ) . '"';
          elseif( $where_comparison_field === 'LIKE' )
            $sql .= ' AND ' . $where_column_field . ' LIKE "%' . esc_sql( $where_inner_value ) . '%"';
        }

        continue;
      }

      $sql .= ' AND ' . $where_column_field . ' = "' . esc_sql( $where_value ) . '"';
    }

    return $sql;
  }

  public function get_event_count_by_args( $args ) {
    $sql = 'SELECT COUNT(action_tracking.id) as entries';

    if( isset( $args['group_by'] ) ) {
      if( is_array( $args['group_by'] ) ) {
        $sql .= ', action_tracking.`' . implode( '`, action_tracking.`', $args['group_by'] ) . '`';
      } else {
        $sql .= ', action_tracking.`' . $args['group_by'] . '`';
      }
    }

    $sql .= ' FROM ' . $this->table_action_tracking . ' action_tracking';

    if( isset( $args['post_type'] ) )
      $sql .= ' LEFT JOIN ' . $this->wpdb->posts . ' post_table ON post_table.ID = action_tracking.post_id';

    $sql .= ' WHERE 1 = 1';

    if( isset( $args['post_type'] ) )
      $sql .= ' AND post_table.`post_type` = "' . $args['post_type'] . '"';

    if( isset( $args['type' ] ) ) {
      if( is_array( $args["type"] ) && !empty( $args["type"] ) ) {
        $sql .= ' AND action_tracking.`type` IN ("' . implode( '","', array_map( 'esc_sql', $args["type"] ) ) . '")';
      } else {
        $sql .= ' AND action_tracking.`type` = "' . esc_sql( $args["type"] ) . '"';
      }
    }

    if( isset( $args[ 'segment' ] ) ) {
      if( is_array( $args["segment"] ) && !empty( $args["segment"] ) ) {
        $sql .= ' AND action_tracking.`segment` IN ("' . implode( '","', $args["segment"] ) . '")';
      } else {
        $sql .= ' AND action_tracking.`segment` = "' . esc_sql( $args["segment"] ) . '"';
      }
    }

    if( isset( $args['time_from' ] ) )
      $sql .= ' AND action_tracking.`created_at` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';

    if( isset( $args['time_to' ] ) )
      $sql .= ' AND action_tracking.`created_at` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

    if( isset( $args['group_by'] ) ) {
      $sql .= ' GROUP BY';

      if( is_array( $args['group_by'] ) ) {
        $sql .= ' action_tracking.`' . implode( '`, action_tracking.`', $args['group_by'] ) . '`';
      } else {
        $sql .= ' action_tracking.`' . $args['group_by'] . '`';
      }
    }

    return ( isset( $args['group_by'] ) ? $this->wpdb->get_results( $sql ) : absint( $this->wpdb->get_var( $sql ) ) );
  }

  public function get_unique_cols_by_args( $column, $args ) {
    $sql = 'SELECT ';

    if( $column === 'post_title' )
      $sql .= ' post_table.`post_title`';
    elseif( $column === 'post_id=>post_title' )
      $sql .= ' action_tracking.`post_id`, post_table.`post_title`';
    else
      $sql .= ' action_tracking.`' . $column . '`';

    $sql .= ' FROM ' . $this->table_action_tracking . ' action_tracking';

    if( isset( $args['post_type'] ) || in_array( $column, [ 'post_title', 'post_id=>post_title' ] ) )
      $sql .= ' LEFT JOIN ' . $this->wpdb->posts . ' post_table ON post_table.ID = action_tracking.post_id';

    $sql .= ' WHERE ' . ( in_array( $column, [ 'post_title', 'post_id=>post_title' ] ) ? 'post_table.`post_title`' : 'action_tracking.`' . $column . '`' ) . ' != ""';

    if( isset( $args['post_type'] ) )
      $sql .= ' AND post_table.`post_type` = "' . $args['post_type'] . '"';

    if( isset( $args['type' ] ) ) {
      if( is_array( $args["type"] ) && !empty( $args["type"] ) ) {
        $sql .= ' AND action_tracking.`type` IN ("' . implode( '","', array_map( 'esc_sql', $args["type"] ) ) . '")';
      } else {
        $sql .= ' AND action_tracking.`type` = "' . esc_sql( $args["type"] ) . '"';
      }
    }

    if( isset( $args['segment' ] ) )
      $sql .= ' AND action_tracking.`segment` = "' . esc_sql( $args["segment"] ) . '"';

    if( isset( $args['time_from' ] ) )
      $sql .= ' AND action_tracking.`created_at` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';

    if( isset( $args['time_to' ] ) )
      $sql .= ' AND action_tracking.`created_at` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

    $sql .= ' GROUP BY ';

    if( $column === 'post_title' )
      $sql .= ' post_table.`post_title`';
    elseif( $column === 'post_id=>post_title' )
      $sql .= ' action_tracking.`post_id`';
    else
      $sql .= ' action_tracking.`' . $column . '`';

    if( $column === 'post_id=>post_title' ) {
      $data = $this->wpdb->get_results( $sql );

      if( empty( $data ) )
        return [];

      $resp = [];

      foreach( $data as $d )
        $resp[ $d->post_id ] = $d->post_title;

      return $resp;
    }

    return $this->wpdb->get_col( $sql );
  }

}