<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;

class Challenge extends Skeleton {

  public $current_item      = 'course';
  public $current_item_id   = null;

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = true;

  private $_course_sections = [];

  /**
   * Standard constructor.
   * @param $current_course_id
   */
  public function __construct( $current_course_id ) {
    $this->section_slug = wpep_get_setting( 'course-challenge-section-slug' );

    $this->_skeleton_init( $current_course_id );

    if( isset( $_GET[ $this->section_slug ] ) )
      $this->current_item_id = $_GET[ $this->section_slug ];

    if( !is_numeric( $this->current_item_id ) )
      $this->current_item_id = WPEP_Controller::instance()->userActivity->get_delivered_course_section_id( $current_course_id );
  }

  public function has_sidebar_content() {
    return true;
  }

  public function get_alias() {
    return "challenge";
  }

  public function init() {
    parent::init();

    $this->_course_sections = WPEP_Controller::instance()->userActivity->get_course_sections( $this->course_id, get_current_user_id() );

    add_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_true' ], 100 );
    add_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
    add_action( 'wpep_single_sidebar_after', [ $this, '_after_sidebar_content'] );
  }

  public function close() {
    parent::close();

    $this->_course_sections = [];

    remove_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_true' ], 100 );
    remove_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
    remove_action( 'wpep_single_sidebar_after', [ $this, '_after_sidebar_content'] );
  }

  public function content() {
    if( empty( $this->_course_sections ) )
      return '';

    if( !isset( $this->_course_sections[ $this->current_item_id ] ) )
      return '';

    $sharing_link = get_post_meta( $this->course_id, 'wpep_social_share_link', true );

    if( $sharing_link == '' )
      $sharing_link = remove_query_arg( $this->section_slug, wpep_get_current_url() );

    $args = [
      'sharing_link'          => $sharing_link,
      'course_title'          => get_the_title( $this->course_id ),
      'course_id'             => $this->course_id,
      'course_read_only'      => $this->read_only,
      'course_image'          => wpep_course_thumbnail_url( $this->course_id ),
      'section_id'            => $this->current_item_id,
      'section_title'         => $this->_course_sections[ $this->current_item_id ]['title'],
      'section_content'       => $this->_course_sections[ $this->current_item_id ]['content'],
      'section_lessons'       => $this->_course_sections[ $this->current_item_id ]['lessons']
    ];

    $args = apply_filters( 'wpep_course_single_content_challenge_args', $args );

    if( $this->top_level_request && wpep_course_has_discussion( $this->course_id ) )
      add_filter( 'wpep_course_single_content_challenge_html', [ $this, '_content_attach_discussion_for_section' ], 1 );

    $response = apply_filters(
      'wpep_course_single_content_challenge_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/challenge-section-content.php', $args ),
      $args
    );

    if( $this->top_level_request && wpep_course_has_discussion( $this->course_id ) )
      remove_filter( 'wpep_course_single_content_challenge_html', [ $this, '_content_attach_discussion_for_section' ], 1 );

    return $response;
  }

  public function display_sidebar_content( $course_id = false ) {
    $course_id = ( $course_id == false ? get_the_ID() : $course_id );

    if( $course_id != $this->course_id )
      return;

    echo '<div class="wpep-widget wpep-course-widget-primary" data-wpep-breakpoint="768">';
    echo    '<div class="wpep-widget-wrapper">';
    echo      '<div class="challenge-course-thumbnail-widget">';
    echo        '<img src="' . wpep_course_thumbnail_url( $this->course_id ) . '"/>';
    echo        '<h2>' . get_the_title( $this->course_id ) . '</h2>';
    echo      '</div>';
    echo    '</div>';
    echo '</div>';

    echo '<div class="wpep-widget wpep-widget-challenge-navigation">';
    echo    '<div class="wpep-widget-wrapper">';
    echo      '<ul>';

    $current_permalink = wpep_get_current_url();
    $current_permalink = remove_query_arg( $this->section_slug, $current_permalink );

    foreach( $this->_course_sections as $course_section_id => $course_section ) {
      $is_disabled = ( isset( $course_section['options'] ) && isset( $course_section['options']['is_disabled'] ) && $course_section['options']['is_disabled'] );

      $class = 'wpep-lesson-complete-trigger-watcher';

      if( $course_section_id == $this->current_item_id )
        $class .= ' wpep-is-active';

      if( $is_disabled )
        $class .= ' wpep-section-disabled';

      echo '<li class="' . $class . '" data-wpep-is-completed="' . intval( $course_section['is_completed'] ) . '" data-wpep-section-id="' . $course_section_id . '">';
      echo    '<a href="' . ( $is_disabled ? $current_permalink : add_query_arg( $this->section_slug, $course_section_id, $current_permalink ) ) . '">' . $course_section['title'] . '</a>';
      echo    '<span class="wpep-section-status-icon"></span>';
      echo '</li>';

    }

    echo      '</ul>';
    echo    '</div>';
    echo '</div>';

    add_filter( 'is_active_sidebar', [ $this, '_return_false' ], 100 );
  }

  public function _after_sidebar_content() {
    remove_filter( 'is_active_sidebar', [ $this, '_return_false' ], 100 );
  }

}