<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;

abstract class Skeleton {

  public $top_level_request = false;
  public $course_id;
  public $course_has_index_page = false;
  public $course_has_redirect_on_completion = false;
  public $read_only = false;

  public $current_item    = null;
  public $current_item_id = null;
  public $lesson_slug     = 'lesson';
  public $section_slug    = 'section';

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = true;

  public function init() {
    if( $this->top_level_request == false )
      add_filter( 'wpep_course_permalink', [ $this, 'course_permalink' ], 20, 2 );
  }

  public function close() {
    if( $this->top_level_request == false )
      remove_filter( 'wpep_course_permalink', [ $this, 'course_permalink' ], 20 );
  }

  abstract public function content();
  
  abstract public function get_alias();

  public function has_sidebar_content() {
    return false;
  }

  public function _skeleton_init( $course_id ) {
    $this->course_id = $course_id;
    $this->read_only = wpep_post_read_only( $this->course_id );
  }

  public function get_header() {
    $progress = WPEP_Controller::instance()->userActivity->course_progress_percentage( $this->course_id, get_current_user_id() );

    $args = [
      'course_id'         => $this->course_id,
      'course_thumbnail'  => get_post_meta( $this->course_id, 'wpep_icon_image', true),
      'course_title'      => get_the_title( $this->course_id ),
      'course_sub_title'  => wpep_format_content( get_post_meta( $this->course_id , 'wpep_course_description', true) ),
      'progress_percent'  => $progress,
      'progress_label'    => number_format( $progress > 99 ? 100 : $progress, 2 ),
      'progressbar_title' => wpep_get_setting_translated( 'opt-course-progressbar' ),
      'show_progress_bar' => wpep_course_has_progress_bar( $this->course_id, $this->read_only )
    ];

    $args = apply_filters( 'wpep_course_single_content_header_args', $args );

    return apply_filters(
      'wpep_single_content_header_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/header.php', $args ),
      $args
    );
  }

  public function _set_item_data() {
    if( isset( $_GET[$this->lesson_slug] ) ) {
      $this->current_item     = $this->lesson_slug;
      $this->current_item_id  = intval( $_GET[ $this->lesson_slug ] );
    } else if( ! $this->course_has_index_page ) {
      $this->current_item     = $this->lesson_slug;
      $this->current_item_id  = WPEP_Controller::instance()->userActivity->get_delivered_course_lesson_id( $this->course_id, get_current_user_id() );
    }
  }

  public function _init_item_based_course( $set_item_data = true ) {
    if( $set_item_data )
      $this->_set_item_data();

    if( isset( $_GET['complete-lesson'] ) && $this->current_item == $this->lesson_slug ) {
      if( !wp_verify_nonce( $_GET['complete-lesson'], 'complete-lesson' ) )
        wp_nonce_ays( 'complete-lesson' );

      WPEP_Controller::instance()->userActivity->set_lesson_status(
        $this->current_item_id, 1, get_current_user_id()
      );

      if( $this->course_has_redirect_on_completion ) {
        $next_lesson_id = WPEP_Controller::instance()->userActivity->get_course_next_lesson_id( $this->current_item_id, get_current_user_id() );

        if( $next_lesson_id != 0 ) {
          wpep_redirect( wpep_course_lesson_permalink( $next_lesson_id, $this->course_id ) );
          exit;
        }
      }
    }

    if( wpep_get_setting_flag( 'enable-lesson-incomplete-option' ) && isset( $_GET['incomplete-lesson'] ) && $this->current_item == $this->lesson_slug ) {
      if( !wp_verify_nonce( $_GET['incomplete-lesson'], 'incomplete-lesson' ) )
        wp_nonce_ays( 'incomplete-lesson' );

      WPEP_Controller::instance()->userActivity->set_lesson_status(
        $this->current_item_id, 0, get_current_user_id()
      );
    }

    if( wpep_assessments_enabled() && $this->current_item == $this->lesson_slug ) {

      $assessment_requirements = wpep_user_assessment_requirements_for_lesson( $this->current_item_id );

      if( !empty( $assessment_requirements ) ) {
        $assessment_permalink = get_permalink( array_shift( $assessment_requirements ) );

        wpep_redirect(
          add_query_arg(
            'redirect-from-lesson',
            '1',
            $assessment_permalink
          )
        );
        exit;
      }
    }
  }

  public function _content_attach_discussion_for_course( $response ) {
    if( !is_user_logged_in() )
      return $response;

    if( !wpep_content_type_course_instance()->discussionIntegration->current_user_can_view( $this->course_id ) )
      return $response;

    $response .= $this->_content_attach_get_discussion_content( [
      'course_id'   => $this->course_id,
      'user_id'     => get_current_user_id()
    ] );

    return $response;
  }

  public function _content_attach_discussion_for_section( $response ) {
    if( !is_user_logged_in() )
      return $response;

    if( !wpep_content_type_course_instance()->discussionIntegration->current_user_can_view( $this->course_id ) )
      return $response;

    $response .= $this->_content_attach_get_discussion_content( [
      'course_id'   => $this->course_id,
      'section_id'  => $this->current_item_id,
      'user_id'     => get_current_user_id()
    ] );

    return $response;
  }

  public function _content_attach_discussion_for_lesson( $response ) {
    if( !is_user_logged_in() )
      return $response;

    if( !wpep_content_type_course_instance()->discussionIntegration->current_user_can_view( $this->course_id ) )
      return $response;

    $response .= $this->_content_attach_get_discussion_content( [
      'course_id'   => $this->course_id,
      'lesson_id'   => $this->current_item_id,
      'user_id'     => get_current_user_id()
    ] );

    return $response;
  }

  private function _content_attach_get_discussion_content( $identifier_args ) {
    $discussion_id = wpep_content_type_course_instance()->discussionIntegration->discussion_id_by_identifier_args( $identifier_args, $this->course_id );

    return '<div class="wpep-course-discussion-container">' .
              wpep_content_type_course_instance()->discussionIntegration->get_comments_template( $discussion_id ) .
           '</div>';
  }

  public function _return_true() {
    return true;
  }

  public function _return_false() {
    return false;
  }

  public function course_permalink( $course_permalink, $course_id ) {
    if( $this->top_level_request == true )
      return $course_permalink;

    if( $this->course_id != $course_id )
      return $course_permalink;

    $link = wpep_get_current_url();

    $link = remove_query_arg( $this->lesson_slug, $link );
    $link = remove_query_arg( $this->section_slug, $link );
    $link = remove_query_arg( 'complete-lesson', $link );
    $link = remove_query_arg( 'incomplete-lesson', $link );

    return $link;
  }

}