<?php

namespace WPEP\Entity\CourseType;

use WPEP\Controller as WPEP_Controller;

class Video extends Skeleton {

  public $current_item    = 'course';
  public $current_item_id = 0;

  public $has_next_suggestions = true;
  public $has_next_suggestions_after_completion = true;

  private $_course_sections = [];

  /**
   * Video constructor.
   * @param $current_course_id
   */
  public function __construct( $current_course_id ) {
    $this->_skeleton_init( $current_course_id );

    $this->course_has_index_page             = wpep_get_setting_flag( 'course-video-allow-index-page' );
    $this->course_has_redirect_on_completion = wpep_get_setting_flag( 'course-video-redirect-on-completion' );

    $course_has_index_page = get_post_meta( $this->course_id, 'wpep_video_course_overview_page', true );

    if( $course_has_index_page != 'default' )
      $this->course_has_index_page = $course_has_index_page;

    $this->_set_item_data();
  }

  public function has_sidebar_content() {
    return ( $this->current_item != $this->lesson_slug );
  }

  public function get_alias() {
    return "video";
  }

  public function init() {
    parent::init();

    $this->_course_sections = WPEP_Controller::instance()->userActivity->get_course_sections( $this->course_id, get_current_user_id() );

    do_action( 'wpep_course_single_content_video_coordinator_before_init', $this );

    if( isset( $_GET[$this->lesson_slug] ) ) {
      $this->current_item     = $this->lesson_slug;
      $this->current_item_id  = intval( $_GET[$this->lesson_slug] );
    }

    $this->_init_item_based_course( false );

    do_action( 'wpep_course_single_content_video_coordinator_after_init', $this );

    if( $this->has_sidebar_content()) {
      add_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_false' ], 50 );
    } else {
      add_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_true' ], 50 );
      add_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
    }
  }

  public function close() {
    parent::close();

    $this->_course_sections = [];

    if( $this->has_sidebar_content() ) {
      remove_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_false' ], 50 );
    } else {
      remove_filter( 'wpep_single_content_sidebar_active', [ $this, '_return_true' ], 50 );
      remove_action( 'wpep_single_sidebar_before', [ $this, 'display_sidebar_content' ] );
    }
  }

  public function content() {
    if( $this->current_item == $this->lesson_slug )
      return $this->get_content_lesson( $this->current_item_id );

    return $this->get_content();
  }

  public function get_content() {
    $args = [
      'course_id'        => $this->course_id,
      'course_read_only' => $this->read_only,
      'course_sections'  => $this->_course_sections,
      'course_title'     => get_the_title( $this->course_id ),
      'course_content'   => wpep_format_content( get_post_meta( $this->course_id , 'wpep_course_description', true) ),
      'course_status'    => $this->_get_course_status()
    ];

    $args = apply_filters( 'wpep_course_single_content_video_args', $args );

    return apply_filters(
      'wpep_single_content_video_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/video-content.php', $args ),
      $args
    );
  }

  private function _get_course_status() {
    $completed_counter = 0;

    foreach( $this->_course_sections as $course_section )
      if( $course_section['is_completed'] )
        $completed_counter++;

    if( $completed_counter == count( $this->_course_sections ) )
      return 'completed';

    if( $completed_counter != 0 )
      return 'progress';

    foreach( $this->_course_sections as $course_section )
      foreach( $course_section['lessons'] as $course_section_lesson )
        if( $course_section_lesson['is_completed'] == 1 )
          return 'progress';

    return 'pending';
  }

  public function get_content_lesson( $lesson_id ) {
    $course_lesson = WPEP_Controller::instance()->userActivity->get_course_lesson( $lesson_id, get_current_user_id(), true );

    if( $course_lesson == null )
      return '';

    if( isset( $this->_course_sections[ $course_lesson->section_id ][ 'lessons' ][ $course_lesson->id ][ 'options' ][ 'is_disabled' ] )
        && $this->_course_sections[ $course_lesson->section_id ][ 'lessons' ][ $course_lesson->id ][ 'options' ][ 'is_disabled' ] )
      return '';

    $lesson_options = ( is_string( $course_lesson->options ) && $course_lesson->options !== '' ? json_decode( $course_lesson->options ) : $course_lesson->options );
    $lesson_content = wpautop( $course_lesson->content );

    if( isset( $lesson_options->downloads_list ) && !empty( $lesson_options->downloads_list ) ) {
      $downloads_content = WPEP_Controller::instance()->template->get_template( 'utilities/downloads-list.php', [
        'downloads_list'  => $lesson_options->downloads_list,
        'title'           => ''
      ] );

      if( wpep_get_setting( 'course-downloads-position' ) == 'top' )
        $lesson_content = $downloads_content . $lesson_content;
      else
        $lesson_content .= $downloads_content;

      unset( $downloads_content );
    }

    $args = [
      'lesson_list'           => $this->_course_sections[ $course_lesson->section_id ]['lessons'],
      'course_read_only'      => $this->read_only,
      'course_id'             => $this->course_id,
      'course_name'           => get_the_title( $this->course_id ),
      'lesson_id'             => $course_lesson->id,
      'lesson_title'          => $course_lesson->title,
      'lesson_content'        => $lesson_content,
      'lesson_options'        => $lesson_options,
      'lesson_is_completed'   => $course_lesson->is_completed,
      'next_lesson_id'        => $course_lesson->next_lesson_id,
      'next_lesson_title'     => $course_lesson->next_lesson_title,
      'previous_lesson_id'    => $course_lesson->previous_lesson_id,
      'previous_lesson_title' => $course_lesson->previous_lesson_title,
      'has_index_page'        => $this->course_has_index_page
    ];

    if( $course_lesson->previous_lesson_id !== null
        && isset( $this->_course_sections[ $course_lesson->previous_lesson_section_id ][ 'lessons' ][ $course_lesson->previous_lesson_id ][ 'options' ][ 'is_disabled' ] )
        && $this->_course_sections[ $course_lesson->previous_lesson_section_id ][ 'lessons' ][ $course_lesson->previous_lesson_id ][ 'options' ][ 'is_disabled' ]
    )
      $args[ 'previous_lesson_id' ] = null;

    if( $course_lesson->next_lesson_id !== null
        && isset( $this->_course_sections[ $course_lesson->next_lesson_section_id ][ 'lessons' ][ $course_lesson->next_lesson_id ][ 'options' ][ 'is_disabled' ] )
        && $this->_course_sections[ $course_lesson->next_lesson_section_id ][ 'lessons' ][ $course_lesson->next_lesson_id ][ 'options' ][ 'is_disabled' ]
    )
      $args[ 'next_lesson_id' ] = null;

    $args = apply_filters( 'wpep_course_single_content_video_lesson_args', $args );

    if( $this->top_level_request && wpep_course_has_discussion( $this->course_id ) )
      add_filter( 'wpep_single_content_video_lesson_html', [ $this, '_content_attach_discussion_for_lesson' ], 1 );

    $response = apply_filters(
      'wpep_single_content_video_lesson_html',
      WPEP_Controller::instance()->template->get_template( 'course-single/video-lesson-content.php', $args ),
      $args
    );

    if( $this->top_level_request && wpep_course_has_discussion( $this->course_id ) )
      remove_filter( 'wpep_single_content_video_lesson_html', [ $this, '_content_attach_discussion_for_lesson' ], 1 );

    return $response;
  }

  public function display_sidebar_content( $course_id = false ) {
    $course_id = ( $course_id == false ? get_the_ID() : $course_id );

    if( $course_id != $this->course_id )
      return;

    $thumbnail = wpep_course_thumbnail_url( $this->course_id );

    echo '<div class="wpep-widget wpep-course-widget-primary">';
    echo    '<div class="wpep-widget-wrapper">';
    echo      '<div class="video-course-thumbnail-widget">';
    echo        '<img src="' . $thumbnail . '"/>';
    echo        '<h2>' . get_the_title( $this->course_id ) . '</h2>';
    echo      '</div>';
    echo    '</div>';
    echo '</div>';

    if( wpep_course_has_progress_bar( $this->course_id ) ) {
      echo '<div class="wpep-widget">';
      echo    '<div class="wpep-widget-wrapper">';
      echo      '<div class="video-course-progress-widget">';
      echo        '<h2>' . wpep_get_setting_translated( 'course-video-course-progress' ) . '</h2>';

      echo         wpep_progress_bar( WPEP_Controller::instance()->userActivity->course_progress_percentage( $this->course_id, get_current_user_id() )  );
      echo      '</div>';
      echo    '</div>';
      echo '</div>';
    }

  }

}