<?php

namespace WPEP\Entity;

class Discussion {

  protected static $_instance;

  public static function instance() {
    if( self::$_instance == null )
      self::$_instance = new self();

    return self::$_instance;
  }

  /**
   * @var \wpdb;
   */
  public $wpdb;
  public $table_discussion;
  public $table_discussion_data;
  public $table_discussion_comment;

  private $_table_discussion         = 'wpep_discussion';
  private $_table_discussion_data    = 'wpep_discussion_data';
  private $_table_discussion_comment = 'wpep_discussion_comment';

  private $_table_discussion_field_whitelist = [ 'id', 'post_id', 'identifier', 'namespace' ];
  private $_table_discussion_comment_field_whitelist = [ 'id', 'parent_id', 'discussion_id', 'user_id', 'content', 'created_at', 'last_activity', 'is_pinned', 'type' ];

  public function __construct() {
    $this->_setup();
  }

  private function _setup() {
    global $wpdb;

    $this->wpdb = $wpdb;

    $this->table_discussion         = $this->wpdb->prefix . $this->_table_discussion;
    $this->table_discussion_data    = $this->wpdb->prefix . $this->_table_discussion_data;
    $this->table_discussion_comment = $this->wpdb->prefix . $this->_table_discussion_comment;
  }

  public function discussion_id_by_identifier( $identifier, $post_id = 0, $namespace = '' ) {
    $discussion = $this->get_discussion( [
      'identifier'  => [ '=' => $identifier ]
    ] );

    if( $discussion === null ) {
      $this->wpdb->insert( $this->table_discussion, [
        'post_id'     => $post_id,
        'identifier'  => $identifier,
        'namespace'   => $namespace,
        'created_at'  => current_time( 'mysql' )
      ] );

      return intval( $this->wpdb->insert_id );
    }

    return intval( $discussion->id );
  }

  public function add_discussion_comment( $data, $update_activity_flags = true ) {
    $this->wpdb->insert( $this->table_discussion_comment, $data );

    $discussion_comment_id = $this->wpdb->insert_id;

    if( $update_activity_flags && isset( $data['discussion_id'] ) ) {
      $this->set_discussion_user_data( WPEP_DISCUSSION_USER_DATA_LAST_VIEW, time(), $data['discussion_id'], $data['user_id'] );

      $this->wpdb->update( $this->table_discussion, [
        'last_activity' => current_time( 'mysql' )
      ], [
        'id' => $data['discussion_id']
      ] );
    }

    return $discussion_comment_id;
  }

  public function get_discussion( $where ) {
    $sql = 'SELECT *
              FROM ' . $this->table_discussion;
    $sql .= ' ' . $this->_sql_where( $where, $this->_table_discussion_field_whitelist );

    return $this->wpdb->get_row( $sql );
  }

  public function get_discussions( $where, $args = [] ) {
    $sql = 'SELECT SQL_CALC_FOUND_ROWS *
              FROM ' . $this->table_discussion;
    $sql .= ' ' . $this->_sql_where( $where, $this->_table_discussion_field_whitelist );

    $sql .= ' ORDER BY last_activity DESC';

    if( isset( $args['limit'] ) && $args['limit'] != 0 )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    if( isset( $args['offset'] ) && $args['offset'] != 0 )
      $sql .= ' OFFSET ' . absint( $args[ 'offset' ] );

    if( !isset( $args[ 'count_total' ] ) || $args[ 'count_total' ] == false )
      return $this->wpdb->get_results( $sql );

    return [
      'results' => $this->wpdb->get_results( $sql ),
      'total'   => absint( $this->wpdb->get_var( 'SELECT FOUND_ROWS();' ) )
    ];
  }

  public function get_discussion_comment( $where ) {
    $sql = 'SELECT * FROM ' . $this->table_discussion_comment;
    $sql .= ' ' . $this->_sql_where( $where, $this->_table_discussion_comment_field_whitelist );

    return $this->wpdb->get_row( $sql );
  }

  public function delete_discussion_comment_by_id( $discussion_comment_id ) {
    $discussion_comment_id = absint( $discussion_comment_id );

    $this->wpdb->delete( $this->table_discussion_comment, [ 'parent_id' => $discussion_comment_id ] );
    $this->wpdb->delete( $this->table_discussion_comment, [ 'id' => $discussion_comment_id ] );
  }

  public function pin_discussion_comment_by_id( $discussion_comment_id, $status = 1 ) {
    $this->wpdb->update(
      $this->table_discussion_comment,
      [
        'is_pinned' => absint( $status )
      ],
      [
        'id' => absint( $discussion_comment_id )
      ]
    );
  }

  public function delete_discussion_comment( $where ) {
    foreach( $where as $key => $value ) {
      if( !in_array( $key, $this->_table_discussion_comment_field_whitelist ) )
        unset( $where[ $key ] );

      $where[ $key ] = esc_sql( $value );
    }

    if( empty( $where ) )
      return false;

    return $this->wpdb->delete( $this->table_discussion_comment, $where );
  }

  /**
   * If WordPress will ever work smoothly with procedures, we want to refactor this.
   * @param $discussion_id
   * @param $search_term
   * @param int|false $filter_by_user_id
   * @return array
   */
  public function search_for_discussion_comment_ids( $discussion_id, $search_term, $filter_by_user_id = false ) {
    $sql = 'SELECT id, 
                   parent_id 
              FROM `' . $this->table_discussion_comment . '`
             WHERE discussion_id = ' . $discussion_id;

    if( $search_term != '' )
      $sql .= ' AND content LIKE "%' . esc_sql( $search_term ) . '%"';

    if( $filter_by_user_id !== false )
      $sql .= ' AND user_id = ' . absint( $filter_by_user_id );

    $results = $this->wpdb->get_results( $sql );

    if( empty( $results ) )
      return [];

    $response = [];
    $required_parent_ids = [];

    foreach( $results as $result ) {
      if( !isset( $response[ $result->parent_id ] ) )
        $response[ $result->parent_id ] = [];

      $response[ $result->parent_id ][] = $result->id;

      if( $result->parent_id != 0 )
        $required_parent_ids[] = $result->parent_id;
    }

    if( empty( $required_parent_ids ) )
      return $response;

    while( !empty( $required_parent_ids ) ) {
      $sql = 'SELECT id, 
                     parent_id 
                FROM `' . $this->table_discussion_comment . '`
               WHERE discussion_id = ' . $discussion_id . '
                 AND id IN (' . implode( ",", $required_parent_ids ) . ')';

      $results = $this->wpdb->get_results( $sql );

      if( empty( $results ) )
        return $response;

      $required_parent_ids = [];

      foreach( $results as $result ) {
        if( !isset( $response[ $result->parent_id ] ) )
          $response[ $result->parent_id ] = [];

        $response[ $result->parent_id ][] = $result->id;

        if( $result->parent_id != 0 )
          $required_parent_ids[] = $result->parent_id;
      }
    }

    return $response;
  }

  public function get_discussion_comments( $where, $args = [] ) {
    $sql = 'SELECT SQL_CALC_FOUND_ROWS * FROM `' . $this->table_discussion_comment . '`';

    $sql .= ' ' . $this->_sql_where( $where, $this->_table_discussion_comment_field_whitelist, $this->table_discussion_comment );

    $sql .= ' ORDER BY';

    if( !isset( $args[ 'do_not_order_by_pinned'] ) || $args[ 'do_not_order_by_pinned'] == false )
      $sql .= ' is_pinned DESC,';

    $sql .= ' ' . ( isset( $args['order_by'] ) ? $args['order_by'] : 'last_activity' );
    $sql .= ' ' . ( isset( $args['order'] ) && $args['order'] == 'ASC' ? 'ASC' : 'DESC' );

    if( isset( $args['limit'] ) && $args['limit'] != 0 )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    if( isset( $args['offset'] ) && $args['offset'] != 0 )
      $sql .= ' OFFSET ' . absint( $args[ 'offset' ] );

    if( !isset( $args[ 'count_total' ] ) || $args[ 'count_total' ] == false )
      return $this->wpdb->get_results( $sql );

    return [
      'results' => $this->wpdb->get_results( $sql ),
      'total'   => absint( $this->wpdb->get_var( 'SELECT FOUND_ROWS();' ) )
    ];
  }

  private function _sql_where( $where, $whitelist, $table_name = false ) {
    $sql = 'WHERE 1 = 1';

    foreach( $where as $where_column_field => $where_value ) {
      if( !in_array( $where_column_field, $whitelist )
        || $where_value === null )
        continue;

      if( is_array( $where_value ) ) {
        foreach( $where_value as $where_comparison_field => $where_inner_value ) {
          if( $where_comparison_field === 'IN' )
            $sql .= ' AND `' . $where_column_field . '` IN ("' . implode( '","', array_map( 'esc_sql', $where_inner_value ) ) . '")';
          elseif( $where_comparison_field === 'NOT IN' )
            $sql .= ' AND `' . $where_column_field . '` NOT IN ("' . implode( '","', array_map( 'esc_sql', $where_inner_value ) ) . '")';
          elseif( in_array( $where_comparison_field, [ '=', '>', '<', '>=', '<=', '!=' ] ) )
            $sql .= ' AND `' . $where_column_field . '` ' . $where_comparison_field . ' "' . esc_sql( $where_inner_value ) . '"';
          elseif( $where_comparison_field === 'LIKE' )
            $sql .= ' AND `' . $where_column_field . '` LIKE "%' . esc_sql( $where_inner_value ) . '%"';
          elseif( $where_comparison_field === 'relevant' && $where_column_field == 'parent_id' && $table_name != false ) {
            $sql .= ' AND `parent_id` IN ( ';
            $sql .=   'SELECT DISTINCT `parent_id`
                         FROM `' . $table_name . '` 
                        WHERE `user_id` = ' . $where_inner_value . '
                        UNION
                       SELECT `id`
                         FROM `' . $table_name . '` 
                        WHERE `user_id` = ' . $where_inner_value;
            $sql .= ' )';

            $sql .= ' AND `user_id` != ' . $where_inner_value;
          }
        }

        continue;
      }

      $sql .= ' AND `' . $where_column_field . '` = "' . esc_sql( $where_value ) . '"';
    }

    return $sql;
  }

  public function get_discussion_user_data_value( $key = false, $discussion_id = false, $user_id = false, $single_row = true ) {
    if( $single_row ) {
      $user_data = $this->get_discussion_user_data( $key, $discussion_id, $user_id, true );

      if( $user_data !== null )
        return $user_data->value;

      return null;
    }

    $response = [];

    foreach( $this->get_discussion_user_data( $key, $discussion_id, $user_id, false ) as $user_data )
      $response[$user_data->id] = $user_data->value;

    return $response;
  }

  public function get_discussion_user_data( $key = false, $discussion_id = false, $user_id = false, $single_row = true, $value = false ) {
    if ( $single_row ) {
      $user_id = ( $user_id === false ? get_current_user_id() : $user_id );

      if ($user_id == 0 )
        return null;
    }

    $sql = 'SELECT * FROM ' . $this->table_discussion_data . ' dud WHERE 1 = 1 ';

    if( $user_id != 0 )
      $sql .= ' AND dud.user_id = ' . intval( $user_id );

    if( $key !== false )
      $sql .= ' AND dud.key = "' . wpep_utilities_sanitize_input( $key ) . '"';

    if( $discussion_id !== false )
      $sql .= ' AND dud.discussion_id  = ' . intval( $discussion_id );

    if( $value !== false ) {
      if( is_array( $value ) && isset( $value[0] ) && isset( $value[1] )  ) {
        if( $value[0] !== false )
          $sql .= ' AND dud.value  >= ' . intval( $value[0] );
        if( $value[1] !== false )
          $sql .= ' AND dud.value  <= ' . intval( $value[1] );
      } else {
        $sql .= ' AND dud.value  = ' . intval( $value );
      }
    }

    return ( $single_row ? $this->wpdb->get_row( $sql ) : $this->wpdb->get_results( $sql ) );
  }

  public function set_discussion_user_data( $key, $value, $discussion_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return false;

    $discussion_id  = intval( $discussion_id );

    $existent_entry = $this->get_discussion_user_data( $key, $discussion_id, $user_id, true );

    $value_updated = true;

    if( $existent_entry === null )
      $this->add_discussion_user_data( $key, $value, $discussion_id, $user_id );
    else if( $existent_entry->value != $value ) {
      global $wpdb;

      $wpdb->update(
        $this->table_discussion_data,
        [ 'value' => $value ],
        [ 'id'    => $existent_entry->id ]
      );
    } else {
      $value_updated = false;
    }

    if( !$value_updated )
      return true;

    do_action( "wpep_user_set_discussion_data", $key, $value, $discussion_id, $user_id, $existent_entry );

    return true;
  }

  public function add_discussion_user_data( $key, $value, $discussion_id, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return;

    $discussion_id  = intval( $discussion_id );

    $this->wpdb->insert( $this->table_discussion_data, [
      'user_id'       => $user_id,
      'discussion_id' => $discussion_id,
      'key'           => $key,
      'value'         => $value,
    ] );

    do_action( "wpep_user_add_discussion_data", $key, $value, $discussion_id, $user_id );
  }

  public function remove_discussion_user_data( $key = null, $value = null, $discussion_id = null, $user_id = false ) {
    if( $user_id === false )
      $user_id = get_current_user_id();

    if( $user_id == 0 )
      return null;

    $delete_where = [ 'user_id' => $user_id ];

    if( $key !== null )
      $delete_where['key'] = esc_sql( $key );

    if( $value !== null )
      $delete_where['value'] = esc_sql( $value );

    if( $discussion_id !== null )
      $delete_where['discussion_id'] = intval( $discussion_id );

    $this->wpdb->delete( $this->table_discussion_data, $delete_where );

    do_action( "wpep_user_remove_discussion_data", $key, $value, $discussion_id, $user_id );
  }

  public function get_discussion_user_ids( $discussion_id, $args = [] ) {
    $sql = 'SELECT SQL_CALC_FOUND_ROWS user_id
		          FROM `' . $this->table_discussion_comment . '`';

    $sql .= ' ' . $this->_sql_where( [ 'discussion_id' => absint( $discussion_id ) ], $this->_table_discussion_comment_field_whitelist );

    $sql .= ' GROUP BY user_id';

    if( isset( $args[ 'offset' ] ) && isset( $args[ 'limit' ] ) && $args[ 'limit' ] != false )
      $sql .= ' LIMIT ' . absint( $args[ 'offset' ] ) . ',' . absint( $args[ 'limit' ] );
    elseif( isset( $args[ 'limit' ] ) && $args[ 'limit' ] !== false )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    $response = [
      'ids'   => $this->wpdb->get_col( $sql ),
      'total' => absint( $this->wpdb->get_var( 'SELECT FOUND_ROWS();' ) )
    ];

    return $response;
  }

  public function get_discussions_by_user_id( $user_id, $args = [] ) {
    $sql = 'SELECT SQL_CALC_FOUND_ROWS discussion_id
		          FROM `' . $this->table_discussion_comment . '`
             WHERE user_id = ' . absint( $user_id ) . '
          GROUP BY discussion_id';

    if( isset( $args[ 'offset' ] ) && isset( $args[ 'limit' ] ) && $args[ 'limit' ] != false )
      $sql .= ' LIMIT ' . absint( $args[ 'offset' ] ) . ',' . absint( $args[ 'limit' ] );
    elseif( isset( $args[ 'limit' ] ) && $args[ 'limit' ] !== false )
      $sql .= ' LIMIT ' . absint( $args[ 'limit' ] );

    $response = [
      'ids'   => $this->wpdb->get_col( $sql ),
      'total' => absint( $this->wpdb->get_var( 'SELECT FOUND_ROWS();' ) )
    ];

    return $response;
  }

  /**
   * @param \WPEP_Discussion_Integration[] $discussion_integrations
   * @param $limit
   * @return array
   */
  public function get_email_notifications_data( $discussion_integrations, $limit = 20 ) {
    if( empty( $discussion_integrations ) )
      return [];

    $sql  = 'SELECT inner_query.user_id,
                    GROUP_CONCAT( CONCAT( inner_query.id, "=", user_last_activity ) SEPARATOR ",") as discussion_activity_map
               FROM (' . $this->_get_notification_data_inner_query( $discussion_integrations ). ') inner_query
               GROUP BY inner_query.user_id
               LIMIT ' . $limit;

    $notifications_list = $this->wpdb->get_results( $sql );

    if( empty( $notifications_list ) )
      return [];

    $response = [
      'user_id_map'   => [],
      'discussion_ids'=> [],
      'discussions'   => []
    ];

    foreach( $notifications_list as $notification_data ) {
      $discussions_activity_map = explode( ",", $notification_data->discussion_activity_map );

      foreach( $discussions_activity_map as $discussion_activity_map ) {
        $discussion_activity_map = explode( "=", $discussion_activity_map );

        $response['user_id_map'][ $notification_data->user_id ][ $discussion_activity_map[0] ] = $discussion_activity_map[1];
        $response['discussion_ids'][ $discussion_activity_map[0] ] = $discussion_activity_map[0];
      }
    }

    $response['discussions'] = wpep_utilities_map_object( $this->get_discussions( [ 'id' => [ 'IN' => $response['discussion_ids'] ] ] ), 'id' );

    return $response;
  }

  private function _get_notification_data_inner_query( $discussion_integrations ) {
    $notifications_interval = intval( wpep_get_setting( 'comments-email-notification-frequency' ) );

    $sql = '';
    $sql .=   'SELECT discussion.id,
                      discussion_dlv.user_id as user_id,
                      IF( discussion_dln.value > discussion_dlv.value, discussion_dln.value, discussion_dlv.value ) as user_last_activity
                 FROM `' . $this->table_discussion . '` discussion
            LEFT JOIN `' . $this->table_discussion_data . '` discussion_dlv 
                   ON discussion_dlv.discussion_id = discussion.id 
                  AND discussion_dlv.key = "last_view"
            LEFT JOIN `' . $this->table_discussion_data . '` discussion_dln 
                   ON discussion_dln.discussion_id = discussion.id 
                  AND discussion_dln.key = "last_notification"
                  AND discussion_dln.user_id = discussion_dlv.user_id
            LEFT JOIN `' . $this->table_discussion_data . '` discussion_dns 
                   ON discussion_dns.discussion_id = discussion.id 
                  AND discussion_dns.key = "notification_status"
                  AND discussion_dns.user_id = discussion_dlv.user_id
                WHERE IFNULL( discussion_dns.value, 1 ) != 0
                  AND from_unixtime( IF( discussion_dln.value > discussion_dlv.value, discussion_dln.value, discussion_dlv.value ) ) < discussion.last_activity
                  AND ( IF( discussion_dln.value > discussion_dlv.value, discussion_dln.value, discussion_dlv.value ) + ' . $notifications_interval . ' ) < ' . time() . '
                  AND (' . $this->_get_notification_data_inner_query_integration_in( $discussion_integrations ) . ')';

    return $sql;
  }

  private function _get_notification_data_inner_query_integration_in( $discussion_integrations ) {
    $sql = '';

    $i = 0; foreach( $discussion_integrations as $discussion_integration ) {
      $sql .= ( $i >= 1 ? ' OR ' : ' ' ) . 'discussion.namespace = "' . $discussion_integration->get_namespace() . '"';
      $i++;
    }

    return $sql;
  }

  public function get_comment_count_by_args( $namespace, $args = [] ) {
    $sql = 'SELECT COUNT(discussion_comment.id) FROM ' . $this->table_discussion_comment . ' discussion_comment
                        LEFT JOIN ' . $this->table_discussion . ' discussion ON discussion.id = discussion_comment.discussion_id
                            WHERE discussion.namespace = "' . $namespace . '"';

    if( isset( $args['time_from' ] ) )
      $sql .= ' AND discussion_comment.`created_at` >= "' . date( "Y-m-d H:i:s", $args['time_from'] ) . '"';

    if( isset( $args['time_to' ] ) )
      $sql .= ' AND discussion_comment.`created_at` < "' . date( "Y-m-d H:i:s", $args['time_to'] ) . '"';

    return $this->wpdb->get_var( $sql );
  }

}