<?php

class WPEPCronController {

  public static function setup() {
    add_filter( 'cron_schedules', [ 'WPEPCronController', "_register_cron_schedules"] );

    register_activation_hook(WPEP_BASE_FILE_PATH, [ 'WPEPCronController', "_add_cron_events"] );

    register_deactivation_hook(WPEP_BASE_FILE_PATH, [ 'WPEPCronController', "_remove_cron_events"] );
  }

  public static function is_job_registered( $func ) {
    return in_array( $func, self::get_job_list() );
  }

  public static function get_jobs_status() {
    $crons = _get_cron_array();

    if ( empty( $crons ) )
      return null;

    // The "Final" Rule is checking if all jobs are actually working.
    $jobs_with_errors = self::get_job_list();

    if( empty( $jobs_with_errors ) )
      return true;

    foreach ( $crons as $timestamp => $cron ) {
      foreach( $jobs_with_errors as $hook => $recurrence ) {
        if ( isset( $cron[ $hook ] ) ) {
          $first_array_element = $cron[ $hook ][ key( $cron[ $hook ] ) ];

          if( $first_array_element[ 'schedule' ] != $recurrence )
            continue;

          unset( $jobs_with_errors[ $hook ] );

          if( empty( $jobs_with_errors ) )
            return true;
        }
      }
    }

    return empty( $jobs_with_errors );
  }

  public static function is_job_list_executing() {
    $crons = _get_cron_array();

    if ( empty( $crons ) )
      return null;

    $due_jobs = self::get_job_list();

    if( empty( $due_jobs ) )
      return true;

    foreach ( $crons as $timestamp => $cron ) {
      foreach( $due_jobs as $hook => $recurrence ) {
        if ( !isset( $cron[ $hook ] ) )
          continue;

        if( $timestamp + 600 > time() )
          unset( $due_jobs[ $hook ] );
      }
    }

    return empty( $due_jobs );
  }

  public static function get_job_list_report() {
    $crons = _get_cron_array();

    if ( empty( $crons ) )
      return [];

    $job_list = self::get_job_list();

    if( empty( $job_list ) )
      return [];

    foreach( $job_list as $hook => $recurrence )
      $job_list[ $hook ] = 0;

    $cron_schedules = wp_get_schedules();

    foreach ( $crons as $timestamp => $cron ) {
      foreach( $job_list as $hook => $recurrence ) {
        if ( !isset( $cron[ $hook ] ) )
          continue;

        $first_array_element = $cron[ $hook ][ key( $cron[ $hook ] ) ];

        $job_list[ $hook ] = [
          'scheduled'     => $timestamp,
          'interval'      => ( isset( $first_array_element['interval'] ) ? $first_array_element['interval'] : false ),
          'schedule'      => $first_array_element['schedule'],
          'schedule_name' => (
              isset( $cron_schedules[ $first_array_element['schedule'] ] )
            ? $cron_schedules[ $first_array_element['schedule'] ][ 'display' ]
            : false
          )
        ];
      }
    }

    return $job_list;
  }

  public static function has_jobs() {
    $job_list = self::get_job_list();

    return !empty( $job_list );
  }

  public static function get_job_list() {
    $response = [];

    if( wpep_get_setting_flag( 'data-system-enable-background-backups' ) == true )
      $response['wpep_cron_daily'] = 'daily';

    if( wpep_get_setting_flag( 'email-weekly-recap-status' ) == true ) {
      $response['wpep_cron_five_minutes'] = 'five_minutes';
      $response['wpep_cron_weekly'] = 'weekly';
    }

    $response[ 'wpep_cron_discussion_email_notifications' ] = 'five_minutes';

    return apply_filters( "wpep_cron_job_list", $response );
  }

  public static function _get_cancelled_job_list() {
    $response = [ 'wpep_ie_vault_daily' ];

    if( wpep_get_setting_flag( 'data-system-enable-background-backups' ) == false )
      $response[] = 'wpep_cron_daily';

    if( wpep_get_setting_flag( 'email-weekly-recap-status' ) == false ) {
      $response[] = 'wpep_cron_five_minutes';
      $response[] = 'wpep_cron_weekly';
    }

    return $response;
  }

  public static function reset_cron_events() {
    self::_add_cron_events( false );
  }

  public static function _register_cron_schedules( $schedules ) {
    if( !isset( $schedules['five_minutes'] ) ) {
      $schedules['five_minutes'] = [
        'interval'  => 300,
        'display'   => __( 'Every Five Minute', 'wpep' )
      ];
    }

    if( !isset( $schedules['weekly'] ) ) {
      $schedules['weekly'] = [
        'interval'  => 604800,
        'display'   => __( 'Once Weekly', 'wpep' )
      ];
    }

    return $schedules;
  }

  public static function _add_cron_events( $override_existent = true ) {
    $current_time = time();

    $today_morning_time = strtotime( 'today 08:00', $current_time);
    $today_time         = strtotime( 'today 00:00', $current_time );
    $cron_schedules     = wp_get_schedules();

    foreach( self::get_job_list() as $hook => $recurrence ) {
      if( !$override_existent ) {
        $schedule_hook = wp_get_schedule( $hook );

        if( wp_get_schedule( $hook ) !== false ) {
          if( $recurrence === $schedule_hook )
            continue;

          if( !isset( $cron_schedules[$schedule_hook] ) )
            wp_clear_scheduled_hook( $hook );
        }
      }

      $event_time = $current_time;

      if( $recurrence == 'weekly' ) {
        if( date('D', $current_time ) === 'Mon' && $today_morning_time > $current_time) {
          $event_time = $today_morning_time;
        } else {
          $event_time = strtotime( 'next monday 08:00', $current_time );
        }
      } elseif( $recurrence == 'daily' ) {
        $event_time = ( $today_time > $current_time ? $today_time : strtotime( 'tomorrow 00:00', $current_time ) );
      }

      wp_schedule_event( $event_time, $recurrence, $hook );
    }
  }

  public static function _remove_cron_events() {
    foreach( self::get_job_list() as $hook => $recurrence )
      wp_clear_scheduled_hook( $hook );

    foreach( self::_get_cancelled_job_list() as $hook )
      wp_clear_scheduled_hook( $hook );
  }

}

WPEPCronController::setup();