<?php

use WPEP\Entity\Discussion as WPEP_Discussion;

abstract class WPEP_Discussion_Integration {

  public function __construct() {
    wpep_controller()->discussion->register_integration( $this );
  }

  public function get_discussion_name( $discussion ) {
    return isset( $discussion->post_id ) ? get_the_title( $discussion->post_id ) : $discussion->identifier;
  }

  public function get_discussion_link( $discussion, $user_id = false ) {
    if( isset( $discussion->post_id ) )
      return get_the_permalink( $discussion->post_id );

    return '#';
  }

  public function pre_administration_discussions_query() {
    return [];
  }

  abstract public function get_namespace();
  abstract public function current_user_can_view( $discussion_id );
  abstract public function current_user_can_post( $discussion_id );

  public function current_user_can_delete_comment( $discussion_comment ) {
    if( get_current_user_id() === 0 )
      return false;

    if( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return true;

    $discussion_comment = is_numeric( $discussion_comment ) ? WPEP_Discussion::instance()->get_discussion_comment( [ 'id' => absint( $discussion_comment ) ] ) : $discussion_comment;

    if( $discussion_comment->user_id == get_current_user_id() )
      return true;

    return false;
  }

  public function current_user_can_pin_comment( $discussion_comment ) {
    if( get_current_user_id() === 0 )
      return false;

    if( current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
      return true;

    return false;
  }

  public function get_email_notifications_status() {
    return false;
  }

  /**
   * @param $identifier_args
   * @param $post_id
   * @return int
   */
  public function discussion_id_by_identifier_args( $identifier_args, $post_id ) {
    $discussion_id = WPEP_Discussion::instance()->discussion_id_by_identifier( $this->get_discussion_identifier( $identifier_args ), $post_id, $this->get_namespace() );

    return $discussion_id;
  }

  public function get_discussion_identifier( $args ) {
    return 'post_comments_' . ( isset( $args['post_id'] ) ? $args['post_id'] : 0 );
  }

  /**
   * @param string $identifier
   * @return false|array
   */
  public function get_discussion_identifier_args_from_string( $identifier ) {
    if( strpos( $identifier, 'post_comments_' ) !== 0 )
      return false;

    return [
      'post_id' => intval( str_replace( 'post_comments_', '', $identifier ) )
    ];
  }

  public function has_comment_type( $type ) {
    if( $type === '' )
      return true;

    $comment_types = $this->get_comment_types();

    return isset( $comment_types[ $type ] );
  }

  public function get_comment_types() {
    return [];
  }

  public function get_comment_form_editor_args() {
    return [
      'quicktags'     => [],
      'media_buttons' => false,
      'tinymce'       => true,
      'default_editor'=> 'tinymce',
      'editor_height' => 100,
    ];
  }

  /**
   * @param bool $discussion_id
   * @param bool $list_only
   * @param string $comments_structure
   * @return string
   */
  public function get_comments_template( $discussion_id = false, $list_only = false, $comments_structure = 'threaded' ) {
    ob_start();

    $this->load_comments_template( $discussion_id, $list_only, $comments_structure );

    $ret = ob_get_contents();

    ob_end_clean();

    return $ret;
  }

  /**
   * @param int $discussion_id
   * @param bool $list_mode
   * @param string $comments_structure
   */
  public function load_comments_template( $discussion_id, $list_mode = false, $comments_structure = 'threaded' ) {
    do_action( "wpep_discussion_comments_load_template_before", $this, $list_mode, $comments_structure );

    if( $list_mode ) {
      echo wpep_discussion_comments_template( $discussion_id, $this->get_current_page(), $this->_discussion_comments_args( $comments_structure ), $this );

      WPEP\Entity\Discussion::instance()->set_discussion_user_data( WPEP_DISCUSSION_USER_DATA_LAST_VIEW, time(), $discussion_id, get_current_user_id() );
    } else {
      WPEP\Controller::instance()->template->load_template( 'discussion/template.php', [
        'discussion_namespace'     => $this->get_namespace(),
        'discussion_id'            => $discussion_id,
        'discussion_search_term'   => $this->get_search_term(),
        'discussion_filter_by_user'=> $this->get_filter_by_user_id()
      ] );
    }

    do_action( "wpep_discussion_comments_load_template_after", $this, $list_mode, $comments_structure );
  }

  private function _discussion_comments_args( $comments_structure ) {
    $comment_args = [
      'show_pagination' => true,
      'search'          => $this->get_search_term(),
      'filter_by_user'  => $this->get_filter_by_user_id(),
      'structure'       => $comments_structure,
      'type'            => $this->get_comment_type(),
      'parent__in'      => $this->get_comment_parent__in(),
      'date_query'      => $this->get_comment_date_query()
    ];

    if( isset( $_POST[ 'wpep_post_comments_sync' ] ) && $_POST[ 'wpep_post_comments_sync' ] )
      $comment_args[ 'structure' ] = false;

    return $comment_args;
  }

  public function get_current_page() {
    if( isset( $_POST[ 'wpep_post_comments_page' ] ) )
      return intval( $_POST[ 'wpep_post_comments_page' ] );

    if( isset( $_GET[ 'wpep-post-comments-page' ] ) )
      return intval( $_GET[ 'wpep-post-comments-page' ] );

    return 1;
  }

  public function get_search_term() {
    if( isset( $_POST[ 'wpep_post_comments_search'] ) )
      return sanitize_text_field( $_POST[ 'wpep_post_comments_search'] );

    return '';
  }

  public function get_filter_by_user_id() {
    if( isset( $_POST[ 'wpep_post_comments_filter_by_user_id' ] ) ) {
      if( !current_user_can( WPEP_CAP_MANAGE_MEMBERS ) )
        return get_current_user_id();

      $user_id = absint( $_POST[ 'wpep_post_comments_filter_by_user_id' ] );

      return ( $user_id === 0 ? false : $user_id );
    }

    return false;
  }

  public function get_comment_type() {
    $comment_type = '';

    if( isset( $_POST[ 'wpep_post_comments_type'] ) )
      $comment_type = sanitize_text_field( $_POST[ 'wpep_post_comments_type'] );
    else if( isset( $_GET[ 'wpep-post-comments-type'] ) )
      $comment_type = sanitize_text_field( $_GET[ 'wpep-post-comments-type'] );

    return $this->has_comment_type( $comment_type ) ? $comment_type : '';
  }

  public function get_comment_parent__in() {
    $parent_ids = [];

    if( isset( $_POST[ 'wpep_post_comments_parent_ids'] ) )
      $parent_ids = $_POST[ 'wpep_post_comments_parent_ids'];
    else if( isset( $_GET[ 'wpep-post-comments-parent-ids'] ) )
      $parent_ids = $_POST[ 'wpep-post-comments-parent-ids'];

    if( empty( $parent_ids ) )
      return '';

    $parent_ids = is_string( $parent_ids ) ? explode( ",", $parent_ids ) : $parent_ids;

    foreach( $parent_ids as $parent_id_key => $parent_id )
      $parent_ids[ $parent_id_key ] = intval( $parent_id );

    return empty( $parent_ids ) ? '' : $parent_ids;
  }

  public function get_comment_date_query() {
    $time_from  = 0;
    $time_to    = 0;

    if( isset( $_POST[ 'wpep_post_comments_time_from'] ) )
      $time_from = intval( $_POST[ 'wpep_post_comments_time_from'] );
    else if( isset( $_GET[ 'wpep-post-comments-time-from' ] ) )
      $time_from = intval( $_GET[ 'wpep-post-comments-time-from' ] );

    if( isset( $_POST[ 'wpep_post_comments_time_to'] ) )
      $time_to = intval( $_POST[ 'wpep_post_comments_time_to'] );
    else if( isset( $_GET[ 'wpep-post-comments-time-to' ] ) )
      $time_to = intval( $_GET[ 'wpep-post-comments-time-to' ] );

    if( $time_from === 0 && $time_to === 0 )
      return null;

    $response = [];

    if( $time_from !== 0 )
      $response[ '>' ] = date( 'Y-m-d H:i:s', $time_from );

    if( $time_to !== 0 )
      $response[ '<' ] = date( 'Y-m-d H:i:s', $time_to );

    return $response;
  }

  public function get_discussion_users( $discussion_id, $args = [] ) {
    $data = WPEP\Entity\Discussion::instance()->get_discussion_user_ids( $discussion_id, $args );

    return [
      'total'   => $data['total'],
      'entries' => ( empty( $data['ids'] ) ? [] : get_users( [ 'include' => $data['ids'] ] ) )
    ];
  }

  public function get_discussions_by_user_id( $user_id, $args = [] ) {
    $data = WPEP\Entity\Discussion::instance()->get_discussions_by_user_id( $user_id, $args );

    return [
      'total'   => $data['total'],
      'entries' => ( empty( $data['ids'] ) ? [] : WPEP\Entity\Discussion::instance()->get_discussions( [ 'id' => [ 'IN' => $data['ids'] ] ] ) )
    ];
  }

}